package molmaster;

/** Base class for all listening events.  To avoid the
 *  annoyances of casting everywhere, some functionality
 *  that might be better suited to descendant classes has
 *  been collapsed here.
 *
 * @author  Russell
 */
public class Event implements java.io.Serializable{
    static final long serialVersionUID = 10275539472837495L;
    protected EventID eventID;
    protected Object eventData;
    protected Object additional;
    
    
    /** Creates a new event with the given integer id and
     *  value object.  Note that the Event class simply
     *  stores this id - translating id's back to a meaningful
     *  statement must handled by the classes using the event.
     */
    public Event(EventID eventID, Object eventData ) {
        this.eventID = eventID;
        this.eventData = eventData;
    }
    
    public Event( EventID eventID ) {
        this.eventID = eventID;
        this.eventData = null;
    }
    
    public Event(EventID eventID, Object eventData, Object additionalData ){
        this( eventID, eventData );
        additional = additionalData;
    }
    
    public EventID getID() { return eventID; }
    public Object getData() { return eventData; }
    public String toString() { 
        String out = eventID.toString();
        out += "\n\n" + eventData.toString();
        return out;
    }
    
    public Object getAdditionalData() { return additional; }
}
