package molmaster;

import java.util.*;

/** An EventID uniquely identifies a type of event within molmaster.  A
 *  particular event type has an associated EventID.
 *
 * @author  Russell Power
 */
public class EventID implements java.io.Serializable{
    
    static private final long serialVersionUID = 10275539472837495L;
    protected String description = "";
    protected int hashCode;
    
    /** An empty event id - this can be used in the case that no particular
     *  action has occurred, but a watched object wants to trigger an update.
     */    
    static public EventID blankID = new EventID("");
    static public Vector idMap = new Vector();
    
    /** Creates a new instance of EventID */
    public EventID(String description) {
        this.description = description;
        if ( idMap == null ) { idMap = new Vector(); }
        if ( !description.equals("")  ){
            idMap.add( this );
        }
        
        //  don't need a hashcode for the blank id...
        if ( description.equals("") ){ return; }
        
        //  a hacked up hash, but it should work okay...
        int length = description.length();
        if ( length > 3 ){
            hashCode = (description.charAt(0)<<24) + (description.charAt(1)<<16) + (description.charAt(length-2) << 8) + (description.charAt(length-1));
        }
        else {
            hashCode = description.toCharArray()[0];
        }
    }
    
    public EventID(Class owner, String description) {
        this( owner.getName() + " - " + description );
    }
    
    /** Returns a String description of this event.
     */    
    public String getDescription() {
        return description;
    }
    
    public String toString() {
        return getDescription();
    }
    
    /** We serialize this class, so we can't use the default
     *  hashcode and equals methods, as they operate by the 
     *  objects memory address. */
    public int hashCode() {
        return hashCode;
    }
    
    /** Returns whether the given Object is of type EventID and equivalent to the this
     * object.
     */    
    public boolean equals( Object o ) throws ClassCastException {
        if ( o == null ) { return false; }
        return ((EventID)o).description.equals( description );
    }
}
        
