package molmaster;

import java.io.*;
/** Implements some basic logging functionality...
 *
 * @author  rjpower
 */
public class Log {
    
    public static final int LOG_DEBUG =     0x00;
    public static final int LOG_INFO =      0x01;
    public static final int LOG_WARNING =   0x02;
    public static final int LOG_ERROR =     0x03;
    public static final int LOG_NUKE =      0x04;
    
    public static int logLevel = 0x01;
    
    protected static final String[] logLevels = new String[]{ "DEBUG", "INFO", "WARNING", "**ERROR**", "****----NUKE----****"};
    protected static PrintStream out = System.out;
    
    /** Returns an output stream suitable for logging.
     */    
    public static PrintStream getLogStream() { return out; }
    public static final void init() {
		PrintStream dStream = new DialogStream();
        System.setErr( dStream );
        String s = System.getProperty( "loglevel" );
        if ( s != null ) {
            try {
                logLevel = Integer.parseInt( s );
            }catch(Exception e){}
        }
    }
    
    public static void info( Object val ) {
    	writeLog( Log.LOG_INFO, val );
    }
    
    public static void debug( Object val ) {
    	writeLog( Log.LOG_DEBUG, val);
    }
    
    public static void error( Object val ) {
    	writeLog( Log.LOG_ERROR, val );
    }
    
    public static void warn( Object val ) {
    	writeLog( Log.LOG_WARNING, val );
    }
    
    /** Write the given Object to the log.  The string representation
     * of the object is written to the log file, along with information
     * regarding the current executing thread.  If the log level is below
     * the current logging threshold, the message is ignored.
     */    
    private static void writeLog( int priority, Object val ) {
        if ( priority >= logLevel ) {
            if ( val == null ) 
                val = "(null)";
            
            Exception e = new Exception();
            StackTraceElement[] st = e.getStackTrace();
           	String method = st[2].getFileName() + "(" + st[2].getMethodName() + ":" + st[2].getLineNumber() + ")";
            
            out.println( 
            	logLevels[priority] + " " + method + ":\t" + val.toString() );
        }
    }
    
    /** Write the given Object to the log using the default log
     * level. The string representation of the object is written
     * to the log file, along with information
     * regarding the current executing thread.  If the log level is below
     * the current logging threshold, the message is ignored.
     */    
    public static void writeLog( Object val ) {
        writeLog( Log.LOG_INFO, val );
    }
}

/** Implements a text stream that catches attempts to print
 * objects of type Throwable - this occurs when swing or
 * awt finds an unhandled exception.
 *
 * In response to this exception or error, a dialog is shown
 * prompting the user that a critical error has occured.
 */
class DialogStream extends PrintStream {
    /** Creates a new DialogStream.
     */    
    DialogStream() {
        super( System.out );
    }
 
    /** Overridden to check for attempts to print objects of type
     * Throwable.
     */    
    public void print( Object o ) {
        if ( o instanceof Throwable ) {
            if ( !molmaster.gui.tools.ErrorDialog.isInUse() ) {
                new molmaster.gui.tools.ErrorDialog( (Throwable)o, "Unexpected Application Error", false ).show();
            }
        }
        super.print( o );
    }
}