package molmaster;

import molmaster.model.*;

/** Represents the base class for any objects
 *  that  are to exist in the molmaster universe.
 *
 * @author  rjpower
 */
public class MolObject {
    protected MoleculeFile owner;
    protected MolAttributes attributes = new MolAttributes();
    private Object userObject;
    
    /** Creates a new MolObject with default parameters.
     */    
    protected MolObject() {
        uniqueID = MolModel.getModel().getUniqueID();
    }
    
    /** Sets the owner of this MolObject to be the given MoleculeFile.
     */    
    public void setOwner(MoleculeFile mf ) { owner = mf; }
    /** Returns the current owner of this MolObject.
     */    
    public MoleculeFile getOwner(){ return owner; }
    
    /** Returns the attributes for this object.
     */    
    public MolAttributes getAttributes(){
        return attributes;
    }
    
    /** Performs an equality comparison based on the unique id of
     * the input object that this->getUniqueID().
     */    
    public boolean equals(Object o) { return ((MolObject)o).uniqueID == uniqueID; }
    
    protected int uniqueID;
    /** Returns a unique identifier for this object.  This identifier will
     * not be assigned to any other MolObject for the duration of the program.
     */    
    public int getUniqueID() { return uniqueID; }
    /** Returns the unique identifier for this object.
     */    
    public int hashCode() { return uniqueID; }
    
    /** Returns the user data currently assigned to this object.
     */    
    public Object getUserObject() { return userObject; }
    /** Sets the user data of this object to the input.
     */    
    public void setUserObject( Object o ){ userObject = o; }
}
