package molmaster.gui;

import java.awt.*;
import java.awt.event.*;
import java.io.File;
import java.util.*;

import javax.swing.*;
import javax.swing.filechooser.*;
import javax.vecmath.*;
import javax.media.j3d.*;

import molmaster.*;
import molmaster.util.*;
import molmaster.model.*;
import molmaster.gui.help.*;
import molmaster.gui.input.*;

/** The main window for molmaster.  Contains the rendering
 *  canvas, as well as the tool and status bars.  Features
 *  some convenience methods for easier access to status
 *  and control bar information.
 * 
 * Yes - I am aware of the disgusting amount of garbage that has all been piled
 * into this single file.  I'm working on cleaning it up... I just haven't
 * gotten to it yet :p.
 *
 * @author  rjpower
 */
public class MainFrame extends javax.swing.JFrame {
	static public MolUniverse uni;
	static public ImageCaptureCanvas3D canvas;
	static public TransformGroup root;
	static public BranchGroup branchRoot;
	static protected MainFrame mainWindow;

	public SceneControl scene;
	public OrbitBehavior mouseControl;

	public static String currentAtom = "Hydrogen";

	static public MainFrame getActiveWindow() {
		return mainWindow;
	}

	/** Creates new form MainFrame */
	public MainFrame() {
		initComponents();

		mainWindow = this;
		progress.setMaximum(100);
		progress.setMinimum(0);
		statusMessage.setText("Ready.");

		editBar.setOrientation(molmaster.gui.editbar.EditBar.VERTICAL);
	}

	protected void menuAtomTableClicked(java.awt.event.ActionEvent evt) {
		new molmaster.gui.tools.AtomEditor(this, true).show();
	}

	/** Initializes the main rendering canvas and the viewer for the scene.
	 */
	public void initSpace() {
		branchRoot = new BranchGroup();
		root = new TransformGroup();

		branchRoot.setCapability(BranchGroup.ALLOW_CHILDREN_EXTEND);
		branchRoot.setCapability(BranchGroup.ALLOW_CHILDREN_WRITE);
		branchRoot.setCapability(BranchGroup.ALLOW_DETACH);

		branchRoot.clearCapabilityIsFrequent(BranchGroup.ALLOW_CHILDREN_EXTEND);
		branchRoot.clearCapabilityIsFrequent(BranchGroup.ALLOW_CHILDREN_WRITE);
		branchRoot.clearCapabilityIsFrequent(BranchGroup.ALLOW_DETACH);

		root.setCapability(TransformGroup.ALLOW_CHILDREN_EXTEND);
		root.setCapability(TransformGroup.ALLOW_CHILDREN_WRITE);
		root.setCapability(TransformGroup.ALLOW_BOUNDS_READ);

		root.clearCapabilityIsFrequent(TransformGroup.ALLOW_BOUNDS_READ);
		root.clearCapabilityIsFrequent(TransformGroup.ALLOW_CHILDREN_EXTEND);
		root.clearCapabilityIsFrequent(TransformGroup.ALLOW_CHILDREN_WRITE);

		branchRoot.addChild(root);
		GraphicsConfigTemplate3D template = new GraphicsConfigTemplate3D();
		template.setDoubleBuffer(GraphicsConfigTemplate3D.REQUIRED);
		template.setSceneAntialiasing(GraphicsConfigTemplate3D.PREFERRED);

		GraphicsConfiguration config =
			GraphicsEnvironment
				.getLocalGraphicsEnvironment()
				.getDefaultScreenDevice()
				.getBestConfiguration(template);

		canvas = new ImageCaptureCanvas3D(config);
		//canvas.setIgnoreRepaint( true );
		canvas.setDoubleBufferEnable(true);
		getContentPane().add(canvas, BorderLayout.CENTER);

		Bounds bounds =
			new BoundingSphere(new Point3d(), Double.POSITIVE_INFINITY);

		uni = new MolUniverse(canvas);
		uni.getViewingPlatform().setNominalViewingTransform();
		TransformGroup viewTrans =
			uni.getViewingPlatform().getViewPlatformTransform();
		Transform3D trans = new Transform3D();
		viewTrans.getTransform(trans);
		Transform3D moveBack = new Transform3D();
		moveBack.setTranslation(new Vector3f(0, 0, 100));
		trans.mul(moveBack);
		viewTrans.setTransform(trans);

		//KeyHandler k = new KeyHandler( canvas, uni.getViewingPlatform().getViewPlatformTransform() );
		//root.addChild( k );

		BranchGroup behaviorGroup = new BranchGroup();

		OrbitBehavior b =
			new OrbitBehavior(canvas, OrbitBehavior.PROPORTIONAL_ZOOM);
		b.setSchedulingBounds(bounds);
		b.setReverseRotate(true);
		b.setReverseTranslate(true);
		b.setTransFactors(2.0, 2.0);

		uni.getViewingPlatform().setViewPlatformBehavior(b);

		mouseControl = b;
		MouseHandler m = new MouseHandler(canvas, branchRoot);
		m.addHandler(new MouseSelect(canvas, branchRoot, uni));
		root.addChild(m);

		uni.getViewer().getView().setBackClipDistance(500);

		Background back = new Background();
		back.setColor(0.9f, 0.9f, 0.9f);
		back.setApplicationBounds(bounds);
		branchRoot.addChild(back);

		branchRoot.compile();
		uni.attach(branchRoot);

		setSize(400, 400);
	}

	private void initComponents() {

		//		GEN-BEGIN:initComponents
		editBar = new molmaster.gui.editbar.EditBar();
		statusBar = new javax.swing.JToolBar();
		progress = new javax.swing.JProgressBar();
		statusMessage = new javax.swing.JLabel();
		menuBar = new javax.swing.JMenuBar();
		JPopupMenu.setDefaultLightWeightPopupEnabled(false);
		ToolTipManager.sharedInstance().setLightWeightPopupEnabled(false);
		menuFile = new javax.swing.JMenu();
		menuItemOpen = new javax.swing.JMenuItem();
		menuItemSave = new javax.swing.JMenuItem();
		menuItemSaveImage = new javax.swing.JMenuItem();
		menuItemQuit = new javax.swing.JMenuItem();
		menuTools = new javax.swing.JMenu();
		menuItemOptions = new javax.swing.JMenuItem();
		menuView = new javax.swing.JMenu();
		menuToolPanel = new javax.swing.JMenuItem();

		setTitle("molmaster");
		addWindowListener(new java.awt.event.WindowAdapter() {
			public void windowClosing(java.awt.event.WindowEvent evt) {
				exitForm(evt);
			}
		});

		editBar.setToolTipText("Select the current tool for use.");
		getContentPane().add(editBar, java.awt.BorderLayout.EAST);

		progress.setBackground(new java.awt.Color(255, 255, 255));
		progress.setForeground(new java.awt.Color(51, 51, 255));
		progress.setToolTipText(
			"Percentage of the current operation that has been completed");
		progress.setDoubleBuffered(true);
		progress.setMinimumSize(new java.awt.Dimension(50, 14));
		progress.setPreferredSize(new java.awt.Dimension(100, 14));
		progress.setStringPainted(true);
		statusBar.add(progress);

		statusMessage.setMaximumSize(new java.awt.Dimension(32767, 32767));
		statusMessage.setMinimumSize(new java.awt.Dimension(100, 10));
		statusMessage.setPreferredSize(new java.awt.Dimension(400, 10));
		statusBar.add(statusMessage);

		getContentPane().add(statusBar, java.awt.BorderLayout.SOUTH);

		menuBar.setDoubleBuffered(true);
		menuBar.setFont(new java.awt.Font("Dialog", 0, 11));
		menuFile.setForeground(new java.awt.Color(0, 0, 0));
		menuFile.setMnemonic('f');
		menuFile.setText("File");
		menuItemOpen.setAccelerator(
			javax.swing.KeyStroke.getKeyStroke(
				java.awt.event.KeyEvent.VK_O,
				java.awt.event.InputEvent.CTRL_MASK));
		menuItemOpen.setMnemonic('o');
		menuItemOpen.setText("Open");
		menuItemOpen.setToolTipText("Load a file from disk.");
		menuItemOpen.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				menuItemOpenActionPerformed(evt);
			}
		});

		menuFile.add(menuItemOpen);
		menuItemSave.setAccelerator(
			javax.swing.KeyStroke.getKeyStroke(
				java.awt.event.KeyEvent.VK_A,
				java.awt.event.InputEvent.CTRL_MASK));
		menuItemSave.setMnemonic('a');
		menuItemSave.setText("Save As");
		menuItemSave.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				menuItemSaveActionPerformed(evt);
			}
		});

		menuFile.add(menuItemSave);
		menuItemSaveImage.setAccelerator(
			javax.swing.KeyStroke.getKeyStroke(
				java.awt.event.KeyEvent.VK_I,
				java.awt.event.InputEvent.CTRL_MASK));
		menuItemSaveImage.setMnemonic('i');
		menuItemSaveImage.setText("Save Image");
		menuItemSaveImage
			.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				menuItemSaveImageActionPerformed(evt);
			}
		});

		menuFile.add(menuItemSaveImage);
		menuItemQuit.setAccelerator(
			javax.swing.KeyStroke.getKeyStroke(
				java.awt.event.KeyEvent.VK_X,
				java.awt.event.InputEvent.CTRL_MASK));
		menuItemQuit.setMnemonic('x');
		menuItemQuit.setText("Exit");
		menuItemQuit.setToolTipText("Exit MolMaster.");
		menuItemQuit.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				menuItemQuitActionPerformed(evt);
			}
		});

		menuFile.add(menuItemQuit);
		menuBar.add(menuFile);
		menuTools.setMnemonic('t');
		menuTools.setText("Tools");
		menuTools.setToolTipText("Tools and options for customizing MolMaster");
		menuItemOptions.setMnemonic('o');
		menuItemOptions.setText("Options");
		menuItemOptions.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				menuItemOptionsActionPerformed(evt);
			}
		});

		menuTools.add(menuItemOptions);
		menuBar.add(menuTools);
		menuView.setMnemonic('V');
		menuView.setText("View");
		menuToolPanel.setMnemonic('T');
		menuToolPanel.setText("Tool Panel");
		menuToolPanel.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				menuToolPanelActionPerformed(evt);
			}
		});

		menuView.add(menuToolPanel);
		menuBar.add(menuView);
		setJMenuBar(menuBar);

		pack();
		JMenuItem menuAtomTable = new JMenuItem("Edit Atom Table", 'e');
		menuAtomTable.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				menuAtomTableClicked(evt);
			}
		});
		menuView.add(menuAtomTable);

		JMenuItem menuScriptEditor = new JMenuItem("Script Editor", 's');
		menuScriptEditor
			.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				new molmaster.gui.script.ScriptDialog(
					MainFrame.this,
					false).setVisible(
					true);
			}
		});
		menuView.add(menuScriptEditor);

		JMenu menuHelp = new JMenu("Help");
		menuBar.add(menuHelp);
		JMenuItem menuItemAbout = new JMenuItem("About", 'a');

		menuItemAbout.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				new molmaster
					.gui
					.help
					.AboutDialog(getActiveWindow(), true)
					.show();
			}
		});
		menuHelp.add(menuItemAbout);

		JMenuItem menuItemPlugins = new JMenuItem("Plugin Configuration", 'p');
		menuItemPlugins.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				new molmaster.gui.editbar.PluginDialog().show();
			}
		});

		menuTools.add(menuItemPlugins);

		JMenuItem menuItemContents = new JMenuItem("Contents");
		menuItemContents.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				new HelpDialog(MainFrame.this, false).show();
			}
		});

		menuHelp.add(menuItemContents);

		JMenuItem menuItemReport = new JMenuItem("Report an error");
		menuItemReport.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				new molmaster
					.gui
					.tools
					.ErrorDialog(new Exception("User error report."))
					.show();
			}
		});
		menuHelp.add(menuItemReport);

	} //GEN-END:initComponents

	private void menuToolPanelActionPerformed(java.awt.event.ActionEvent evt) {
		//GEN-FIRST:event_menuToolPanelActionPerformed
		molmaster.gui.tools.ToolFrame.toolFrame.setVisible(true);
	} //GEN-LAST:event_menuToolPanelActionPerformed

	private void menuItemSaveImageActionPerformed(
		java.awt.event.ActionEvent evt) {
		//GEN-FIRST:event_menuItemSaveImageActionPerformed
		JFileChooser fc = new JFileChooser(new java.io.File("."));
		FileFilter ff = new FileFilter() {
			public boolean accept(File f) {
				if ( f.getName().toLowerCase().endsWith(".png")) {
					return true;
				}
				return false;
			}

			public String getDescription() {
				return "PNG Image (*.png)";
			}
		};
		fc.setFileFilter(ff);
		fc.showSaveDialog(this);

		java.io.File f = fc.getSelectedFile();
		if (f == null) {
			return;
		}
		try {
			MainFrame.setStatusMessage("Saving image...");
			ImageCapture ic = new ImageCapture(f);
			ic.wait();
		} catch (Exception e) {
		}

		System.setProperty("user.dir", fc.getCurrentDirectory().toString());
	} //GEN-LAST:event_menuItemSaveImageActionPerformed

	private void menuItemSaveActionPerformed(java.awt.event.ActionEvent evt) {
		//GEN-FIRST:event_menuItemSaveActionPerformed
		JFileChooser fc = new JFileChooser(new java.io.File("."));
		fc.showSaveDialog(this);

		java.io.File f = fc.getSelectedFile();
		if (f == null) {
			return;
		}

		String name = f.getName();
		try {
			MoleculeFile mf =
				MoleculeFile.getLoader(name.substring(name.length() - 3));
			mf.saveToFile(f.getAbsolutePath());
		} catch (Exception e) {
			JOptionPane.showMessageDialog(
				this,
				"File save was unsuccessful - error follows: \n\n " + e,
				"File Save Failure",
				JOptionPane.ERROR_MESSAGE);
			e.printStackTrace(Log.getLogStream());
		}

		System.setProperty("user.dir", fc.getCurrentDirectory().toString());
	} //GEN-LAST:event_menuItemSaveActionPerformed

	private void menuItemQuitActionPerformed(java.awt.event.ActionEvent evt) {
		//GEN-FIRST:event_menuItemQuitActionPerformed
		System.exit(0);
	} //GEN-LAST:event_menuItemQuitActionPerformed

	private void menuItemOptionsActionPerformed(
		java.awt.event.ActionEvent evt) {
		//GEN-FIRST:event_menuItemOptionsActionPerformed
		molmaster.gui.tools.OptionWindow.getWindow().setVisible(true);
	} //GEN-LAST:event_menuItemOptionsActionPerformed

	private void menuItemOpenActionPerformed(java.awt.event.ActionEvent evt) {
		//GEN-FIRST:event_menuItemOpenActionPerformed
		JFileChooser fc = new JFileChooser(new java.io.File("."));
		fc.showOpenDialog(this);

		java.io.File f = fc.getSelectedFile();

		if (f == null) {
			return;
		}

		String n = f.getName();
		String suffix = n.substring(n.length() - 3);
		//Thread.currentThread().setPriority( Thread.MAX_PRIORITY );
		try {
			MoleculeFile mf = MoleculeFile.getLoader(suffix);
			LoaderThread t = new LoaderThread(mf, f.getAbsolutePath());
			new Thread(t).start();
		} catch (Exception e) {
			JOptionPane.showMessageDialog(
				this,
				e.getMessage(),
				"File Load Error",
				JOptionPane.ERROR_MESSAGE);
			//e.printStackTrace();
		}
		System.setProperty("user.dir", fc.getCurrentDirectory().toString());
		//Thread.currentThread().setPriority( Thread.NORM_PRIORITY );

	} //GEN-LAST:event_menuItemOpenActionPerformed

	/** Exit the Application */
	private void exitForm(
		java.awt.event.WindowEvent evt) { //GEN-FIRST:event_exitForm
		System.exit(0);
	} //GEN-LAST:event_exitForm

	/** Sets the progress meter on the active main window the the given value.  Input
	 * should range from 0 to 100.
	 */
	public static void setProgressMeter(int value) {
		if (value >= 100) {
			value = 0;
		}
		if ( value == 0 ) 
			mainWindow.progress.setEnabled(false);
		else {
			if (!mainWindow.progress.isEnabled())
			mainWindow.progress.setEnabled(true);	
		}
		mainWindow.progress.setValue(value);
	}

	static Stack messages = new Stack();
	/** Temporarily changes the status message of the main display.
	 */
	public static void setStatusMessage(String m) {
		mainWindow.statusMessage.setText(m);
	}

	/** Sets the status message of the main display.  This message will be recalled later
	 * if another message is set and then removed.  Be sure to remove it when you are
	 * finished displaying.
	 */

	// Variables declaration - do not modify//GEN-BEGIN:variables
	private javax.swing.JMenu menuTools;
	private javax.swing.JMenuItem menuItemOpen;
	private javax.swing.JMenu menuFile;
	private javax.swing.JLabel statusMessage;
	private javax.swing.JMenuBar menuBar;
	private javax.swing.JMenuItem menuItemSave;
	private javax.swing.JProgressBar progress;
	private javax.swing.JMenuItem menuToolPanel;
	private javax.swing.JMenuItem menuItemOptions;
	private javax.swing.JMenuItem menuItemSaveImage;
	private molmaster.gui.editbar.EditBar editBar;
	private javax.swing.JMenu menuView;
	private javax.swing.JToolBar statusBar;
	private javax.swing.JMenuItem menuItemQuit;
	// End of variables declaration//GEN-END:variables

	/**
	 * Returns the canvas.
	 * @return ImageCaptureCanvas3D
	 */
	public static ImageCaptureCanvas3D getCanvas() {
		return canvas;
	}

}

class LoaderThread implements Runnable {

	MoleculeFile myFile;
	String fileName;
	public LoaderThread(MoleculeFile f, String fileName) {
		this.myFile = f;
		this.fileName = fileName;
	}
	public void run() {
		Thread.currentThread().setPriority(Thread.MIN_PRIORITY);
		try {
			myFile.loadFromFile(fileName);
			myFile.addToScene(MolModel.getModel());
		} catch (Exception e) {
			Log.info("Couldn't load from file.");
			e.printStackTrace(Log.getLogStream());
			new molmaster.gui.tools.ErrorDialog(e).show();
		}
		MainFrame.setStatusMessage("Ready.");
		MainFrame.setProgressMeter(0);
	}

}