package molmaster.gui;

import molmaster.*;

import javax.media.j3d.*;
import javax.vecmath.*;

/** This class provides access to commonly used 
 *  properties of the world viewer - this includes
 *  the capability to set and get the position, 
 *  direction and scaling of the view.
 */
public class MolViewingPlatform {
    
    protected TransformGroup tg = new TransformGroup();
    protected Transform3D trans = new Transform3D();
    protected Matrix4f mat = new Matrix4f();
    protected Matrix3f rotScaleMat = new Matrix3f();
    protected Vector3f pos = new Vector3f();
    
    protected MolUniverse uni;
    
    public MolViewingPlatform( MolUniverse uni ) {
        this.uni = uni;
    }
    
    public TransformGroup getViewPlatformTransform() {
        return uni.getViewingPlatform().getViewPlatformTransform();
    }
    
    /** Returns the current position of the view in space.  */
    public Vector3f getPosition(){
        tg = getViewPlatformTransform();
        tg.getTransform( trans );
        trans.get( pos );
        return pos;
    }
    /** Sets the current position of the view in space.  */
    public void setPosition( Vector3f p ){
        Log.writeLog( "Position set to: " + p );
        tg = getViewPlatformTransform();
        tg.getTransform( trans );
        //trans.get( rotScaleMat );
        trans.setTranslation( p );
        //trans.setRotation ( rotScaleMat );
        tg.setTransform( trans );
    }
    
    public AxisAngle4f getDirection() {
        tg = getViewPlatformTransform();
        tg.getTransform( trans );
        Quat4f dir = new Quat4f();
        trans.get( dir );
        AxisAngle4f a = new AxisAngle4f();
        a.set( dir );
        return a;
    }
    
    public void setDirection( AxisAngle4f dir ){
        Log.writeLog( "Direction set to: " + dir );
        tg = getViewPlatformTransform();
        tg.getTransform( trans );
        trans.setRotation( dir );
        tg.setTransform( trans );
    }
    
    public boolean getOrthogonalTransform() { 
        View v = MainFrame.uni.getViewer().getView();
        return ( v.getProjectionPolicy() == View.PARALLEL_PROJECTION );
    }
    
    private Point3d tempLeft = new Point3d();
    private final float SCALE_CONSTANT = 0.396f;
    
    /** Turns on or off the orthogonal( parallel ) view transformation.
     */    
    public void setOrthogonalTransform( boolean b ) {
        View v = MainFrame.uni.getViewer().getView();
        if ( b ) {
            v.setProjectionPolicy( View.PARALLEL_PROJECTION );
            v.setScreenScalePolicy( View.SCALE_EXPLICIT );
            Log.writeLog( "Distance to origin: " + getPosition().length() );
            v.setScreenScale( SCALE_CONSTANT / getPosition().length() );
            MainFrame.getActiveWindow().mouseControl.setScaling( true );
        }
        else {
            v.setProjectionPolicy( View.PERSPECTIVE_PROJECTION );
            v.setScreenScalePolicy( View.SCALE_SCREEN_SIZE );
            v.setScreenScale( 1.0 );
            MainFrame.getActiveWindow().mouseControl.setScaling( false );
        }
    }
    
    /** Sets the factor by which to scale the screen image.
     */    
    public void setScreenScale( double scale ) {
        View v = MainFrame.uni.getViewer().getView();
        v.setScreenScale( scale );
        Log.info( "Scale factor := " + scale );
    }
    public double getScreenScale () {
        View v = MainFrame.uni.getViewer().getView();
        return v.getScreenScale();
    }
    
    public boolean getResetView() { return false; }
    /** Resets the view to the default location and direction.
     */    
    public void setResetView(boolean b) {
       tg = getViewPlatformTransform();
       trans.setTranslation( new Vector3f( 0, 0, 110.0f ) );
       trans.setRotation( new Quat4f( 0, 0, 1, 0 ) );
       tg.setTransform( trans );
    }
}