package molmaster.gui;

import molmaster.util.*;

/** Performs cleanup and saves state data for use
 *  when the program is run again.
 *
 * @author  Russell Power
 */
public class ShutdownHook implements Runnable {
    
    private static java.util.List listeners = new java.util.ArrayList();
    
    public void run() {
        DataStore ds = new DataStore( "appSettings" );
        ds.put( "MainFrame.Size", molmaster.gui.MainFrame.getActiveWindow().getSize() );
        ds.put( "MainFrame.Location", molmaster.gui.MainFrame.getActiveWindow().getLocation() );
        ds.put( "ToolBar.Location", molmaster.gui.tools.ToolFrame.toolFrame.getLocation() );
        ds.flush();
        ds = new DataStore( "lightSettings" );
        ds.put( "MolLight.LightList", molmaster.gui.MolLight.lightList );
        ds.flush();    
        
        /** We don't want to start new threads for each runnable
         *  process, so we simply execute them in sequence.
         */
        java.util.Iterator i = listeners.iterator();
        while ( i.hasNext() ) {
            ((Runnable)i.next()).run();
        }
    }
    
    /** Adds a listener to the shutdown hook.  This listener
     *  will be called when the program is in the process of 
     *  being shutdown.
     */
    public static void addShutdownListener( Runnable r ) {
        listeners.add( r );
    }
    
    public static void removeShutdownListener( Runnable r ) {
        listeners.remove( r );
    }
}
