package molmaster.gui;

import molmaster.*;
import molmaster.gui.tools.*;
import molmaster.model.*;
import molmaster.util.DataStore;

import java.awt.*;
import java.lang.reflect.InvocationTargetException;

import javax.swing.*;
import javax.media.j3d.*;

import com.l2fprod.gui.plaf.skin.*;

/** Loads all classes and data necessary for molmaster.
 *
 * @author  Russell
 */
public class Startup extends javax.swing.JWindow {

	static public String fontName = "Sans Serif";
	static public Font appFont;
	static public String appDir;
	static public String libDir;

	static {

		// the shortest class search path is the root molmaster directory, as the
		// other paths are for libraries within the molmaster directory (really, it's not
		// a total hack!
		java.net.URL[] dirs =
			((java.net.URLClassLoader) Startup.class.getClassLoader())
				.getURLs();
		String shortest = dirs[0].getPath();
		for (int i = 0; i < dirs.length; ++i) {
			if (dirs[i].toString().length() < shortest.length())
				shortest = dirs[i].getPath();
		}

		Log.info("Found molmaster directory to be: " + shortest);
		appDir = shortest;

		String sep = System.getProperty("file.separator");
		libDir = shortest + sep + "lib";

		OptionWindow ow = molmaster.gui.tools.OptionWindow.getWindow();
		ow.addValue(
			"Application",
			"Font name",
			"The font family to be used for application windows",
			Startup.class,
			"fontName");

		/*ow.addValue(
			"Application",
			"Storage directory",
			"The location where molmaster will store persistant data",
			Startup.class,
			"appDir");

		ow.addValue(
			"Application",
			"Library directory",
			"The location of native library files.",
			Startup.class,
			"libDir");
			
		*/
		System.setProperty("basedir", appDir);
		System.setProperty("libdir", libDir);
		appFont = new Font(fontName, Font.PLAIN, 11);
	}
	/** Creates new form Startup */
	public Startup(java.awt.Frame parent, boolean modal) {
		super(parent);
		initComponents();
	}

	private void initComponents() { //GEN-BEGIN:initComponents
		Log.writeLog("Loading startup components.");
		loadingStatus = new javax.swing.JProgressBar();
		final RenderingHints rh =
			new RenderingHints(
				RenderingHints.KEY_ANTIALIASING,
				RenderingHints.VALUE_ANTIALIAS_ON);

		lblLoading = new javax.swing.JLabel() {
			public void paint(Graphics g) {
				((Graphics2D) g).setRenderingHints(rh);
				super.paint(g);
			}
		};
		backLabel = new JLabel() {
			public void paint(Graphics g) {
				Image img = ((ImageIcon) getIcon()).getImage();
				g.drawImage(img, 0, 0, getWidth(), getHeight(), this);
			}
		};

		getContentPane().setLayout(null);

		addWindowListener(new java.awt.event.WindowAdapter() {
			public void windowClosing(java.awt.event.WindowEvent evt) {
				closeDialog(evt);
			}
		});

		loadingStatus.setBounds(30, 195, 170, 10);
		loadingStatus.setOpaque(false);
		//loadingStatus.setBorder(BorderFactory.createLineBorder(Color.black));
		loadingStatus.setBorder(BorderFactory.createEmptyBorder());
		loadingStatus.setForeground(new Color(128, 0, 0));
		getContentPane().add(loadingStatus);

		lblLoading.setFont(new java.awt.Font("Sans Serif", Font.PLAIN, 10));
		lblLoading.setHorizontalAlignment(javax.swing.SwingConstants.CENTER);
		getContentPane().add(lblLoading);
		lblLoading.setBounds(220, 190, 180, 20);

		backLabel.setIcon(
			new javax.swing.ImageIcon(
				getClass().getResource("/graphics/startup.jpg")));
		getContentPane().add(backLabel);
		backLabel.setBounds(0, 0, 400, 300);

		pack();
		java.awt.Dimension screenSize =
			java.awt.Toolkit.getDefaultToolkit().getScreenSize();
		setSize(new java.awt.Dimension(400, 250));
		setLocation(
			(screenSize.width - 400) / 2,
			(screenSize.height - 250) / 2);
	} //GEN-END:initComponents

	/** Closes the dialog */
	private void closeDialog(
		java.awt.event.WindowEvent evt) { //GEN-FIRST:event_closeDialog
		setVisible(false);
		dispose();
	} //GEN-LAST:event_closeDialog

	private static void setDefaultFonts(Font f) {
		Log.info("Setting default font to: " + f.getFamily());
		UIManager.put("Button.font", f);
		UIManager.put("ToggleButton.font", f);
		UIManager.put("RadioButton.font", f);
		UIManager.put("CheckBox.font", f);
		UIManager.put("ColorChooser.font", f);
		UIManager.put("ComboBox.font", f);
		UIManager.put("Label.font", f);
		UIManager.put("List.font", f);
		UIManager.put("MenuBar.font", f);
		UIManager.put("MenuItem.font", f);
		UIManager.put("RadioButtonMenuItem.font", f);
		UIManager.put("CheckBoxMenuItem.font", f);
		UIManager.put("Menu.font", f);
		UIManager.put("PopupMenu.font", f);
		UIManager.put("OptionPane.font", f);
		UIManager.put("Panel.font", f);
		UIManager.put("ProgressBar.font", f);
		UIManager.put("ScrollPane.font", f);
		UIManager.put("Viewport.font", f);
		UIManager.put("TabbedPane.font", f);
		UIManager.put("Table.font", f);
		UIManager.put("TableHeader.font", f);
		UIManager.put("TextField.font", f);
		UIManager.put("PasswordField.font", f);
		UIManager.put("TextArea.font", f);
		UIManager.put("TextPane.font", f);
		UIManager.put("EditorPane.font", f);
		UIManager.put("TitledBorder.font", f);
		UIManager.put("ToolBar.font", f);
		UIManager.put("ToolTip.font", f);
		UIManager.put("Tree.font", f);

	}
	private static boolean isActive(String argument, String[] args) {
		for (int i = 0; i < args.length; i++) {
			if (args[i] != null) {
				if (args[i].indexOf(argument) != -1) {
					return true;
				}
			}
		}
		return false;
	}

	static private int cProgress = 0;
	static private String cMessage = "";

	private static final void setMessage(String m) {
		cMessage = m;
		final Runnable r = new Runnable() {
			public void run() {
				lblLoading.setText(cMessage);
			}
		};
		try {
			SwingUtilities.invokeAndWait(r);
		} catch (InterruptedException e) {
		} catch (InvocationTargetException e) {
		}
	}

	private static final void incrementProgress() {
		++cProgress;
		final Runnable r = new Runnable() {
			public void run() {
				loadingStatus.setValue(cProgress);
			}
		};
		try {
			SwingUtilities.invokeAndWait(r);
		} catch (InterruptedException e) {
		} catch (InvocationTargetException e) {
		}
	}

	/**
	 * @param args the command line arguments
	 */
	public static void main(String args[]) {
		try {
			Log.writeLog("Checking arguments");

			if (isActive("-skin", args)) {
				Log.writeLog("Loading skin...");
				try {
					SkinLookAndFeel.loadThemePack("lib/themepack.zip");
					SkinLookAndFeel.enable();
				} catch (Throwable e) {
					molmaster.Log.writeLog("Couldn't load skin: " + e);
				}
			} else {
				setDefaultFonts(appFont);
			}

			Log.writeLog("Finished reading arguments...");

			Startup s = new Startup(new javax.swing.JFrame(), true);
			s.show();

			VirtualUniverse.setJ3DThreadPriority(Thread.MIN_PRIORITY);

			loadingStatus.setMaximum(7);

			//force some static calls to go through
			new molmaster.gui.BondView();
			new molmaster.gui.AtomView();
			MoleculeFile.class.getMethods();

			setMessage("Initializing atom table...");
			new molmaster.model.AtomTable("elements.dat");
			incrementProgress();

			setMessage("Initializing main window...");
			MainFrame f = new MainFrame();
			incrementProgress();

			setMessage("Starting 3D renderer...");
			f.initSpace();
			incrementProgress();

			MolLight.initLights();

			setMessage("Creating scene control structures...");
			f.scene =
				new SceneControl(
					MolModel.getModel(),
					MainFrame.root,
					MainFrame.uni);
			incrementProgress();

			setMessage("Loading tools...");
			molmaster.gui.tools.ToolFrame toolBar =
				new molmaster.gui.tools.ToolFrame();
			toolBar.show();
			incrementProgress();

			setMessage("Bringing up main window...");
			Log.init();
			loadingStatus.setValue(loadingStatus.getMaximum());

			DataStore ds = new DataStore("appSettings");
			Dimension size = (Dimension) ds.get("MainFrame.Size");
			Point loc = (Point) ds.get("MainFrame.Location");
			if (size != null) {
				f.setSize(size);
			}
			if (loc != null) {
				f.setLocation(loc);
			}

			loc = (Point) ds.get("ToolBar.Location");
			if (loc != null) {
				toolBar.setLocation(loc);
			}

			f.show();

			setMessage("Done.");
			incrementProgress();

			//  try to run the startup script
			try {
				bsh.Interpreter interp = new bsh.Interpreter();
				java.io.FileReader fr =
					new java.io.FileReader("data/startup.mms");
				interp.eval(fr);
				fr.close();
			} catch (Throwable t) {
				System.out.println("Couldn't run startup script: " + t);
			}

			//  check for updates
			new Thread(new molmaster.util.UpdateCheck()).start();

			//  add a shutdown hook
			Runtime.getRuntime().addShutdownHook(
				new Thread(new ShutdownHook()));

			try {
				Thread.sleep(1000);
			} catch (Exception e) {
			}

			s.setVisible(false);
			s.dispose();
		} catch (Exception e) {
			new ErrorDialog(e, "Unexpected Program Error", true).show();
			System.exit(0);
		}
	}
	// Variables declaration - do not modify//GEN-BEGIN:variables
	private static javax.swing.JLabel lblLoading;
	private static javax.swing.JProgressBar loadingStatus;
	private javax.swing.JLabel backLabel;
	// End of variables declaration//GEN-END:variables

}
