/*
 * TextureCache.java
 *
 * Created on May 5, 2002, 11:17 AM
 */

package molmaster.gui;

import molmaster.*;
import java.util.*;
import java.awt.image.*;
import javax.media.j3d.*;
import java.awt.geom.*;

/**
 *
 * @author  Russell Power
 */
public class TextureCache {
    
    static protected Map cache = new HashMap();
    static protected int[] textureSizes = new int[]{ 1, 2, 4, 8, 16, 32, 64, 128, 256, 512, 1024, 2048, 4096, 8192 };
    
    static public Texture2D getTexture( String name ) {        
        BufferedImage b = ImageCache.getImage( name );
        return getTexture(b);
    }
    
    static public Texture2D getTexture( BufferedImage img ) {
        return getTexture( img, true );
    }
    
    static public Texture2D getTexture(BufferedImage img, boolean cacheImage) {
        Object out = cache.get(img);
        
        if ( out != null ) { 
            Log.info( "Hit in texture cache" );
            return (Texture2D)out; 
        }
        
        int nw = closest( img.getWidth() );
        int nh = closest( img.getHeight() );
        
        double sx = (double)nw/(double)img.getWidth();
        double sy = (double)nh/(double)img.getHeight();
        
        AffineTransform trans = AffineTransform.getScaleInstance(sx, sy);
        BufferedImage dest = new BufferedImage( nw, nh, BufferedImage.TYPE_INT_ARGB );
        new AffineTransformOp( trans, AffineTransformOp.TYPE_BILINEAR ).filter( img, dest );
        ImageComponent2D ic = new ImageComponent2D(ImageComponent2D.FORMAT_RGBA, dest );
        Texture2D t = new Texture2D(Texture.BASE_LEVEL, Texture.RGBA, ic.getWidth(), ic.getHeight());
        t.setImage(0, ic);
        t.setMinFilter( Texture.NICEST );
        t.setMagFilter( Texture.NICEST );
        t.setCapability( Texture.ALLOW_SIZE_READ );
        
        if ( cacheImage )
            cache.put( img, t );
        
        return t;
    }
        
    static private int closest( int s ) {
        double closest = 10000000;
        int best = -1;
        
        for ( int i = 0; i < textureSizes.length; i++ ) {
            int t = textureSizes[i];
            if ( s == t) { return s; }
            if ( Math.abs(t - s) < closest ) {
                closest = Math.abs(t - s);
                best = t;
            }
        }
        return best;
    }
}
