/*
 * AngleCalculator.java
 *
 * Created on May 24, 2002, 9:15 PM
 */

package molmaster.gui.editbar;

import molmaster.*;
import molmaster.gui.*;
import molmaster.gui.input.*;

import javax.vecmath.*;

/**
 *
 * @author  Russell
 */
public class AngleCalculator extends DefaultAction {
    
    /** Creates a new instance of AngleCalculator */
    public AngleCalculator() {}
    
    /** Called when a significant update has occured.  Significant is
     * defined as 'whatever Russell felt like implementing', but in general
     * all expected events ( mouse clicks, drags, etc. ) will be reported.
     */
    public void actionPerformed(Object source, molmaster.Event evt) {
        if ( evt.getID().equals(MouseSelect.mouseSelect) ){
            if ( selected.size() == 3 ) { clearSelected(); }
            Object o = evt.getData();
            if ( o instanceof AtomView ){
                selected.add(o);
                ((AtomView)o).select();
            }
        }
        
        if ( selected.size() == 3 ) {
            Atom a = ((AtomView)selected.get(0)).getAtom();
            Atom b = ((AtomView)selected.get(1)).getAtom();
            Atom c = ((AtomView)selected.get(2)).getAtom();
            Vector3d al = a.getLocation();
            Vector3d bl = b.getLocation();
            Vector3d cl = c.getLocation();
            
            al.sub( bl );
            cl.sub( bl );
            
            double dot = al.dot( cl );
            double length = al.length() * cl.length();
            float angle = (float)Math.acos( dot / length );
            angle = (float)Math.toDegrees( angle );
            MainFrame.setStatusMessage( "Angle: " + angle );
        }
        
        super.actionPerformed( source, evt );
    }

    /** Returns the handler group this belongs to - selection handlers might compose
     * one group, while angle calculations would belong to another.  The groups
     * will be clustered on the screen to indicate their relation.
     */
    public String getHandlerGroup() {
        return "Angle";
    }
    
    /** Optional.  Return null if not implemented.  Returns a graphical icon to
     * display along with the text for this handler.    */
    public java.awt.Image getHandlerImage() {
        return null;
    }
    
    /** This is displayed to the user on the selection button for this handler.  */
    public String getHandlerName() {
        return "Bond Angle";
    }
    
}
