package molmaster.gui.editbar;

import molmaster.gui.*;
import molmaster.gui.toolbar.*;
import molmaster.util.*;
import molmaster.model.*;

import com.sun.j3d.utils.geometry.*;
import java.util.*;
import javax.vecmath.*;
import javax.media.j3d.*;

/** Allows the toggling on and off of the
 *  display of coordinate axes on the screen.
 *
 * @author  rjpower
 */
public class AxisDisplay extends DefaultAction
implements CustomView, java.awt.event.ActionListener, molmaster.model.ModelListener  {
    
    static private BranchGroup axisGroup;
    static private TransformGroup axisTransform;
    
    static {
        axisGroup = new BranchGroup();
        axisTransform = new TransformGroup();
        axisTransform.setCapability( TransformGroup.ALLOW_TRANSFORM_WRITE );
        axisGroup.setCapability( BranchGroup.ALLOW_DETACH );
        axisGroup.clearCapabilityIsFrequent( BranchGroup.ALLOW_DETACH );
        
        Appearance lineAppearance = new Appearance();
        
        PolygonAttributes pa = new PolygonAttributes();
        pa.setCullFace( PolygonAttributes.CULL_NONE );
        pa.setPolygonMode( PolygonAttributes.POLYGON_LINE );
        lineAppearance.setPolygonAttributes( pa );
        
        LineAttributes la = new LineAttributes();
        la.setLineWidth( 2.0f );
        la.setLineAntialiasingEnable( true );
        lineAppearance.setLineAttributes( la );
        
        ColoringAttributes ca = new ColoringAttributes();
        ca.setColor( 0, 0, 0 );
        ca.setShadeModel( ColoringAttributes.FASTEST );
        lineAppearance.setColoringAttributes( ca );
        
        Shape3D axes = new Shape3D();
        axes.setAppearance( lineAppearance );
        
        float inf = 9999999999.0f;
        
        GeometryArray aGeom = new TriangleArray( 3, TriangleArray.COORDINATES );
        //  x-axis
        aGeom.setCoordinates( 0,
        new float[] {
            0.0f, 0.0f, 0.0f,
            1.0f, 0.001f, 0.001f,
            1.0f, 0.001f, 0.001f
        } );
        axes.addGeometry( aGeom );
        
        //  y-axis
        aGeom  = new TriangleArray( 3, TriangleArray.COORDINATES );
        aGeom.setCoordinates( 0,
        new float[] {
            0.0f, 0.0f, 0.0f,
            0.001f, 1.0f, 0.001f,
            0.001f, 1.0f, 0.001f
        } );
        axes.addGeometry( aGeom );
        
        //  z-axis
        aGeom = new TriangleArray( 3, TriangleArray.COORDINATES );
        aGeom.setCoordinates( 0,
        new float[] {
            0.0f, 0.0f, 0.0f,
            0.001f, 0.001f, 1.0f,
            0.001f, 0.001f, 1.0f
        } );
        axes.addGeometry( aGeom );
        axes.setPickable( false );
        
        axisTransform.addChild( axes );
        axisGroup.addChild( axisTransform );
    }
    
    protected EditButton checkButton = new EditButton("Display Axes");
    
    protected boolean attached = false;
    
    /** Creates a new instance of AxisDisplay */
    public AxisDisplay() {
        checkButton.addActionListener( this );
        molmaster.model.MolModel.getModel().addListener( this );
    }
    
    private double x,y,z;
    private BranchGroup arrows;
    
    public static final double scaleFactor = 1.5;
    public static final double arrowLength = 1;
    public static final float arrowDiameter = 0.2f;
    public static final double textLoc = arrowLength + 0.5f;
    
    /** Called when a significant update has occured.  Significant is
     *  defined as 'whatever Russell felt like implementing', but in general
     *  all expected events ( mouse clicks, drags, etc. ) will be reported.
     */
    public void actionPerformed(Object source, molmaster.Event evt) {
        if ( evt.getID() == handlerLoaded ) {
            MainFrame.uni.addBranchGraph( axisGroup );
            attached = true;
        }
        else if ( evt.getID() == handlerActivate ) {
            if ( attached ) { axisGroup.detach(); }
            else {
                loadGeometry();
            }
            attached = !attached;
            checkButton.setSelected( attached );
            ActiveAction.setDefaultHandler();
            
        }
        else {
            super.actionPerformed( source, evt );
        }
    }
    
    private double tiny = 0.00001;
    
    private void loadGeometry() {
        x = y = z = 0;
        axisGroup.detach();
        
        Iterator i = MolModel.getModel().getFiles().iterator();
        Point3d temp = new Point3d();
        while ( i.hasNext() ){
            MoleculeFile mf = (MoleculeFile)i.next();
            BoundingBox b = mf.getBounds();
            b.getUpper(temp);
            comp( temp );
            b.getLower(temp);
            comp( temp );
        }
        if ( arrows != null ) {
            arrows.detach();
        }
        
        x *= scaleFactor;
        y *= scaleFactor;
        z *= scaleFactor;
        
        arrows = new BranchGroup();
        arrows.setCapability( BranchGroup.ALLOW_DETACH );
        
        arrows.addChild(
        Arrow.createArrow( arrowDiameter, new Vector3d( x, 0, 0 ), new Vector3d(  x + arrowLength, 0, 0) ) );
        
        arrows.addChild(
        Arrow.createArrow( arrowDiameter, new Vector3d( 0, y, 0 ), new Vector3d( 0, y + arrowLength , 0) ) );
        
        arrows.addChild(
        Arrow.createArrow( arrowDiameter, new Vector3d( 0, 0, z ), new Vector3d( 0, 0, z + arrowLength ) ) );
        
        arrows.addChild( getTextShape( "z", new Vector3d( 0, 0, z + textLoc) ) );
        arrows.addChild( getTextShape( "y", new Vector3d( 0, y + textLoc, 0) ) );
        arrows.addChild( getTextShape( "x", new Vector3d( x + textLoc, 0, 0) ) );
        
        //arrows.addChild( genTickMarks( new Vector3d( 0, 0, z ), 1, (int)z ) );
        //arrows.addChild( genTickMarks( new Vector3d( 0, y, 0 ), 1, (int)y ) );
        //arrows.addChild( genTickMarks( new Vector3d( x, 0, 0 ), 1, (int)x ) );
        
        Transform3D trans = new Transform3D();
        axisTransform.getTransform( trans );
        trans.setScale( new Vector3d( x, y, z) );
        axisTransform.setTransform( trans );
        
        axisGroup.addChild( arrows );
        MainFrame.uni.addBranchGraph( axisGroup );
        molmaster.Log.info( "Attached coordinate-axes to screen." );
    }
    
    private BranchGroup genTickMarks( Vector3d endPoint, double tickDistance, int numTicks ) {
        BranchGroup tickRoot = new BranchGroup();
        
        Vector3d increment = new Vector3d( endPoint );
        //increment.scale( tickDistance / increment.length() );
        increment.normalize();
        
        Vector3d rot = new Vector3d( endPoint );
        rot.normalize();
        
        rot.scale( Math.PI / 2 );
        
        Transform3D trans = new Transform3D();
        trans.setEuler( rot );
        trans.setScale( 5 );
        
        Vector3d current = new Vector3d();
        Text2D textGen = new Text2D( "-", new Color3f( 0, 0, 0 ), "Dialog", 36, java.awt.Font.BOLD );    
        
        for ( int i = 1; i < numTicks; i++ ) {
            current.add( increment );
            trans.setTranslation( current );
            TransformGroup tg = new TransformGroup( trans );    
            tg.addChild( 
            new OrientedShape3D( textGen.getGeometry(), textGen.getAppearance(), 
            OrientedShape3D.ROTATE_ABOUT_AXIS, new Vector3f(0,1,0) ) );
            tickRoot.addChild( tg );
        }
        
        return tickRoot;
    }
    
    /** Determines the maximal x, y and z values.   */
    private void comp( javax.vecmath.Tuple3d p ) {
        double max = p.x;
        if ( max < p.y ) { max = p.y; }
        if ( max < p.z ) { max = p.z; }
        
        if ( max > x ) { x = max; y = max; z = max; }
    }
    
    /** Returns a shape representing the given input
     *  text on the given axis.
     */
    protected TransformGroup getTextShape( String s, Vector3d axis ) {
        Text2D textGen = new Text2D( s, new Color3f( 0, 0, 0 ), "Dialog", 36, java.awt.Font.BOLD );
        textGen.getAppearance().setPolygonAttributes(
        new PolygonAttributes( PolygonAttributes.POLYGON_FILL, PolygonAttributes.CULL_NONE, 0 )
        );
        
        OrientedShape3D align = new OrientedShape3D(
        textGen.getGeometry(), textGen.getAppearance(),
        OrientedShape3D.ROTATE_ABOUT_POINT, new Point3f( ), false, 1 );
        
        TransformGroup tg = new TransformGroup();
        Transform3D trans = new Transform3D();

        trans.setTranslation( axis );
        trans.setScale( 5 );
        
        tg.setTransform( trans );
        tg.setCapability( TransformGroup.ALLOW_TRANSFORM_WRITE );
        tg.setCapability( TransformGroup.ALLOW_TRANSFORM_READ );
        
        tg.addChild( align );
        
        return tg;
        
    }
    
    /** Returns the handler group this belongs to - selection handlers might compose
     *  one group, while angle calculations would belong to another.  The groups
     *  will be clustered on the screen to indicate their relation.
     */
    public String getHandlerGroup() {
        return "Rendering";
    }
    
    /** Optional.  Return null if not implemented.  Returns a graphical icon to
     *  display along with the text for this handler.    */
    public java.awt.Image getHandlerImage() {
        return null;
    }
    
    /** This is displayed to the user on the selection button for this handler.  */
    public String getHandlerName() {
        return "Show Axes";
    }
    
    public molmaster.gui.toolbar.EditButton getButton() {
        return checkButton;
    }
    
    public void actionPerformed(java.awt.event.ActionEvent actionEvent) {
        actionPerformed( null, new molmaster.Event(handlerActivate) );
    }
    
    /** Called in response to a change in the state of the model.  The event type can
     * be any of the defined constants in ModelListener.
     * @param opType The type of event that has occured.
     * @param opValue A handle to the object that has been added or removed from the model.
     *
     */
    public void modelUpdated(MolModel scene, int opType, Object opValue) {
        if ( opType == FILE_REMOVED || opType == FILE_ADDED ) {
            if ( attached ) { loadGeometry(); }
        }
    }
    
}
