package molmaster.gui.editbar;

import molmaster.*;
import molmaster.gui.*;

import molmaster.model.MolModel;
import java.util.Iterator;

/** Clears the scene - at the moment this simply refers
 *  to settings all atoms and bonds to their unselected
 *  state.
 *
 * @author  Russell Power
 */
public class ClearScene implements EditAction {
    
    /** Creates a new instance of ClearScene */
    public ClearScene() {
    }
    
    /** Called when a significant update has occured.  Significant is
     *  defined as 'whatever Russell felt like implementing', but in general
     *  all expected events ( mouse clicks, drags, etc. ) will be reported.
     *
     */
    public void actionPerformed(Object source, molmaster.Event evt) {
        if ( evt.getID() == handlerActivate ) {
            clear();
        }
        ActiveAction.setDefaultHandler();
    }
    
    public static void clear() {
        Iterator i = MolModel.getModel().getAtoms().iterator();
        while ( i.hasNext() ) {
            AtomView.atomToView((Atom)i.next()).reset();
        }
        i = MolModel.getModel().getBonds().iterator();
        while ( i.hasNext() ) {
            BondView.bondToView( i.next() ).unselect();
        }
    }
    
    /** Returns the handler group this belongs to - selection handlers might compose
     *  one group, while angle calculations would belong to another.  The groups
     *  will be clustered on the screen to indicate their relation.
     *
     */
    public String getHandlerGroup() { return "Selection";  }
    
    /** Optional.  Return null if not implemented.  Returns a graphical icon to
     *  display along with the text for this handler.
     */
    public java.awt.Image getHandlerImage() { return null;
    }
    
    /** This is displayed to the user on the selection button for this handler.  */
    public String getHandlerName() { return "Clear Selected";  }
    public String getVersion() { return "0.1";  }
    
}
