/*
 * DihedralCalculator.java
 *
 * Created on May 24, 2002, 11:48 PM
 */

package molmaster.gui.editbar;

import molmaster.*;
import molmaster.gui.*;
import molmaster.gui.input.*;

import javax.vecmath.*;

/**
 *
 * @author  Russell
 */
public class DihedralCalculator extends DefaultAction {
    
    /** Creates a new instance of DihedralCalculator */
    public DihedralCalculator() {
    }
    
    public String getHandlerGroup() {
        return "Angle";
    }
    
    /** Called when a significant update has occured.  Significant is
     * defined as 'whatever Russell felt like implementing', but in general
     * all expected events ( mouse clicks, drags, etc. ) will be reported.
     */
    public void actionPerformed(Object source, molmaster.Event evt) {
        if ( evt.getID().equals( handlerActivate ) ){
            MainFrame.setStatusMessage( "Select initial atom." );
        }
        if ( evt.getID().equals( MouseSelect.mouseSelect ) ){
            if ( selected.size() == 4 ) {
                clearSelected();
            }
            selected.add( evt.getData() );
            ((MolView)evt.getData()).select();
        }
        if ( selected.size() == 1 ) {
            MainFrame.setStatusMessage( "Select planar atoms." );
        }
        if ( selected.size() == 4 ) {
            Atom a= ((AtomView)selected.get(0)).getAtom();
            Atom b = ((AtomView)selected.get(1)).getAtom();
            Atom c = ((AtomView)selected.get(2)).getAtom();
            Atom d = ((AtomView)selected.get(3)).getAtom();
            
            Vector3d p1 = getNormal( a, b, c );
            Vector3d p2 = getNormal( b, c, d );
            
            p1.normalize(); p2.normalize();
            
            float f = (float)Math.acos(p1.dot( p2 ));
            f = (float)Math.toDegrees( f );
            MainFrame.setStatusMessage( "Dihedral angle: " + f );
        }
        
        super.actionPerformed( source, evt );
    }
    protected Vector3d getNormal( Atom a, Atom b, Atom c ) {
        Vector3d al = a.getLocation();
        Vector3d bl = b.getLocation();
        Vector3d cl = c.getLocation();
        
        Vector3d bTmp = new Vector3d();
        Vector3d cTmp = new Vector3d();
        bTmp.sub( bl, al );
        cTmp.sub( cl, al );
        Vector3d normal = new Vector3d();
        normal.cross( bTmp, cTmp );
        return normal;
    }
    
    /** This is displayed to the user on the selection button for this handler.  */
    public String getHandlerName() {
        return "Dihedral Angle";
    }
}
