/*
 * EditAction.java
 *
 * Created on May 24, 2002, 9:07 PM
 */

package molmaster.gui.editbar;

import molmaster.*;

/** Represents an action to be performed by a specific editing or
 *  selection handler in response to some kind of event from another
 *  location in the program.  For example, a selection handler designed
 *  to compute the angle between 3 atoms would work within this 
 *  event structure.
 *
 * @author  Russell
 */
public interface EditAction {
    
    public static final EventID handlerActivate = new EventID( EditAction.class, "Handler Activated" );
    public static final EventID handlerDeactivate = new EventID( EditAction.class, "Handler Deactivated" );
    public static final EventID handlerLoaded = new EventID( EditAction.class, "Handler Loaded" );
    public static final EventID handlerUnloaded = new EventID( EditAction.class, "Handler Unloaded" );
    
    /** Called when a significant update has occured.  Significant is 
     *  defined as 'whatever Russell felt like implementing', but in general
     *  all expected events ( mouse clicks, drags, etc. ) will be reported.
     */
    public void actionPerformed( Object source, molmaster.Event evt );
    
    /** This is displayed to the user on the selection button for this handler. */
    public String getHandlerName();
    
    /** Optional.  Return null if not implemented.  Returns a graphical icon to 
     *  display along with the text for this handler.   */
    public java.awt.Image getHandlerImage();
    
    /** Returns the handler group this belongs to - selection handlers might compose
     *  one group, while angle calculations would belong to another.  The groups
     *  will be clustered on the screen to indicate their relation. 
     */
    public String getHandlerGroup();
    public String getVersion();
    
}
