package molmaster.gui.editbar;

import javax.swing.*;
import java.util.*;
import javax.media.j3d.*;

import molmaster.gui.*;
import molmaster.util.*;
import molmaster.model.*;
import molmaster.model.loaders.*;

/** Renders isosurfaces loaded from an input field file.
 *  Isosurfaces are generated using the marching tetrahedron
 *  method.
 *
 * @author  Russell Power
 */
public class IsoDialog extends javax.swing.JDialog
	implements ModelListener, Listener {

	/** The minimum value the current field takes. */
	protected double min;

	/** The maximum value used by the current field. */
	protected double max;
	/** The current field we are working with. */
	protected FieldFile field;

	/** Represents the current surface object we are working with.  This keeps track
	 * of the associated FieldFile, coloring, resolution...
	 */
	protected Surface currentSurface;

	public static class Surface extends DefaultWatchedObject {
		public Shape3D surfaceData;
		public BranchGroup root;
		public FieldFile owner;
		public String name;
		public java.awt.Color color;

		public float transparency;
		private double isoValue;
		private double quality;

		public String toString() {
			return name;
		}

		public void setIsoValue(double isoValue) {
			this.isoValue = isoValue;
			this.signal(this, 0, null);
		}

		public double getIsoValue() {
			return isoValue;
		}

		public void setQuality(double quality) {
			this.quality = quality;
			this.signal(this, 0, null);
		}

		public double getQuality() {
			return quality;
		}

	}

	/** A default appearance applied to surfaces. */
	static protected Appearance surfaceAppearance = new Appearance();

	static {
		Material m = new Material();
		m.setLightingEnable(true);
		m.setShininess(0);
		ColoringAttributes coloringAttr = new ColoringAttributes();
		coloringAttr.setShadeModel(ColoringAttributes.SHADE_GOURAUD);
		PolygonAttributes pa = new PolygonAttributes();
		pa.setCullFace( PolygonAttributes.CULL_NONE );

		surfaceAppearance.setColoringAttributes(coloringAttr);
		surfaceAppearance.setPolygonAttributes( pa );
		surfaceAppearance.setMaterial(m);
		surfaceAppearance.setTransparencyAttributes(
			new TransparencyAttributes(TransparencyAttributes.FASTEST, 0.5f));
	}

	public Surface getCurrentSurface() {
		return currentSurface;
	}

	public void addSurface(String name) {
		currentSurface = new Surface();
		currentSurface.addListener(this);
		currentSurface.name = name;
		((DefaultListModel) lstSurfaces.getModel()).addElement(currentSurface);
		updateField(true);
	}
	
	public Surface getSurface(String name){
		ListModel lm = lstSurfaces.getModel();
		for ( int i = 0; i < lm.getSize(); ++i ) {
			if ( lm.getElementAt(i).toString().equals(name)) {
				return (Surface)lm.getElementAt(i); 
			}
		}
		return null;
	}

	/** Creates new form IsoDialog */
	public IsoDialog() {
		super(MainFrame.getActiveWindow(), false);
		initComponents();
		lstSurfaces.setModel(new DefaultListModel());
		cmbFields.setModel(new DefaultComboBoxModel());
		loadCombo();

		isoScroll.setMaximum(500 + isoScroll.getModel().getExtent());
		isoScroll.setMinimum(0);
		isoScroll.setValue(250);

		Dictionary hm = new Hashtable();
		hm.put(new Integer(0), new JLabel("Worst"));
		hm.put(new Integer(quality.getMaximum()), new JLabel("Best"));
		quality.setLabelTable(hm);

		btnColor.setBackground(new java.awt.Color(0.6f, 0.6f, 1.0f));
		btnColor.setForeground(btnColor.getBackground());

		MolModel.getModel().addListener(this);
	}

	private void initComponents() { //GEN-BEGIN:initComponents
		pnlSettings = new javax.swing.JPanel();
		jLabel2 = new javax.swing.JLabel();
		cmbFields = new javax.swing.JComboBox();
		lblTitle = new javax.swing.JLabel();
		lblFieldVal = new javax.swing.JLabel();
		isoScroll = new javax.swing.JScrollBar();
		jLabel1 = new javax.swing.JLabel();
		quality = new javax.swing.JSlider();
		jLabel4 = new javax.swing.JLabel();
		jLabel5 = new javax.swing.JLabel();
		transValue = new javax.swing.JSlider();
		btnColor = new javax.swing.JPanel();
		pnlListing = new javax.swing.JPanel();
		btnAdd = new javax.swing.JButton();
		btnRemove = new javax.swing.JButton();
		listScroll = new javax.swing.JScrollPane();
		lstSurfaces = new javax.swing.JList();
		btnClose = new javax.swing.JButton();

		addWindowListener(new java.awt.event.WindowAdapter() {
			public void windowClosing(java.awt.event.WindowEvent evt) {
				closeDialog(evt);
			}
		});

		pnlSettings.setLayout(null);

		jLabel2.setText("Current File");
		pnlSettings.add(jLabel2);
		jLabel2.setBounds(0, 10, 80, 16);

		cmbFields.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				cmbFieldsActionPerformed(evt);
			}
		});

		pnlSettings.add(cmbFields);
		cmbFields.setBounds(50, 30, 190, 25);

		lblTitle.setText("Iso-surface value:");
		pnlSettings.add(lblTitle);
		lblTitle.setBounds(0, 60, 120, 16);

		lblFieldVal.setText("lblFieldVal");
		pnlSettings.add(lblFieldVal);
		lblFieldVal.setBounds(130, 60, 120, 16);

		isoScroll.setOrientation(javax.swing.JScrollBar.HORIZONTAL);
		isoScroll
			.addAdjustmentListener(new java.awt.event.AdjustmentListener() {
			public void adjustmentValueChanged(
				java.awt.event.AdjustmentEvent evt) {
				isoScrollAdjustmentValueChanged(evt);
			}
		});

		pnlSettings.add(isoScroll);
		isoScroll.setBounds(30, 90, 190, 17);

		jLabel1.setText("Rendering Quality:");
		pnlSettings.add(jLabel1);
		jLabel1.setBounds(0, 110, 110, 16);

		quality.setFont(new java.awt.Font("Dialog", 0, 10));
		quality.setMajorTickSpacing(3);
		quality.setMaximum(10);
		quality.setMinorTickSpacing(1);
		quality.setPaintTicks(true);
		quality.setSnapToTicks(true);
		quality.setValue(5);
		quality.addChangeListener(new javax.swing.event.ChangeListener() {
			public void stateChanged(javax.swing.event.ChangeEvent evt) {
				qualityStateChanged(evt);
			}
		});

		pnlSettings.add(quality);
		quality.setBounds(30, 130, 190, 40);

		jLabel4.setText("Color:");
		pnlSettings.add(jLabel4);
		jLabel4.setBounds(0, 180, 33, 16);

		jLabel5.setText("Transparency");
		pnlSettings.add(jLabel5);
		jLabel5.setBounds(100, 180, 90, 16);

		transValue.addChangeListener(new javax.swing.event.ChangeListener() {
			public void stateChanged(javax.swing.event.ChangeEvent evt) {
				transValueStateChanged(evt);
			}
		});

		pnlSettings.add(transValue);
		transValue.setBounds(120, 200, 130, 20);

		btnColor.setBackground(new java.awt.Color(153, 153, 0));
		btnColor.setBorder(
			new javax.swing.border.BevelBorder(
				javax.swing.border.BevelBorder.RAISED));
		btnColor.addMouseListener(new java.awt.event.MouseAdapter() {
			public void mouseClicked(java.awt.event.MouseEvent evt) {
				btnColorMouseClicked(evt);
			}
		});

		pnlSettings.add(btnColor);
		btnColor.setBounds(10, 210, 90, 20);

		getContentPane().add(pnlSettings, java.awt.BorderLayout.CENTER);

		pnlListing.setLayout(
			new javax.swing.BoxLayout(
				pnlListing,
				javax.swing.BoxLayout.Y_AXIS));

		pnlListing.setBorder(new javax.swing.border.EtchedBorder());
		btnAdd.setText("Add");
		btnAdd.setToolTipText("Add a new isosurface to the scene");
		btnAdd.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				btnAddActionPerformed(evt);
			}
		});

		pnlListing.add(btnAdd);

		btnRemove.setText("Remove");
		btnRemove.setToolTipText(
			"Remove the currently selected isosurface from the scene");
		btnRemove.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				btnRemoveActionPerformed(evt);
			}
		});

		pnlListing.add(btnRemove);

		listScroll.setPreferredSize(new java.awt.Dimension(100, 131));
		lstSurfaces
			.addListSelectionListener(
				new javax
				.swing
				.event
				.ListSelectionListener() {
			public void valueChanged(
				javax.swing.event.ListSelectionEvent evt) {
				lstSurfacesValueChanged(evt);
			}
		});

		listScroll.setViewportView(lstSurfaces);

		pnlListing.add(listScroll);

		btnClose.setText("Close");
		btnClose.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				btnCloseActionPerformed(evt);
			}
		});

		pnlListing.add(btnClose);

		getContentPane().add(pnlListing, java.awt.BorderLayout.EAST);

		pack();
		java.awt.Dimension screenSize =
			java.awt.Toolkit.getDefaultToolkit().getScreenSize();
		setSize(new java.awt.Dimension(437, 259));
		setLocation(
			(screenSize.width - 437) / 2,
			(screenSize.height - 259) / 2);
	} //GEN-END:initComponents

	private void transValueStateChanged(
		javax.swing.event.ChangeEvent evt) {
		//GEN-FIRST:event_transValueStateChanged
		updateField(false);
	} //GEN-LAST:event_transValueStateChanged

	private void lstSurfacesValueChanged(
		javax.swing.event.ListSelectionEvent evt) {
		//GEN-FIRST:event_lstSurfacesValueChanged
		Surface s = (Surface) lstSurfaces.getSelectedValue();
		if (s == null) {
			if (lstSurfaces.getModel().getSize() > 0) {
				lstSurfaces.setSelectedIndex(0);
			}
			return;
		}
		if (s == currentSurface) {
			return;
		}
		locked = true;
		currentSurface = s;
		cmbFields.setSelectedItem(s.owner);
		btnColor.setBackground(s.color);
		transValue.setValue((int) s.transparency);
		isoScroll.setValue((int) s.getIsoValue());
		quality.setValue((int) s.getQuality());
		locked = false;

		updateField(false);
	} //GEN-LAST:event_lstSurfacesValueChanged

	private void cmbFieldsActionPerformed(
		java.awt.event.ActionEvent evt) {
		//GEN-FIRST:event_cmbFieldsActionPerformed
		updateField(false);
	} //GEN-LAST:event_cmbFieldsActionPerformed

	private void btnColorMouseClicked(
		java.awt.event.MouseEvent evt) { //GEN-FIRST:event_btnColorMouseClicked
		java.awt.Color out =
			JColorChooser.showDialog(
				this,
				"Select field color",
				new java.awt.Color(0.5f, 0.5f, 0.5f));
		//System.out.println( out );
		btnColor.setBackground(out);
		btnColor.setForeground(out);
		updateField(false);
	} //GEN-LAST:event_btnColorMouseClicked

	private void btnColorActionPerformed(
		java.awt.event.ActionEvent evt) {
		//GEN-FIRST:event_btnColorActionPerformed

	} //GEN-LAST:event_btnColorActionPerformed

	private void btnRemoveActionPerformed(
		java.awt.event.ActionEvent evt) {
		//GEN-FIRST:event_btnRemoveActionPerformed
		Surface s = (Surface) lstSurfaces.getSelectedValue();
		if (s != null) {
			s.root.detach();
		}
		currentSurface = null;
		((DefaultListModel) lstSurfaces.getModel()).removeElement(s);
	} //GEN-LAST:event_btnRemoveActionPerformed

	private void btnAddActionPerformed(
		java.awt.event.ActionEvent evt) {
		//GEN-FIRST:event_btnAddActionPerformed
		addSurface("new surface");
	} //GEN-LAST:event_btnAddActionPerformed

	private void btnCloseActionPerformed(
		java.awt.event.ActionEvent evt) {
		//GEN-FIRST:event_btnCloseActionPerformed
		closeWindow();
	} //GEN-LAST:event_btnCloseActionPerformed

	private void qualityStateChanged(
		javax.swing.event.ChangeEvent evt) {
		//GEN-FIRST:event_qualityStateChanged
		double q = (quality.getMaximum() - quality.getValue()) + 2;
		if (currentSurface != null)
			currentSurface.setQuality(q);
	} //GEN-LAST:event_qualityStateChanged
	
	private boolean locked = false;

	/**
	 * @see molmaster.model.Listener#wakeUp(java.lang.Object, int, java.lang.Object)
	 */
	public void wakeUp(Object signaler, int signalType, Object value) {
		updateField(true);
	}

	private void updateField(boolean recalc) {
		if (currentSurface == null) {
			return;
		}
		if (locked) {
			return;
		}

		field = (FieldFile) cmbFields.getSelectedItem();
		if (field == null) {
			return;
		}

		locked = true;
		max = field.getFieldMax();
		min = field.getFieldMin();

		int val = (int)currentSurface.getIsoValue();
		double valD = val * 1.0 / 500.0;
		valD = (min + valD * (max - min));

		lblFieldVal.setText("" + (float) valD);

		//if ( (isoScroll.getValue() != currentSurface.getIsoValue()) ||
		//     (quality.getValue() != currentSurface.getQuality() ) ) {
		if (recalc) {
			double[][][] fileField = field.getField();
			MarchingCubes mc = new MarchingCubes();

			double q = (quality.getMaximum() - quality.getValue()) + 1;
			Shape3D s =
				mc.getSurface(fileField, field.getBasis(), valD, (int) q);
			s.setCapability(Shape3D.ALLOW_APPEARANCE_WRITE);

			currentSurface.surfaceData = s;
			if (currentSurface.root != null) {
				currentSurface.root.detach();
			}

			currentSurface.root = new BranchGroup();
			currentSurface.root.setCapability(BranchGroup.ALLOW_DETACH);
			currentSurface.root.addChild(currentSurface.surfaceData);
			currentSurface.root.compile();
		}

		Appearance ap = (Appearance) surfaceAppearance.cloneNodeComponent(true);

		float transAmount = transValue.getValue() / 100.0f;
		javax.vecmath.Color3f modelColor =
			new javax.vecmath.Color3f(btnColor.getBackground());
		ap.getMaterial().setDiffuseColor(modelColor);
		ap.getTransparencyAttributes().setTransparency(transAmount);

		currentSurface.surfaceData.setAppearance(ap);
		currentSurface.owner = field;

		currentSurface.setQuality(quality.getValue());
		currentSurface.setIsoValue(isoScroll.getValue());
		currentSurface.transparency = transValue.getValue();
		currentSurface.color = btnColor.getBackground();

		if (!currentSurface.root.isLive()) {
			MainFrame.root.addChild(currentSurface.root);
		}
		locked = false;
	}

	private void isoScrollAdjustmentValueChanged(
		java.awt.event.AdjustmentEvent evt) {
		//GEN-FIRST:event_isoScrollAdjustmentValueChanged
		if (currentSurface != null)
			currentSurface.setIsoValue(isoScroll.getValue());
	} //GEN-LAST:event_isoScrollAdjustmentValueChanged

	/** Closes the dialog */
	private void closeDialog(
		java.awt.event.WindowEvent evt) { //GEN-FIRST:event_closeDialog
		closeWindow();
	} //GEN-LAST:event_closeDialog

	private void closeWindow() {
	}

	private void loadCombo() {
		DefaultComboBoxModel cm = new DefaultComboBoxModel();
		Iterator i = MolModel.getModel().getFiles().iterator();
		while (i.hasNext()) {
			Object o = i.next();
			if (o instanceof FieldFile) {
				cm.addElement(o);
			}
		}
		cmbFields.setModel(cm);
	}

	/** Used to keep track of when files have been added or removed from the scene. */
	public void modelUpdated(MolModel scene, int opType, Object opValue) {
		if (opType == FILE_ADDED || opType == FILE_REMOVED) {
			loadCombo();
		}
	}

	// Variables declaration - do not modify//GEN-BEGIN:variables
	private javax.swing.JSlider quality;
	private javax.swing.JButton btnClose;
	private javax.swing.JPanel pnlListing;
	private javax.swing.JScrollBar isoScroll;
	private javax.swing.JPanel pnlSettings;
	private javax.swing.JSlider transValue;
	private javax.swing.JPanel btnColor;
	private javax.swing.JButton btnAdd;
	private javax.swing.JLabel lblFieldVal;
	private javax.swing.JList lstSurfaces;
	private javax.swing.JComboBox cmbFields;
	private javax.swing.JScrollPane listScroll;
	private javax.swing.JLabel jLabel5;
	private javax.swing.JLabel jLabel4;
	private javax.swing.JLabel jLabel2;
	private javax.swing.JLabel jLabel1;
	private javax.swing.JLabel lblTitle;
	private javax.swing.JButton btnRemove;

	// End of variables declaration//GEN-END:variables

}
