/*
 * MultiSelect.java
 *
 * Created on May 28, 2002, 10:33 AM
 */

package molmaster.gui.editbar;

import molmaster.*;
import molmaster.gui.*;
import molmaster.gui.input.*;

import java.awt.*;
import java.awt.event.*;

import javax.media.j3d.*;

/** Provides for selection of a rectangular range of
 *  objects in the scene.
 *
 * @author  Russell Power
 */
public class MultiSelect extends DefaultAction {
    protected MouseDrag md;
    protected Point startPoint = new Point();
    protected Dimension end = new Dimension();
    protected Canvas3D drawCanvas;
    protected int x, y;
    protected boolean dragged = false;
    
    /** Called when a significant update has occured.  Significant is
     * defined as 'whatever Russell felt like implementing', but in general
     * all expected events ( mouse clicks, drags, etc. ) will be reported.
     */
    public void actionPerformed(Object source, molmaster.Event evt) {
        if ( evt.getID() == handlerActivate ) {
            md = new MouseDrag( MainFrame.canvas, MainFrame.branchRoot, MainFrame.uni );
            MouseHandler.getActiveHandler().addHandler(  md );
            MouseHandler.getActiveHandler().setExclusive( md );
            drawCanvas = MainFrame.canvas;
            MainFrame.setStatusMessage( "Multi selection enabled." );
        }
        
        if ( evt.getID() == handlerDeactivate ) {
            MouseHandler.getActiveHandler().releaseExclusive();
            MouseHandler.getActiveHandler().removeHandler( md );
            clearSelected();
            md = null;
        }
        
        if ( evt.getID() == MouseMethod.mousePressed ) {
            MouseEvent e = (MouseEvent)evt.getData();
            x = e.getX() - drawCanvas.getX(); y = e.getY() - drawCanvas.getY();
            startPoint.x = x; startPoint.y = y;
            end.width = 0; end.height = 0;
        }
        
        if ( evt.getID() == MouseMethod.mouseDragged ) {
            MouseEvent e = (MouseEvent)evt.getData();
            x = e.getX() - drawCanvas.getX(); y = e.getY() - drawCanvas.getY();
            
            if ( drawCanvas.isRendererRunning() ) {
                drawCanvas.stopRenderer();
                dragged = true;
            }
            
            Graphics g = drawCanvas.getGraphics();
            /** We use XOR so we get away with not having to keep a separate
             *  buffer when we show the drag rectangle.  We first restore the
             *  original image, then draw our new rectangle.    */
            g.setXORMode( Color.white );
            
            g.drawRect( startPoint.x, startPoint.y, end.width, end.height);
            int w = x - startPoint.x;
            int h = y - startPoint.y;
            
            end.width = w;
            end.height = h;
            
            g.drawRect( startPoint.x, startPoint.y, w, h);
        }
        
        if ( evt.getID() == MouseMethod.mouseReleased ) {
            if ( !dragged )
                return;
            dragged = false;
            
            clearSelected();
            
            //  rectangle will be destroyed by 3d engine...
            Log.writeLog("Perform multi-select...");
            selected = md.pickArea( startPoint.x, startPoint.y, end.width, end.height );
            Log.writeLog( "Selected " + selected.size() + " shapes.");
            
            java.util.Iterator i = molmaster.model.MolModel.getModel().getAtoms().iterator();
            while( i.hasNext() ) {
                AtomView.atomToView( (Atom)i.next() ).unselect();
            }
            
            i = selected.iterator();
            while ( i.hasNext() ) {
                ((MolView)i.next()).select();
            }
            
            drawCanvas.startRenderer();
        }
        
        super.actionPerformed( source, evt );
    }
    
    /** Returns the handler group this belongs to - selection handlers might compose
     * one group, while angle calculations would belong to another.  The groups
     * will be clustered on the screen to indicate their relation.
     */
    public String getHandlerGroup() {
        return "Selection";
    }
    
    /** Optional.  Return null if not implemented.  Returns a graphical icon to
     * display along with the text for this handler.    */
    public java.awt.Image getHandlerImage() {
        return null;
    }
    
    /** This is displayed to the user on the selection button for this handler.  */
    public String getHandlerName() {
        return "Multi Select";
    }
    
}
