package molmaster.gui.editbar;

import java.util.*;
import javax.swing.table.*;

import molmaster.util.*;

/** Allows the user to specify which plugins
 *  should be used with molmaster.
 *
 * @author  rjpower
 */
public class PluginDialog extends javax.swing.JDialog {
    
    private DefaultTableModel tblModel;
    private List modelData = new ArrayList();
    
    /** Creates new form PluginDialog */
    public PluginDialog() {
        super(molmaster.gui.MainFrame.getActiveWindow(), true);
        initComponents();
        tblModel = (DefaultTableModel)tblPlugins.getModel();
        List plugins = EditBar.getPlugins();
        Object o = null;
        for ( Iterator i = plugins.iterator(); i.hasNext(); o = i.next() ){
            EditAction editPlugin = (EditAction)o;
            if ( tblModel == null ) { continue; }
            if ( editPlugin == null ) { continue; }
            tblModel.addRow( new Object[] {
                editPlugin.getHandlerName(), editPlugin.getHandlerGroup(),
                editPlugin.getVersion(), 
                new Boolean(EditBar.isActive( editPlugin ) )} );
            modelData.add( editPlugin );
        }
    }
    
    private void initComponents() {//GEN-BEGIN:initComponents
        btnPanel = new javax.swing.JPanel();
        btnOkay = new javax.swing.JButton();
        btnCancel = new javax.swing.JButton();
        btnHelp = new javax.swing.JButton();
        tblScroll = new javax.swing.JScrollPane();
        tblPlugins = new javax.swing.JTable();

        addWindowListener(new java.awt.event.WindowAdapter() {
            public void windowClosing(java.awt.event.WindowEvent evt) {
                closeDialog(evt);
            }
        });

        btnOkay.setMnemonic('s');
        btnOkay.setText("Save Settings");
        btnOkay.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnOkayActionPerformed(evt);
            }
        });

        btnPanel.add(btnOkay);

        btnCancel.setMnemonic('c');
        btnCancel.setText("Cancel");
        btnCancel.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnCancelActionPerformed(evt);
            }
        });

        btnPanel.add(btnCancel);

        btnHelp.setMnemonic('h');
        btnHelp.setText("Help");
        btnPanel.add(btnHelp);

        getContentPane().add(btnPanel, java.awt.BorderLayout.SOUTH);

        tblScroll.setFont(new java.awt.Font("Dialog", 0, 10));
        tblPlugins.setModel(new javax.swing.table.DefaultTableModel(
            new Object [][] {

            },
            new String [] {
                "Plugin Name", "Plugin Group", "Version", "Active"
            }
        ) {
            Class[] types = new Class [] {
                java.lang.String.class, java.lang.String.class, java.lang.String.class, java.lang.Boolean.class
            };
            boolean[] canEdit = new boolean [] {
                false, false, false, true
            };

            public Class getColumnClass(int columnIndex) {
                return types [columnIndex];
            }

            public boolean isCellEditable(int rowIndex, int columnIndex) {
                return canEdit [columnIndex];
            }
        });
        tblScroll.setViewportView(tblPlugins);

        getContentPane().add(tblScroll, java.awt.BorderLayout.CENTER);

        pack();
    }//GEN-END:initComponents

    private void btnCancelActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnCancelActionPerformed
        setVisible( false );
        dispose();
    }//GEN-LAST:event_btnCancelActionPerformed

    private void btnOkayActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnOkayActionPerformed
        Vector tblData = tblModel.getDataVector();
        Iterator i = tblData.iterator();
        int count = 0;
        DataStore output = new DataStore( "plugins" );
        while ( i.hasNext() ) {
            Vector row = (Vector)i.next();
            Object[] o = row.toArray();
            EditAction e = (EditAction)modelData.get( count++ );
            output.put( EditBar.getPath( e ), (Boolean)o[3] );
        }
        EditBar.reset();
        setVisible( false );
        dispose();
    }//GEN-LAST:event_btnOkayActionPerformed
    
    /** Closes the dialog */
    private void closeDialog(java.awt.event.WindowEvent evt) {//GEN-FIRST:event_closeDialog
        setVisible(false);
        dispose();
    }//GEN-LAST:event_closeDialog

    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JTable tblPlugins;
    private javax.swing.JScrollPane tblScroll;
    private javax.swing.JPanel btnPanel;
    private javax.swing.JButton btnOkay;
    private javax.swing.JButton btnCancel;
    private javax.swing.JButton btnHelp;
    // End of variables declaration//GEN-END:variables
    
}
