package molmaster.gui.editbar;

import molmaster.*;
import molmaster.gui.*;

import javax.vecmath.*;
import javax.media.j3d.*;
import com.sun.j3d.utils.geometry.*;

/** Provides support for selecting atoms and bonds in the
 *  current scene.
 *
 * @author  Russell
 */
public class ViewSelect extends DefaultAction implements InitialAction {
    
    private BranchGroup textGroup = null;
    private TransformGroup textTransform = null;
    private Text2D viewText = new Text2D( "Stuff", new Color3f( 0.0f, 0.0f, 0.0f ), "Dialog", 36, java.awt.Font.BOLD);
    private OrientedShape3D billboard = new OrientedShape3D();
    
    
    /** Creates a new instance of ViewSelect */
    public ViewSelect() {
        viewText.setCapability( Shape3D.ALLOW_APPEARANCE_WRITE );
        viewText.setCapability( Shape3D.ALLOW_APPEARANCE_READ );
        viewText.setCapability( Shape3D.ALLOW_GEOMETRY_WRITE );
        viewText.setCapability( Shape3D.ALLOW_GEOMETRY_READ );
        Appearance a = viewText.getAppearance();
        a.setCapability( Appearance.ALLOW_TEXTURE_READ );
        a.setCapability( Appearance.ALLOW_TEXTURE_WRITE );
        
        PolygonAttributes polyAttr = new PolygonAttributes();
        polyAttr.setCullFace( PolygonAttributes.CULL_NONE );
        a.setPolygonAttributes( polyAttr );
        
        //ColoringAttributes colorAttr = new ColoringAttributes();
       //colorAttr.setColor( 0.8f, 0.6f, 0.6f );
        //a.setColoringAttributes( colorAttr );
        
        billboard.setAppearance( viewText.getAppearance() );
        
    }
    
    /** Called when a significant update has occured.  Significant is
     * defined as 'whatever Russell felt like implementing', but in general
     * all expected events ( mouse clicks, drags, etc. ) will be reported.
     */
    public void actionPerformed(Object source, molmaster.Event evt ) {
        if ( evt.getID().equals( molmaster.gui.input.MouseSelect.mouseSelect ) ){
            MolView shape = (MolView)evt.getData();
            
            if ( shape != null ) {
                clearSelected();
                shape.select();
                selected.add( shape );
            }
            
            atomInfo( shape );
        }
        
        super.actionPerformed( source, evt );
        
    }
    
    /** Displays some information about the given selected atom. */    
    protected void atomInfo( MolView sel ){
        if ( sel instanceof AtomView ) {
            
            AtomView av = (AtomView)sel;
            Atom a = av.getAtom();
            
            Vector3d p = a.getLocation();
            String txt = "Location (X,Y,Z): (" + p + ")";
            txt += "\n\n Atom Type: "+ ((AtomAttributes)a.getAttributes()).atomName;
            MainFrame.setStatusMessage( txt );
             
            /*
            if ( textGroup != null ) {
                textGroup.detach();
                textGroup.removeAllChildren();
                textTransform.removeAllChildren();
            }
            
            viewText.setString( txt );
            billboard.setRotationPoint( new Point3f ( 0,0,0 ) );
            billboard.setAlignmentMode( billboard.ROTATE_ABOUT_POINT );
            //billboard.setScale( 60 );
            //billboard.setConstantScaleEnable( true );
            
            textGroup = new BranchGroup();
            textGroup.setCapability( BranchGroup.ALLOW_DETACH );
            
            textTransform = new TransformGroup();
            Transform3D trans = new Transform3D();
            trans.setTranslation( p );
            trans.setScale( new Vector3d( 10, 5, 1) );
            textTransform.setTransform( trans );
            
            billboard.setGeometry( viewText.getGeometry() );
            billboard.setAppearance( viewText.getAppearance() );
            
            textTransform.addChild( billboard );
            textGroup.addChild( textTransform );
            MainFrame.root.addChild( textGroup );
            */
           
            
        }
    }
    
    /** Returns the handler group this belongs to - selection handlers might compose
     * one group, while angle calculations would belong to another.  The groups
     * will be clustered on the screen to indicate their relation.
     */
    public String getHandlerGroup() {
        return "Selection";
    }
    
    /** Optional.  Return null if not implemented.  Returns a graphical icon to
     * display along with the text for this handler.    */
    public java.awt.Image getHandlerImage() {
        return null;
    }
    
    /** This is displayed to the user on the selection button for this handler.  */
    public String getHandlerName() {
        return "Object Select";
    }
    
}
