package molmaster.gui.editbar.analysis;

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.*;

import javax.swing.*;

import molmaster.*;
import molmaster.model.*;
import molmaster.util.*;

/** Base functionality for performing analysis.
 *
 * @author  Russell Power
 */
public class AnalysisDialog extends JDialog implements ActionListener, ComponentListener {
    protected JComboBox currentFile = new JComboBox();
    protected FileList theList = FileList.getDefaultList();
    protected Object selected;
    protected Collection atoms;
    protected JButton btnSize = new JButton("Bond Size");
    protected Map specialBonds = new HashMap();
    
    public AnalysisDialog() {
        super( molmaster.gui.MainFrame.getActiveWindow(), false );
        init();
    }
    
    public AnalysisDialog( Frame parent ) {
        super( parent, false );
        init();
    }
    
    public AnalysisDialog( Dialog parent ) {
        super( parent, false );
        init();
    }
    
    protected void init() {
        currentFile.addActionListener( 
        new ActionListener() {
            public void actionPerformed( ActionEvent e ){
                currentFileAction();
            }
        } );
        
        theList.addListener( this );    
        
        btnSize.addActionListener( 
        new ActionListener() {
            public void actionPerformed( ActionEvent e ){
                //selected = currentFile.getSelectedItem();
                MoleculeFile mf = (MoleculeFile)theList.getLoaderMap().get( selected );
                if ( mf == null ) { return; }
                Collection c = mf.getAtoms();
                Iterator i = c.iterator();
                ArrayList typeList = new ArrayList();
                Set typeSet = new HashSet();
                while ( i.hasNext() ) {
                    Atom a = (Atom)i.next();
                    AtomAttributes aa = (AtomAttributes)a.getAttributes();
                    if ( !typeSet.contains( aa.atomicSymbol ) ) {
                        typeSet.add( aa.atomicSymbol );
                        typeList.add( a );
                    }
                }
                i = typeList.iterator();
                while ( i.hasNext() ) {
                    Atom a = (Atom)i.next();
                    System.out.println("Type:" + a);
                }
                
                BondList b = new BondList( typeList, specialBonds );
                b.show();
                specialBonds = b.getSpecial();
            }
        } );
        
        loadCombo();
    }
    protected void currentFileAction() {
        if ( selected instanceof File && selected != null ) {
            MoleculeFile mf = (MoleculeFile)theList.getLoaderMap().get(selected);
            //mf.removeFromScene( MolModel.getModel() );
        }
        
        selected  = currentFile.getSelectedItem();
        if (selected instanceof File) {
            MoleculeFile mf = (MoleculeFile)theList.getLoaderMap().get(selected);
            atoms = mf.getAtoms();
            //mf.addToScene( MolModel.getModel() );
            //System.out.println("Changed file...");
        }
    }
    
    protected void loadCombo() {
        Iterator i = theList.getFiles().iterator();
        DefaultComboBoxModel cm = new DefaultComboBoxModel();
        while ( i.hasNext() ) {
            cm.addElement( i.next() );
        }
        //cm.addElement( "All Files" );
        currentFile.setModel( cm );
        if ( selected == null && cm.getSize() > 0 ) {
            selected = cm.getElementAt(0);
        }
        repaint();
    }
    
    public void actionPerformed(java.awt.event.ActionEvent actionEvent) {
        loadCombo();
        selected = currentFile.getSelectedItem();
        if ( currentFile.getModel().getSize() > 0 ) {
            currentFile.setSelectedIndex(0);
        }
    }
    
    public void componentHidden(java.awt.event.ComponentEvent componentEvent) {
    }
    
    public void componentMoved(java.awt.event.ComponentEvent componentEvent) {
    }
    
    public void componentResized(java.awt.event.ComponentEvent componentEvent) {
    }
    
    public void componentShown(java.awt.event.ComponentEvent componentEvent) {
    }
    
}
