package molmaster.gui.editbar.analysis;

import java.util.*;
import javax.swing.*;

import molmaster.*;
import molmaster.gui.tools.*;
import molmaster.gui.tools.reflect.*;

/** Display a list of bonding ranges between
 *  atom types, allowing the user to modify
 *  the defaults.
 *
 * @author  Russell Power
 */
public class BondList extends javax.swing.JDialog {
    
    private Map specialized;
    private Atom start, end;
    private BondSize bOut;
    
    public BondList( Collection input, Map special ) {
        super ( molmaster.gui.MainFrame.getActiveWindow(), true );
        initComponents();
        DefaultListModel lm = new DefaultListModel();
        Iterator i = input.iterator();
        while ( i.hasNext() ) {
            lm.addElement( i.next() );
        }
        atomStart.setModel( lm );
        atomEnd.setModel( lm );
        
        specialized = special;
        txtHelp.setText( molmaster.util.GetText.get( "BondList.Help" ) );
        atomTypeSplit.setBottomComponent( txtHelp );
    }
    
    private void updateTable() { 
        if ( start == null || end == null ) {
            return;
        }
        
        if ( bOut != null ) {
            Bond b = new Bond( bOut.start, bOut.end, Bond.BOND_TYPE_NORMAL );
            specialized.put( b.getBondTypeID(), bOut );
        }
        Bond b = new Bond( start, end, Bond.BOND_TYPE_NORMAL );
        
        if ( specialized.containsKey( b.getBondTypeID() ) ) {
            bOut = (BondSize)specialized.get( b.getBondTypeID() );
        }else {
            bOut = new BondSize();
            bOut.start = start;
            bOut.end = end;
            bOut.setMaxBondSize( Bond.bondLength( start, end ) );
        }
        atomTypeSplit.setBottomComponent( atomTable );
        atomTable.setModel( new ReflectModel( bOut ) );
    }
    
    public Map getSpecial() { 
        /*
        List out = new ArrayList();
        Iterator i = specialized.values().iterator();
        while ( i.hasNext() ) {
            out.add( i.next() );
        }
        return out;
         */
        return specialized;
    }
    private void initComponents() {//GEN-BEGIN:initComponents
        txtHelp = new javax.swing.JTextArea();
        atomTypeSplit = new javax.swing.JSplitPane();
        topSplit = new javax.swing.JSplitPane();
        atomStart = new javax.swing.JList();
        atomEnd = new javax.swing.JList();
        atomTable = new javax.swing.JTable();

        txtHelp.setEditable(false);
        txtHelp.setLineWrap(true);
        txtHelp.setWrapStyleWord(true);

        addWindowListener(new java.awt.event.WindowAdapter() {
            public void windowClosing(java.awt.event.WindowEvent evt) {
                closeDialog(evt);
            }
        });

        atomTypeSplit.setDividerLocation(100);
        atomTypeSplit.setOrientation(javax.swing.JSplitPane.VERTICAL_SPLIT);
        topSplit.setDividerLocation(100);
        atomStart.addListSelectionListener(new javax.swing.event.ListSelectionListener() {
            public void valueChanged(javax.swing.event.ListSelectionEvent evt) {
                atomStartValueChanged(evt);
            }
        });

        topSplit.setLeftComponent(atomStart);

        atomEnd.addListSelectionListener(new javax.swing.event.ListSelectionListener() {
            public void valueChanged(javax.swing.event.ListSelectionEvent evt) {
                atomEndValueChanged(evt);
            }
        });

        topSplit.setRightComponent(atomEnd);

        atomTypeSplit.setTopComponent(topSplit);

        atomTable.setModel(new ReflectModel( new BondHolder(), ReflectModel.SHOW_ALL ));
        atomTypeSplit.setBottomComponent(atomTable);

        getContentPane().add(atomTypeSplit, java.awt.BorderLayout.CENTER);

        pack();
        java.awt.Dimension screenSize = java.awt.Toolkit.getDefaultToolkit().getScreenSize();
        setSize(new java.awt.Dimension(457, 234));
        setLocation((screenSize.width-457)/2,(screenSize.height-234)/2);
    }//GEN-END:initComponents

    private void atomEndValueChanged(javax.swing.event.ListSelectionEvent evt) {//GEN-FIRST:event_atomEndValueChanged
        end = (Atom)atomEnd.getSelectedValue();
        updateTable();
    }//GEN-LAST:event_atomEndValueChanged

    private void atomStartValueChanged(javax.swing.event.ListSelectionEvent evt) {//GEN-FIRST:event_atomStartValueChanged
        start = (Atom)atomStart.getSelectedValue();
        updateTable();
    }//GEN-LAST:event_atomStartValueChanged
    
    /** Closes the dialog */
    private void closeDialog(java.awt.event.WindowEvent evt) {//GEN-FIRST:event_closeDialog
        setVisible(false);
        dispose();
    }//GEN-LAST:event_closeDialog
        
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JTable atomTable;
    private javax.swing.JSplitPane atomTypeSplit;
    private javax.swing.JTextArea txtHelp;
    private javax.swing.JList atomStart;
    private javax.swing.JList atomEnd;
    private javax.swing.JSplitPane topSplit;
    // End of variables declaration//GEN-END:variables
    
}
