package molmaster.gui.editbar.analysis;

import java.io.*;
import java.util.*;
import javax.swing.*;
import java.awt.event.*;
import javax.swing.table.*;

import javax.vecmath.*;

import molmaster.*;
import molmaster.gui.*;
import molmaster.util.*;
import molmaster.model.*;
import molmaster.gui.tools.*;

/** Graphical display allowing users to select
 *  the criteria for highlighting desired atoms.
 *
 * @author  Russell Power
 */
public class CNDialog extends AnalysisDialog {
    
    static  private final String helpMessage = molmaster.util.GetText.get("CNAnalysis.Help");
    private molmaster.util.BondGenerator gen;
    private molmaster.util.CNAnalysis analyzer;
    private Map fileToLoader = FileList.getDefaultList().getLoaderMap();
    private JButton cmdExport = new JButton("Export data");
    private final JPanel btnColor = new JPanel();
    private boolean selectNeighbors = true;
    
    public CNDialog() {
        initComponents();
        filterPanel.add( currentFile, java.awt.BorderLayout.NORTH );
        cmdPanel.add( cmdExport );
        cmdPanel.add( btnSize );
        setTitle( "CN Analysis" );
        cmdExport.addActionListener( new ActionListener() {
            public void actionPerformed(ActionEvent evt){
                exportData();
            }
        }
        );
        
        JButton btnClear = new JButton("Clear results");
        btnClear.addActionListener( new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                molmaster.gui.editbar.ClearScene.clear();
            }
        });
        cmdPanel.add( btnClear );
        
        JCheckBox chkNeighbors = new JCheckBox( "Select neighbors", selectNeighbors );
        chkNeighbors.addActionListener( new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                selectNeighbors = !selectNeighbors;
            }
        });
        cmdPanel.add( chkNeighbors );
        
        btnColor.addMouseListener( new MouseAdapter() {
            public void mouseClicked( MouseEvent evt ) {
                btnColor.setBackground(
                JColorChooser.showDialog(
                CNDialog.this,
                "Select color for valid atoms",
                btnColor.getBackground() ) );
            }
        });
        
        btnColor.add( new JLabel( "Selected Color" ) );
        btnColor.setBorder( BorderFactory.createBevelBorder( javax.swing.border.BevelBorder.RAISED ) );
        btnColor.setBackground( new java.awt.Color( 0.0f, 1.0f, 0.0f ) );
        cmdPanel.add( btnColor );
        
    }
    
    private void exportData() {
        if (!calcStats()) { return; }
        MoleculeFile mf = (MoleculeFile)fileToLoader.get(selected);
        Collection c = mf.getAtoms();
        Iterator i = c.iterator();
        JFileChooser fc = new JFileChooser();
        fc.setSelectedFile( new File( "cnstats.sav" ) );
        fc.showSaveDialog( CNDialog.this );
        if ( fc.getSelectedFile() != null ) {
            try {
                File out = fc.getSelectedFile();
                FileWriter fOut = new FileWriter( out );
                PrintWriter pOut = new PrintWriter( fOut );
                pOut.println( "Molmaster cna stats: <atom1> <atom2>: <x> <y> <z>" );
                while ( i.hasNext() ) {
                    Atom a = (Atom)i.next();
                    List cnaStat = (List)a.getUserObject();
                    Iterator j = cnaStat.iterator();
                    while ( j.hasNext() ) {
                        Vector4d v = (Vector4d)j.next();
                        String sOut = v.w + " " + a.getID() + ": ";
                        sOut +=  v.x + " " + v.y + " " + v.z;
                        pOut.println( sOut );
                    }
                }
                pOut.close(); fOut.close();
            }catch(Exception e){
                new ErrorDialog( e ).show();
                e.printStackTrace( molmaster.Log.getLogStream() );
            }
        }
    }
    
    public void show() {
        theList.showListDialog( this );
        super.show();
    }
    
    private void initComponents() {//GEN-BEGIN:initComponents
        cmdPanel = new javax.swing.JPanel();
        cmdAdd = new javax.swing.JButton();
        cmdRender = new javax.swing.JButton();
        cmdHelp = new javax.swing.JButton();
        cmdClose = new javax.swing.JButton();
        filterPanel = new javax.swing.JPanel();
        filterScroll = new javax.swing.JScrollPane();
        filterTable = new javax.swing.JTable();

        addWindowListener(new java.awt.event.WindowAdapter() {
            public void windowClosing(java.awt.event.WindowEvent evt) {
                closeDialog(evt);
            }
        });

        cmdPanel.setLayout(new javax.swing.BoxLayout(cmdPanel, javax.swing.BoxLayout.Y_AXIS));

        cmdAdd.setFont(new java.awt.Font("Dialog", 0, 11));
        cmdAdd.setText("Add Filter");
        cmdAdd.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                cmdAddActionPerformed(evt);
            }
        });

        cmdPanel.add(cmdAdd);

        cmdRender.setFont(new java.awt.Font("Dialog", 0, 11));
        cmdRender.setText("Render");
        cmdRender.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                cmdRenderActionPerformed(evt);
            }
        });

        cmdPanel.add(cmdRender);

        cmdHelp.setFont(new java.awt.Font("Dialog", 0, 11));
        cmdHelp.setText("Help");
        cmdHelp.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                cmdHelpActionPerformed(evt);
            }
        });

        cmdPanel.add(cmdHelp);

        cmdClose.setFont(new java.awt.Font("Dialog", 0, 11));
        cmdClose.setText("Close Dialog");
        cmdPanel.add(cmdClose);

        getContentPane().add(cmdPanel, java.awt.BorderLayout.WEST);

        filterPanel.setLayout(new java.awt.BorderLayout());

        filterTable.setModel(new javax.swing.table.DefaultTableModel(
            new Object [][] {
                {new Integer(1), new Integer(5), new Integer(5), new Integer(5)}
            },
            new String [] {
                "count", "# neighbors", "# bonds", "longest path"
            }
        ) {
            Class[] types = new Class [] {
                java.lang.Integer.class, java.lang.Integer.class, java.lang.Integer.class, java.lang.Integer.class
            };

            public Class getColumnClass(int columnIndex) {
                return types [columnIndex];
            }
        });
        filterScroll.setViewportView(filterTable);

        filterPanel.add(filterScroll, java.awt.BorderLayout.CENTER);

        getContentPane().add(filterPanel, java.awt.BorderLayout.CENTER);

        pack();
        java.awt.Dimension screenSize = java.awt.Toolkit.getDefaultToolkit().getScreenSize();
        setSize(new java.awt.Dimension(400, 300));
        setLocation((screenSize.width-400)/2,(screenSize.height-300)/2);
    }//GEN-END:initComponents
    
    private void cmdRenderActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_cmdRenderActionPerformed
        Vector v  = (Vector)((DefaultTableModel)filterTable.getModel()).getDataVector();
        Iterator row = v.iterator();
        ArrayList out = new ArrayList();
        
        while ( row.hasNext() ) {
            Vector thisRow = (Vector)row.next();
            out.add( new Vector4d(
            ((Integer)thisRow.get(0)).intValue(),
            ((Integer)thisRow.get(1)).intValue(),
            ((Integer)thisRow.get(2)).intValue(),
            ((Integer)thisRow.get(3)).intValue()) );
        }
        render( out );
    }//GEN-LAST:event_cmdRenderActionPerformed
    
    
    private void cmdAddActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_cmdAddActionPerformed
        DefaultTableModel model = (DefaultTableModel)filterTable.getModel();
        Vector v = (Vector)model.getDataVector().get( model.getRowCount() - 1 );
        Object[] last = v.toArray();
        
        model.addRow( new Object[] {
            new Integer( ((Integer)last[0]).intValue()),
            new Integer( ((Integer)last[1]).intValue()),
            new Integer( ((Integer)last[2]).intValue()),
            new Integer( ((Integer)last[3]).intValue())
        } );
    }//GEN-LAST:event_cmdAddActionPerformed
    
    private void cmdHelpActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_cmdHelpActionPerformed
        JOptionPane.showMessageDialog(
        molmaster.gui.MainFrame.getActiveWindow(),
        helpMessage, "CN Filtering Help", JOptionPane.INFORMATION_MESSAGE);
    }//GEN-LAST:event_cmdHelpActionPerformed
    
    /** Closes the dialog */
    private void closeDialog(java.awt.event.WindowEvent evt) {//GEN-FIRST:event_closeDialog
        setVisible(false);
        dispose();
    }//GEN-LAST:event_closeDialog
    
    private boolean isValid( Atom a, Collection criteria ) {
        List l = (List)a.getUserObject();
        Iterator j = criteria.iterator();
        while ( j.hasNext() ) {
            Vector4d v = (Vector4d)j.next();
            Vector3d crit = new Vector3d( v.y, v.z, v.w );
            //System.out.println(crit);
            int count = 0;
            if ( l == null ) { continue; }
            Iterator i = l.iterator();
            while ( i.hasNext() ) {
                Vector4d in = (Vector4d)i.next();
                
                if ( crit.x != 0 && crit.x != in.x ) { continue; }
                if ( crit.y != 0 && crit.y != in.y ) { continue; }
                if ( crit.z != 0 && crit.z != in.z ) { continue; }
                
                count++;
            }
            if ( v.x < 0 ) {
                if ( count < -v.x ) { return false; }
            }else {
                if ( count != v.x ) { return false; }
            }
        }
        return true;
    }
    
    private void render( Collection out ) {
        if ( !calcStats() ) { return; }
        MoleculeFile mf = (MoleculeFile)fileToLoader.get(selected);
        Iterator i = mf.getAtoms().iterator();
        System.out.println("Performing validity checking...");
        int count = 0;
        java.awt.Color c = btnColor.getBackground();
        javax.media.j3d.Appearance valid =
        AtomTable.getActive().getAppearance( new Color3f( c ) );
        
        Color3f nearColor = new Color3f(c);
        nearColor.scale( 0.5f );
        javax.media.j3d.Appearance near =
        AtomTable.getActive().getAppearance( nearColor );
        
        while ( i.hasNext() ) {
            Atom a = (Atom)i.next();
            AtomView av = AtomView.atomToView( a );
            if ( isValid(a, out)) {
                count++;
                av.setDefaultAppearance( valid );
                if ( selectNeighbors ) {
                    Iterator j = a.getConnected().iterator();
                    while ( j.hasNext() ) {
                        Atom neighbor = (Atom)j.next();
                        if ( !isValid( neighbor, out ) ) {
                            AtomView nView = AtomView.atomToView( neighbor );
                            nView.setDefaultAppearance( near );
                        }
                    }
                }
            }else{
                av.setAppearance( av.getDefaultAppearance() );
            }
        }
        if ( count == 0 ) {
            JOptionPane.showMessageDialog( this, "No atoms were selected by your filter." );
        }
        System.out.println("Atoms satisfying criteria: " + count );
        gen.clear();
    }
    
    private boolean calcStats() {
        selected = currentFile.getSelectedItem();
        gen = new BondGenerator();
        gen.setVisible( false );
        PeriodicBoundaryCompare pc = new PeriodicBoundaryCompare();
        PeriodicBoundaryCompare.failure = false;
        pc.special = specialBonds;
        gen.setComparisonFunction(pc);
        ArrayList al = new ArrayList();
        al.add( fileToLoader.get(selected)  );
        gen.setFiles( al );
        gen.autoBond();
        MoleculeFile mf = (MoleculeFile)fileToLoader.get( selected );
        if ( selected == null || mf == null ) {
            JOptionPane.showMessageDialog(
            this, "No file was specified for analysis!" );
            return false;
        }
        analyzer = new molmaster.util.CNAnalysis( mf.getAtoms() );
        analyzer.doCalculation();
        
        return true;
    }
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JTable filterTable;
    private javax.swing.JButton cmdRender;
    private javax.swing.JPanel filterPanel;
    private javax.swing.JPanel cmdPanel;
    private javax.swing.JScrollPane filterScroll;
    private javax.swing.JButton cmdClose;
    private javax.swing.JButton cmdHelp;
    private javax.swing.JButton cmdAdd;
    // End of variables declaration//GEN-END:variables
    
}