package molmaster.gui.editbar.analysis;

import molmaster.*;
import molmaster.model.loaders.*;

import java.util.*;
import javax.vecmath.*;

public class PeriodicBoundaryCompare implements molmaster.util.LengthFunction {
    static protected boolean failure = false;
    static protected Map warned = new HashMap();
    
    protected Map special;
    
    public PeriodicBoundaryCompare() { }
    
    private boolean defaultCompare( Atom a, Atom b, double min, double max ) {
        Bond bond = new Bond( a, b, Bond.BOND_TYPE_NORMAL );
        BondSize bs = (BondSize)special.get( bond.getBondTypeID() );
        if ( bs != null ) {
            max = bs.getMaxBondSize();
        }
        if ( bond.getLength() > min && bond.getLength() < max ) {
            return true;
        }
        return false;
    }
    
    public boolean isValid( Atom a, Atom b, double min, double max ) {
        
        if ( a.getID() == b.getID() ) { return false; }
        
        if (warned.containsKey( a.getOwner() ) ) { return defaultCompare(a,b,min,max); }
        if ( !(a.getOwner() instanceof CubeFile) ) {
            molmaster.util.GetText.showHelpDialog(
            molmaster.gui.MainFrame.getActiveWindow(),
            "PeriodicCompare.Error" );
            warned.put( a.getOwner(), "ignoreme" );
            return defaultCompare(a,b,min,max);
        }
        
        Bond bond = new Bond( a, b, Bond.BOND_TYPE_NORMAL );
        BondSize bs = (BondSize)special.get( bond.getBondTypeID() );
        if ( bs != null ) {
            max = bs.getMaxBondSize();
        }
        //Log.writeLog( min + " " + max );
        CubeFile con = (CubeFile)a.getOwner();
        double[] d = con.getBoundingBox();
        Vector3d al = a.getLocation();
        Vector3d bl = b.getLocation();
        
        double dx = Math.abs( bl.x - al.x );
        double x = dx - Math.round( dx / d[0] ) * d[0] ;
        
        double dy = Math.abs(bl.y - al.y);
        double y = dy - Math.round( dy / d[1] ) * d[1];
        
        double dz = Math.abs(bl.z - al.z);
        double z = dz - Math.round( dz / d[2] ) * d[2];
        
        double dist = x * x + y * y + z * z;
        if ( dist > min * min && dist < max * max ) {
            return true;
        }
        return false;
    }
}
