package molmaster.gui.editbar.analysis;

import java.io.*;
import java.util.*;
import javax.swing.*;
import java.awt.Color;
import java.awt.event.*;
import molmaster.util.*;

/** Implements a simple chart plotter.
 *
 * @author  rjpower
 */
public class PlotCanvas extends JLabel implements ComponentListener {
    
    private int[] data;
    private int count;
    private double min, max, res, peak;
    
    private JLabel text = new JLabel();
    private JButton export = new JButton("Export Data");
    private RDSettings settings;
    private boolean interpolate = true;
    private String comment;
    
    java.awt.Color line = new java.awt.Color( 0.6f, 0.8f, 0.8f );
    private static final int buttonW = 100;
    private static final int buttonH = 40;
    
    public PlotCanvas() {
        super("Waiting for data...");
        //text.setEditable( false );
        text.setBorder( javax.swing.BorderFactory.createLineBorder( line ) );
        //text.setOpaque( false );
        text.setBackground( new java.awt.Color( 1.0f, 1.0f, 1.0f, 0.3f ) );
        setLayout( null );
        
        export.addActionListener( new ActionListener() {
            public void actionPerformed( ActionEvent e ){
                JFileChooser f = new JFileChooser();
                f.setSelectedFile( new File( "./chart.sav" ) );
                f.showSaveDialog( molmaster.gui.MainFrame.getActiveWindow() );
                if ( f.getSelectedFile() != null ) {
                    File o = f.getSelectedFile();
                    writeData( o );
                }
                
            }
        });
        
        add( text );
        add( export );
        addComponentListener( this );
    }
    
    public void setComment( String text ) { comment = text; }
    
    private void writeData(File o){
        try {
            FileWriter fOut = new FileWriter( o );
            BufferedWriter bOut = new BufferedWriter( fOut );
            PrintWriter pOut = new PrintWriter( bOut );
            pOut.println("#   molmaster chart data" );
            for ( int i = 0; i < data.length; i++ ){
                double curPos = min + res * i;
                pOut.println( curPos + "\t" + data[i]);
            }
            pOut.close();
            bOut.close();
            fOut.close();
        }catch(Exception e){
            molmaster.Log.writeLog( "Couldn't write data file!" );
            molmaster.Log.writeLog( "Error was:" + e );
        }
    }
    
    public PlotCanvas( int[] in, int count, double min, double max, double res, double peak) {
        this();
        data = in;
        this.count = count;
        this.min = min;
        this.max = max;
        this.res = res;
        if ( this.count == 0 ) { this.count = 1; }
        setDoubleBuffered(true);
    }
    
    public PlotCanvas( int[] in, int count, double min, double max, double res ) {
        this( in,  count,  min,  max, res,  -1.0 );
    }
    
    public void setValues( int[] in, int count, double min, double max, double res ) {
        this.data = in;
        this.count = count;
        this.max = max;
        this.min = min;
        this.res = res;
    }
    
    public void setInterpolate( boolean val ) {interpolate = val;}
    public boolean getInterpolate() { return interpolate; }
    
    static private final int maxRes = 50;
    static private final int maxLine = 10;
    
    public void paintComponent( java.awt.Graphics g ) {
        if ( data == null ) {
            super.paintComponent( g );
            return;
        }
        
        List peaks = new ArrayList();
        
        g.setColor( Color.white );
        g.fillRect(0, 0, getWidth(), getHeight() );
        
        g.setColor( new Color( 0.2f, 0.2f, 0.5f ) );
        int baseX = 50;
        int baseY = getHeight() - 50;
        int lowY = getHeight() - 20;
        int lowX = 10;
        int screenW = getWidth() - baseX;
        int screenH = getHeight() - 50;
        
        double length = ( max - min ) / res;
        
        int lastX = baseX;
        int lastY = baseY;
        int curX = 0;
        int curY = 0;
        
        int xWidth = (int)(screenW / (2 * length) );
        
        if ( length > maxRes ) {
            //  horizontal axis
            for ( int i = 0; i < maxLine; i++ ) {
                int locX = baseX + screenW * i / maxLine;
                int locY = lowY;
                
                double cp = min + (max-min) * i / maxLine;
                g.drawLine( locX, locY, locX, locY - maxLine );
                g.drawString( String.valueOf( cp ), locX, locY - 15 );
            }
        }else {
            for ( double cp = min; cp < max; cp += res ) {
                int locX = baseX + (int)( ((cp - min)/res) * screenW / length);
                int locY = lowY;
                g.drawLine( locX, locY, locX, locY - 10 );
                g.drawString( String.valueOf( cp ), locX, locY - 15 );
            }
        }
        
        //  plot graph
        for ( double cp = min; cp < max; cp += res ) {
            int locX = baseX + (int)( ((cp - min)/res) * screenW / length);
            int locY = lowY;
            
            if ( !interpolate ) {
                int dataPos = (int)( cp / res);
                int val = data[dataPos];
                
                if ( val > peak ) {
                    peaks.add( new Integer( dataPos ) );
                }
                
                curX = locX;
                double s = (double)val/count;
                curY = (int)(baseY - s*screenH);
                
                g.drawLine( curX - xWidth, baseY,  curX - xWidth, curY );
                g.drawLine( curX - xWidth, curY,  curX + xWidth,  curY );
                g.drawLine( curX + xWidth, curY,  curX + xWidth,  baseY );
            }else {
                int dataPos = (int)( (cp-min)/res );
                int val = data[ dataPos ];
                
                if ( val > peak ) { peaks.add( new Integer( dataPos ) ); }
                
                double s = (double)val/count;
                curX = locX;
                curY = (int)(baseY - s*screenH);
                g.drawLine( lastX, lastY, curX, curY );
                lastX = curX; lastY = curY;
            }
        }
        
        //  vertical axis
        for ( int i = 0; i < 10; i++ ) {
            int val = count * i / 10;
            int locX = lowX;
            int locY = baseY - (screenH * i)/10;
            
            g.drawLine( locX, locY, locX + 10,  locY );
            g.drawString( String.valueOf( val ), locX + 15, locY );
        }
        
        if ( comment != null ) {
            text.setText( "<html><p>" + comment + "</p></html>" );
            text.setSize( 160, 30 );
            text.setLocation( 60, 10);
        }
        
        if ( peak > 0 ){
            Iterator it = peaks.iterator();
            String out = "<html><p>Peaks found: </p><p></p>";
            
            while ( it.hasNext() ) {
                int val = ((Integer)it.next()).intValue();
                double loc = min + val * res;
                double height = (float)data[val] / count;
                out = out +
                new PrintfFormat("<p>Location: %.2f, Height: %.2f\n</p>").sprintf(
                new Object[] { new Float( loc ), new Float( height ) } );
                
            }
            out += "</html>";
            
            text.setLocation( getWidth() - 150, 10 );
            text.setSize( 140, 35 + peaks.size() * 17 );
            text.setText( text.getText() + out );
        }
    }
    
    /** Invoked when the component has been made invisible.
     *
     */
    public void componentHidden(ComponentEvent e) {
    }
    
    /** Invoked when the component's position changes.
     *
     */
    public void componentMoved(ComponentEvent e) {
    }
    
    /** Invoked when the component's size changes.
     *
     */
    public void componentResized(ComponentEvent e) {
        export.setSize( buttonW, buttonH );
        export.setLocation( getWidth() - buttonW - 10, 10 );
    }
    
    /** Invoked when the component has been made visible.
     *
     */
    public void componentShown(ComponentEvent e) {
        export.setSize( buttonW, buttonH );
        export.setLocation( getWidth() - buttonW - 10, 10 );
    }
    
}