package molmaster.gui.editbar.analysis;

import java.io.*;
import java.util.*;
import javax.swing.*;
import java.awt.event.*;

import molmaster.*;
import molmaster.util.*;
import molmaster.model.*;
import molmaster.gui.tools.reflect.*;

/** Displays a new dialog for performing radial
 *  distribution function analysis on specified
 *  file(s).
 */
public class RDDialog extends AnalysisDialog implements ActionListener{
    
    protected RDSettings settings = new RDSettings();
    protected ReflectDialog settingsDialog;
    protected Map activeFileData;
    protected PlotCanvas theCanvas = new PlotCanvas();
    protected FileList files = FileList.getDefaultList();
    protected PeriodicBoundaryCompare compare = new PeriodicBoundaryCompare();
    
    private boolean viewBonds = false;
    
    public RDDialog() {
        super();
        initComponents();
        
        displayPanel.add( theCanvas, java.awt.BorderLayout.CENTER );
        settingsDialog = new ReflectDialog(
        molmaster.gui.MainFrame.getActiveWindow(), settings, true );
        files.addListener( this );
        setTitle( "g(R) analysis" );
        cmdPanel.add( btnSize );
        actionPerformed( new ActionEvent( this, 0, "null event" ) );
    }
    
    public void setVisible( boolean b ){
        if ( b ) {
            files.showListDialog( this );
        }
        super.setVisible( b );
    }
    
    public void show() {
        files.showListDialog( this );
        actionPerformed( new ActionEvent( this, 0, "refresh view" ) );
        super.show();
    }
    
    private void initComponents() {//GEN-BEGIN:initComponents
        splitter = new javax.swing.JSplitPane();
        cmdPanel = new javax.swing.JPanel();
        cmdSettings = new javax.swing.JButton();
        cmdHelp = new javax.swing.JButton();
        btnToggleBonds = new javax.swing.JButton();
        displayPanel = new javax.swing.JPanel();
        fileListPanel = new javax.swing.JPanel();
        lblCombo = new javax.swing.JLabel();
        cmbFile = new javax.swing.JComboBox();
        lblConnect = new javax.swing.JLabel();
        cmbConnect = new javax.swing.JComboBox();

        addWindowListener(new java.awt.event.WindowAdapter() {
            public void windowClosing(java.awt.event.WindowEvent evt) {
                closeDialog(evt);
            }
        });

        splitter.setDividerLocation(125);
        cmdPanel.setLayout(new javax.swing.BoxLayout(cmdPanel, javax.swing.BoxLayout.Y_AXIS));

        cmdSettings.setText("Display Settings");
        cmdSettings.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                cmdSettingsActionPerformed(evt);
            }
        });

        cmdPanel.add(cmdSettings);

        cmdHelp.setText("Help");
        cmdHelp.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                cmdHelpActionPerformed(evt);
            }
        });

        cmdPanel.add(cmdHelp);

        btnToggleBonds.setText("Distances");
        btnToggleBonds.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnToggleBondsActionPerformed(evt);
            }
        });

        cmdPanel.add(btnToggleBonds);

        splitter.setLeftComponent(cmdPanel);

        displayPanel.setLayout(new java.awt.BorderLayout());

        fileListPanel.setLayout(new java.awt.GridLayout(2, 2));

        lblCombo.setText("Current File");
        fileListPanel.add(lblCombo);

        cmbFile.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                cmbFileActionPerformed(evt);
            }
        });

        fileListPanel.add(cmbFile);

        lblConnect.setText("Connection");
        fileListPanel.add(lblConnect);

        cmbConnect.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                cmbConnectActionPerformed(evt);
            }
        });

        fileListPanel.add(cmbConnect);

        displayPanel.add(fileListPanel, java.awt.BorderLayout.NORTH);

        splitter.setRightComponent(displayPanel);

        getContentPane().add(splitter, java.awt.BorderLayout.CENTER);

        setSize(new java.awt.Dimension(400, 300));
    }//GEN-END:initComponents
    
    private void btnToggleBondsActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnToggleBondsActionPerformed
        viewBonds = !viewBonds;
        if( viewBonds ){
            btnToggleBonds.setText( "Bond Coordination" );
        }else {
            btnToggleBonds.setText( "Distances" );
        }
        updatePlot();
    }//GEN-LAST:event_btnToggleBondsActionPerformed
    
    private void cmdHelpActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_cmdHelpActionPerformed
        molmaster.util.GetText.showHelpDialog( this, "RDDialog.help" );
    }//GEN-LAST:event_cmdHelpActionPerformed
    
    private void cmbConnectActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_cmbConnectActionPerformed
        updatePlot();
    }//GEN-LAST:event_cmbConnectActionPerformed
    
    private void cmbFileActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_cmbFileActionPerformed
        Object o = cmbFile.getSelectedItem();
        selected = o;
        System.out.println( o );
        Map sum = new HashMap();
        if ( o instanceof File ) {
            MoleculeFile mf = (MoleculeFile)files.getLoaderMap().get( o );
            statFile( mf );
            sum = (Map)mf.getUserData();
        }
        else {
            sum = statAll();
        }
        
        activeFileData = sum;
        Iterator i = sum.keySet().iterator();
        DefaultComboBoxModel cm = new DefaultComboBoxModel();
        while ( i.hasNext() ) {
            cm.addElement(i.next());
        }
        if ( cm.getSize() < 1 ) {
            cm.addElement("No info found.");
        }
        cmbConnect.setModel( cm );
        cmbConnect.setSelectedIndex( 0 );
    }//GEN-LAST:event_cmbFileActionPerformed
    private Map statAll() {
        ComboBoxModel cm = (ComboBoxModel)cmbFile.getModel();
        Map sum = new HashMap();
        for ( int i = 0; i < cm.getSize(); i++ ) {
            Object obj = cm.getElementAt( i );
            if ( obj instanceof File ) {
                MoleculeFile mf = (MoleculeFile)files.getLoaderMap().get(obj);
                statFile( mf );
                Map m = (Map)mf.getUserData();
                Iterator j = m.keySet().iterator();
                while ( j.hasNext() ){
                    Object key = j.next();
                    int[] vals = (int[])m.get(key);
                    int[] cur =  (int[])sum.get( key );
                    if ( cur == null ) {
                        cur = new int[ (int)((settings.max-settings.min)/settings.res) ];
                        sum.put( key, cur );
                    }
                    for ( int k = 0; k < cur.length && k < vals.length; k++ ) {
                        cur[k] += vals[k];
                    }
                }
            }
        }
        return sum;
    }
    
    private void cmdSettingsActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_cmdSettingsActionPerformed
        settingsDialog.show();
        Iterator i = files.getLoaderMap().values().iterator();
        while ( i.hasNext() ) {
            MoleculeFile mf = (MoleculeFile)i.next();
            mf.setUserData( null );
        }
    }//GEN-LAST:event_cmdSettingsActionPerformed
    
    /** Closes the dialog */
    private void closeDialog(java.awt.event.WindowEvent evt) {//GEN-FIRST:event_closeDialog
        setVisible(false);
        dispose();
    }//GEN-LAST:event_closeDialog
    
    private void updatePlot() {
        if ( !isVisible() ) { return; }
        Object o = cmbConnect.getSelectedItem();
        if ( o == null ) { return; }
        
        if ( selected instanceof File ) {
            MoleculeFile mf = (MoleculeFile)files.getLoaderMap().get(selected);
            statFile( mf );
            activeFileData = (Map)mf.getUserData();
        }else {
            activeFileData = statAll();
        }
        
        int[] bins = null;
        String s = o.toString();
        if ( s.length() < 2 ) { return; }
        if ( s.indexOf('-') == -1 ) { return; }
        String start = s.substring(0, s.indexOf('-') );
        String end = s.substring( s.indexOf('-') + 1 );
        
        BondGenerator bg = new BondGenerator(start, end );
        if ( viewBonds && (selected instanceof File) ) {
            ArrayList files = new ArrayList();
            files.add( (MoleculeFile)this.files.getLoaderMap().get(selected));
            bg.setVisible( false );
            bg.setFiles( files );
            bg.setComparisonFunction( compare );
            compare.special = specialBonds;
            bg.genBondSet();
            bins = bg.getCoordination();
        }else {
            bins = (int[])activeFileData.get( o );
        }
        
        int max = 0;
        float avg = 0;
        float tot = 0;
        if ( bins == null ) { return; }
        for ( int i = 0; i < bins.length; i++ ) {
            if ( bins[i] > max ) { max = bins[i]; }
            avg += bins[i] * i;
            tot += bins[i];
        }
        
        avg /= tot;
        
        displayPanel.remove( theCanvas );
        if (viewBonds){
            theCanvas = new PlotCanvas( bins, 
            max * 3 /2,
            bg.getMinCoordination(),
            bg.getMaxCoordination() + 2, 
            1,
            1000 );
        }else {            
            theCanvas = new PlotCanvas( bins,
            max * 3 / 2,
            settings.min,
            settings.max,
            settings.getResolution(),
            settings.getPeakCutoff() );
        }
        theCanvas.setComment( "Average value: " + avg );
        
        displayPanel.add( theCanvas, java.awt.BorderLayout.CENTER );
        System.out.println("Displaying information for: " + o);
        
        //  todo - this is a hack to try and force a repaint...
        setSize( getWidth() + 1,  getHeight() + 1 );
        setSize( getWidth() - 1,  getHeight() - 1 );
        //invalidate();
    }
    
    protected void statFile( MoleculeFile mf ){
        Collection c = mf.getAtoms();
        Iterator i = c.iterator();
        
        javax.vecmath.Vector3d temp = new javax.vecmath.Vector3d();
        
        /** If the values have already been calculated, we
         *  jump out.
         */
        Map m = (Map)mf.getUserData();
        
        if ( m != null && m.size() > 0 ) { return; }
        if ( m == null ) { m = new HashMap(); }
        
        while ( i.hasNext() ) {
            Iterator j = c.iterator();
            Atom curI = (Atom)i.next();
            while ( j.hasNext() ) {
                Atom curJ = (Atom)j.next();
                if ( curI == curJ ) { continue; }
                Bond b = new Bond( curI, curJ, Bond.BOND_TYPE_NORMAL );
                double l = b.getLength();
                if ( l > settings.max ) { continue; }
                int[] bins = (int[])m.get( b.getBondTypeID() );
                if ( bins == null ) {
                    bins = new int[ (int)((settings.max-settings.min)/settings.res) ];
                }
                int binLoc = (int)(l/settings.res);
                if ( binLoc > bins.length - 1 ) { binLoc = bins.length - 1; }
                bins[binLoc] += 1;
                m.put( b.getBondTypeID(), bins );
            }
        }
        
        mf.setUserData( m );
    }
    
    /** Called when a change has occurred with the file list.   */
    public void actionPerformed(java.awt.event.ActionEvent actionEvent) {
        
        Collection c = files.getFiles();
        Iterator i = c.iterator();
        DefaultComboBoxModel cm = new DefaultComboBoxModel();
        
        while ( i.hasNext() ) {
            cm.addElement( i.next() );
        }
        cm.addElement( "All Files" );
        cmbFile.setModel( cm );
        
        if ( cmbFile.getModel().getSize() > 0 )
            cmbFile.setSelectedIndex( 0 );
    }
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton cmdSettings;
    private javax.swing.JLabel lblCombo;
    private javax.swing.JPanel fileListPanel;
    private javax.swing.JPanel cmdPanel;
    private javax.swing.JPanel displayPanel;
    private javax.swing.JButton btnToggleBonds;
    private javax.swing.JButton cmdHelp;
    private javax.swing.JLabel lblConnect;
    private javax.swing.JComboBox cmbFile;
    private javax.swing.JComboBox cmbConnect;
    private javax.swing.JSplitPane splitter;
    // End of variables declaration//GEN-END:variables
    
}