package molmaster.gui.editbar.analysis;

/** Provides an interface to the ring analysis
 *  program written by Xianglong Yuan and A. N. Cormak. 
 *
 *  A dialog is displayed allowing the user to choose
 *  what ring characteristics should be displayed.
 *
 * @author Russell Power
 */
public class RingAnalysis implements molmaster.gui.editbar.EditAction {
    
    static protected RingDialog theDialog;
    
    /** Called when a significant update has occured.  Significant is
     *  defined as 'whatever Russell felt like implementing', but in general
     *  all expected events ( mouse clicks, drags, etc. ) will be reported.
     *
     */
    public void actionPerformed(Object source, molmaster.Event evt) {
        if ( evt.getID() == handlerActivate ) {
            if ( theDialog == null ) {
                theDialog = new RingDialog();
            }
            theDialog.show();
        }
    }
    
    /** Returns the handler group this belongs to - selection handlers might compose
     *  one group, while angle calculations would belong to another.  The groups
     *  will be clustered on the screen to indicate their relation.
     *
     */
    public String getHandlerGroup() {
        return "Analysis";
    }
    
    /** Optional.  Return null if not implemented.  Returns a graphical icon to
     *  display along with the text for this handler.
     */
    public java.awt.Image getHandlerImage() {
        return null;
    }
    
    /** This is displayed to the user on the selection button for this handler.  */
    public String getHandlerName() {
        return "Ring Analysis";
    }
    
    public String getVersion() {
        return "0.1";
    }
    
}
