package molmaster.gui.input;

import java.awt.event.MouseEvent;

import molmaster.*;
import molmaster.gui.*;
import molmaster.model.*;

import javax.vecmath.*;
import javax.media.j3d.*;

/** Attempts to create new atoms in a location on
 *  the screen that appears correct to the user.
 *
 *  Note the clever usage of the word 'attempts' in the
 *  above paragraph.
 *
 * @author  rjpower
 */
public class MouseAdd extends MouseMethod implements IMouseMethod {
    
    public static final int ADD_DEPTH = -10;
    public static final int ADD_WIDTH = -ADD_DEPTH/2;
    
    Canvas3D canvas;
    
    /** Creates a new instance of MouseAdd */
    public MouseAdd(Canvas3D canvas, BranchGroup root, MolUniverse uni ) {
        super( canvas, root, uni);
        this.canvas = canvas;
    }
    
    /** Called by the mouse handler when a mouse
     * event has occurred.  */
    public void processEvent(int x, int y, java.awt.event.MouseEvent e) {
        
        //  we only care about mouse clicks
        if ( e.getID() != MouseEvent.MOUSE_CLICKED && e.getID() != MouseEvent.MOUSE_PRESSED ) {
            return;
        }
        
        if ( pickShape(x,y) == null ) {
            
            System.out.println( "Adding shape..." );
            java.awt.Dimension d = canvas.getSize();
            /*  We need to translate and scale our x,y coordinates..    */
            x -= d.width / 2;
            
            //  The y coordinate system from AWT is opposite that
            //  of java3d's.
            y = d.height - y;
            y -= d.height / 2;
            
            double xShift = (double)x / d.width;
            double yShift = (double)y / d.height;
            
            molmaster.model.AtomTable atoms = molmaster.model.AtomTable.getActive();
            
            Atom a = atoms.getAtom( MainFrame.currentAtom );
            a.setLocation(new Vector3d( xShift * ADD_WIDTH, yShift * ADD_WIDTH, ADD_DEPTH ));
            Transform3D trans = new Transform3D();
            trans.setTranslation( new Vector3d(a.getLocation()) );
            
            Transform3D look = new Transform3D();
            uni.getViewingPlatform().getViewPlatformTransform().getTransform(look);
            look.mul(trans);
            
            Vector3d atomLoc = new Vector3d();
            look.get(atomLoc);
            
            a.setLocation( new Vector3d(atomLoc) );
            
            MolModel.getModel().addAtom( a );
            
            /* TODO - remove AtomView av = new AtomView( a );
            Transform3D look = new Transform3D();
            uni.getViewingPlatform().getViewPlatformTransform().getTransform(look);
            
            Transform3D trans = new Transform3D();
            trans.setTranslation(  );
            
            look.mul(trans);
            
            System.out.println("Inserting...");
            TransformGroup tg = new TransformGroup(look);
            
            tg.setCapability(TransformGroup.ENABLE_PICK_REPORTING);
            tg.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);
            tg.setCapability(TransformGroup.ALLOW_TRANSFORM_READ);
            
            tg.addChild( av );
            
            uni.detach(root);
            root.addChild(tg);
            uni.attach(root);*/
        }
    }
    
    /** Reset any selections currently in progress,
     * and return to default state.
     */
    public void clear() {
    }
    
}
