package molmaster.gui.input;

import java.util.*;
import javax.vecmath.*;
import javax.media.j3d.*;
import java.awt.event.*;

/** Controls what listeners can receive mouse events.  By 
 *  default events are passed to all listeners, however
 *  a particular listener can override this behavior by
 *  using the setExclusive method.
 *
 * @author  rjpower
 */
public class MouseHandler extends PickMouseBehavior implements Listener {
    
    
    static protected MouseHandler activeHandler = null;
    
    protected List handlers = new Vector();
    /** On occasion one handling method may need to ensure
     *  that no other handlers are going to be run while it
     *  is executing - it may accomplish this by requesting to
     *  be set as exclusive.    */
    protected boolean exclusiveMode = false;
    protected IMouseMethod exclusiveHandler;
    
    
    /** Not used. */    
    static public MouseHandler getActiveHandler() { return activeHandler; }
    
    /** Creates a new instance of MouseHandler */
    public MouseHandler( Canvas3D canvas, BranchGroup root ) {
        super( canvas, root, new BoundingSphere( new javax.vecmath.Point3d(), Double.POSITIVE_INFINITY ) );
        setSchedulingBounds( new BoundingSphere(new Point3d(), Double.POSITIVE_INFINITY) );
        KeyWakeupBehavior k = new KeyWakeupBehavior(this);
        k.addKeyCode( KeyEvent.VK_ESCAPE );
        k.addKeyCode( KeyEvent.VK_CANCEL );
        root.addChild( k );
        activeHandler = this;
    }
    
    public void addHandler( IMouseMethod i ) {
        handlers.add( i );
    }
    
    public void removeHandler( IMouseMethod i ) {
        handlers.remove( i );
    }
    
    /** Notifies all listeners of a mouse event at the given input point. */    
    public void updateScene(int x, int y) {
        if (exclusiveMode){ 
            exclusiveHandler.processEvent( x, y, mevent );
            return;
        }
        
        Iterator i = handlers.iterator();
        while ( i.hasNext() ) {
            IMouseMethod m = (IMouseMethod)i.next();
            m.processEvent( x, y, mevent );
        }
    }
    
    public void clearHandlers() {
        Iterator i = handlers.iterator();
        while ( i.hasNext() ) {
            IMouseMethod m = (IMouseMethod)i.next();
            m.clear();
        }
    }
    
    public void wakeup(int keyCode) {
        clearHandlers();
    }
    
    /** Releases the current exclusive listener. */    
    public void releaseExclusive() {
        exclusiveHandler = null;
        exclusiveMode = false;
    }
    /** Sets the given listener object to receive exclusive notifications
     * of mouse events.
     */    
    public boolean setExclusive( IMouseMethod m ) {
        if ( !exclusiveMode ) {
            exclusiveMode = true;
            exclusiveHandler = m;
            return true;
        }
        return false;
    }
    
    public boolean isExclusive() { 
        return exclusiveMode;
    }
    
}
