package molmaster.gui.input;

import molmaster.*;
import molmaster.gui.*;
import molmaster.model.*;

import java.awt.geom.*;

import java.util.*;
import javax.vecmath.*;
import javax.media.j3d.*;
import com.sun.j3d.utils.picking.*;

/**
 *
 * @author  rjpower
 */
public class MouseMethod {
    
    protected PickCanvas pickCanvas;
    protected MolUniverse uni;
    protected BranchGroup root;
    
    
    static public final EventID mouseDragged = new EventID( MouseMethod.class, "Mouse Dragged" );
    static public final EventID mousePressed = new EventID( MouseMethod.class, "Mouse Pressed" );
    static public final EventID mouseReleased = new EventID( MouseMethod.class, "Mouse Released" );
    
    /** Creates a new instance of MouseMethod */
    public MouseMethod(Canvas3D canvas, BranchGroup root, MolUniverse uni) {
        pickCanvas = new PickCanvas( canvas, root );
        pickCanvas.setMode( PickCanvas.GEOMETRY_INTERSECT_INFO );
        pickCanvas.setTolerance( 0.0f );
        this.uni = uni;
        this.root = root;
    }
    
    protected void resetCanvas() {
        //  this is an ugly hack, but it's the easiest way
        //  to reset the canvas...
        pickCanvas = new PickCanvas( pickCanvas.getCanvas(), root );
        pickCanvas.setMode( PickCanvas.GEOMETRY );
        pickCanvas.setTolerance( 0.0f );
    }
    
    public  Node pickShape( int x, int y) {
        resetCanvas();
        
        pickCanvas.setShapeLocation( x, y );
        PickResult[] results = null;
        try {
            results = pickCanvas.pickAllSorted();
        }catch(Exception e){
            /** We tried to grab an unpickable shape... */
            
        }
        if ( results == null )
            return null;
        
        if ( results.length <= 0 )  { return null; }
        
        /** The bonds have a habit of being selected
         *  instead of the atoms - we favor the atoms here. */
        if ( results[0].getObject() instanceof BondView ) {
            if ( results.length > 1 && results[1].getObject() instanceof AtomView ) {
                return results[1].getObject();
            }
        }
        
        return results[0].getObject();
    }
    
    private double pickDepth = 100.0;
    
    public  java.util.List pickArea( int x, int y, int width, int height ) {
        TransformGroup tg = MainFrame.uni.getViewingPlatform().getViewPlatformTransform();
        Transform3D trans = new Transform3D();
        Canvas3D c = pickCanvas.getCanvas();
        
        tg.getTransform( trans );
        Vector3d loc = new Vector3d();
        Vector3d dir;
        Matrix4d mat = new Matrix4d();
        
        trans.get(loc);
        //  The third row of the transform matrix is the unit out vector -
        //  i.e - the vector we want to project our region out from.
        trans.get( mat );
        dir = new Vector3d( mat.m20, mat.m21, mat.m22 );
        
        Point3d closeStart = new Point3d();
        Point3d closeEnd = new Point3d();
        Point3d close = new Point3d();
        
		Log.info("Selecting region from (" + x + "," + y + ") to (" + (x+width) + "," + (y+height) + ")");
        /*
        c.getPixelLocationInImagePlate( x, y+height,  closeStart );
        c.getPixelLocationInImagePlate( x+width, y, closeEnd );
        c.getPixelLocationInImagePlate( x+(width/2), y+(height/2), close );
        
        Vector3d lcDiff = new Vector3d();
        lcDiff.sub( closeStart, close );
        lcDiff.scale( pickDepth );
        
        Vector3d rcDiff = new Vector3d();
        rcDiff.sub( closeEnd, close );
        rcDiff.scale( pickDepth );
        
        Vector3d movement = new Vector3d();
        movement.set( dir );
        movement.scale( pickDepth );
        
        Vector3d far = new Vector3d();
        far.set( close );
        far.add( movement );
        
        Point3d farStart = new Point3d();
        farStart.set( far );
        farStart.add( lcDiff );
        
        Point3d farEnd = new Point3d();
        farEnd.set( far );
        farEnd.add( rcDiff );
        
        
        BoundingBox startBox = new BoundingBox( closeStart, closeEnd );
        BoundingBox endBox = new BoundingBox( farStart,  farEnd );

        */
        /*
        Log.writeLog( "Initial box: " + startBox );
        Log.writeLog( "Box center: " + close );
        Log.writeLog( "Final box: " + endBox );
        Log.writeLog( "Box center" + far );
        
        BoundingPolytope bp = new BoundingPolytope( new Bounds[] { startBox, endBox } );
        SceneGraphPath[] sgp = MainFrame.branchRoot.pickAll( new PickBounds( bp ) );
        if ( sgp == null || sgp.length == 0 ) { return new ArrayList(); }
                for ( int i = 0; i < sgp.length; i++ ) {
            picked.add( sgp[i].getObject().getUserData() );
        }
         
         */
        
        
        //TODO!!!!! HACK!!!!!
        java.util.List picked = new ArrayList();
        Transform3D vWorldTrans = new Transform3D();
        Point3d imagePlate = new Point3d();
        Point2d screenPoint = new Point2d();
        c.getVworldToImagePlate( vWorldTrans );
        Rectangle2D screenRect = new java.awt.geom.Rectangle2D.Double();
        screenRect.setRect( x, y, width, height );
        
        Iterator i = MolModel.getModel().getAtoms().iterator();
        while ( i.hasNext() ) {
            Atom a = (Atom)i.next();
            vWorldTrans.transform( new Point3d(a.getLocation()), imagePlate );
            c.getPixelLocationFromImagePlate( imagePlate,  screenPoint );
            //Log.writeLog ( "Atom Location: " + a.getLocation() );
            //Log.writeLog ( "Image plate: " + imagePlate );
            //Log.writeLog ( "Screen pos: " + screenPoint );
            
            if ( screenRect.contains( screenPoint.x, screenPoint.y ) ){
                picked.add( AtomView.atomToView( a ) );
            }
        }
        
        return picked;
    }
}
