package molmaster.gui.script;

import java.io.*;
import javax.swing.*;

import molmaster.gui.tools.*;
import molmaster.gui.script.editor.*;

import bsh.*;

/**
 *
 * @author  rjpower
 */
public class ScriptDialog extends javax.swing.JDialog {
    
    protected ScriptEditor editor = new ScriptEditor();
    
    /** Creates new form ScriptDialog */
    public ScriptDialog(java.awt.Frame parent, boolean modal) {
        super(parent, modal);
        initComponents();
        getContentPane().add( editor, java.awt.BorderLayout.CENTER  );
		editor.setTokenMarker( new JavaTokenMarker() );
		editor.setCaretVisible(true);
		editor.setCaretBlinkEnabled(true);
    }
    
    private void initComponents() {//GEN-BEGIN:initComponents
        selectPanel = new javax.swing.JPanel();
        btnExecute = new javax.swing.JButton();
        btnSave = new javax.swing.JButton();
        btnLoad = new javax.swing.JButton();

        addWindowListener(new java.awt.event.WindowAdapter() {
            public void windowClosing(java.awt.event.WindowEvent evt) {
                closeDialog(evt);
            }
        });

        btnExecute.setText("Run Script");
        btnExecute.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnExecuteActionPerformed(evt);
            }
        });

        selectPanel.add(btnExecute);

        btnSave.setText("Save Script");
        btnSave.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnSaveActionPerformed(evt);
            }
        });

        selectPanel.add(btnSave);

        btnLoad.setText("Load Script");
        btnLoad.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnLoadActionPerformed(evt);
            }
        });

        selectPanel.add(btnLoad);

        getContentPane().add(selectPanel, java.awt.BorderLayout.SOUTH);

        pack();
        java.awt.Dimension screenSize = java.awt.Toolkit.getDefaultToolkit().getScreenSize();
        setSize(new java.awt.Dimension(459, 393));
        setLocation((screenSize.width-459)/2,(screenSize.height-393)/2);
    }//GEN-END:initComponents

    private void btnExecuteActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnExecuteActionPerformed
        Interpreter interp = new Interpreter();
        try {
            interp.eval("import molmaster.*;");
            interp.eval("import molmaster.model.*;");
            interp.eval("import molmaster.util.*;");
            interp.eval( editor.getText() );
        }catch(Exception e){
            new ErrorDialog( e ).show();   
        }
            
    }//GEN-LAST:event_btnExecuteActionPerformed

    private void btnSaveActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnSaveActionPerformed
        JFileChooser f = new JFileChooser( System.getProperty("user.dir") );
        f.showSaveDialog( molmaster.gui.MainFrame.getActiveWindow() );
        File sOut = f.getSelectedFile();
        if ( sOut == null ) { return; }
        try {
            FileWriter w = new FileWriter( sOut );
            w.write( editor.getText() );
            w.close();
            molmaster.Log.writeLog("Script save successful.");
        }catch(Exception e){
            new molmaster.gui.tools.ErrorDialog( e ).show();
        }
            
    }//GEN-LAST:event_btnSaveActionPerformed
    
    private void btnLoadActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnLoadActionPerformed
        JFileChooser f = new JFileChooser( System.getProperty("user.dir") );
        f.showOpenDialog( molmaster.gui.MainFrame.getActiveWindow() );
        File script = f.getSelectedFile();
        if ( script == null ) { return; }
        try {
            FileReader r = new FileReader( script );
            char scriptText[] = new char[ (int)script.length() ];
            r.read( scriptText );
            r.close();
            editor.setText( new String( scriptText ) );
            System.setProperty( "user.dir",  script.getParent() );
        }catch(Exception e) {
            new molmaster.gui.tools.ErrorDialog( e, true ).show();
        }
    }//GEN-LAST:event_btnLoadActionPerformed
    
    /** Closes the dialog */
    private void closeDialog(java.awt.event.WindowEvent evt) {//GEN-FIRST:event_closeDialog
        setVisible(false);
        dispose();
    }//GEN-LAST:event_closeDialog
    
    /**
     * @param args the command line arguments
     */
    public static void main(String args[]) {
        new ScriptDialog(new javax.swing.JFrame(), true).show();
    }
    
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton btnSave;
    private javax.swing.JButton btnExecute;
    private javax.swing.JPanel selectPanel;
    private javax.swing.JButton btnLoad;
    // End of variables declaration//GEN-END:variables
    
}
