package molmaster.gui.toolbar;

import java.awt.*;
import java.awt.event.*;
import java.util.*;

import javax.swing.*;

import molmaster.gui.*;
import molmaster.util.*;

/** Implements functionality in addition to the default
 *  JToolBar.  Currently this consists of the ability to
 *  persist the state of controls added to the bar.
 *
 * @author  Russell Power
 */
public class ToolBar extends JToolBar implements 
    MouseListener, ComponentListener {
    
    private DataStore controls = new DataStore( "toolbarState" );
    class ComponentSave implements Runnable {
        public void run() {
            Component[] c = getComponents();
            ArrayList al = new ArrayList();
            for ( int i = 0; i < c.length; i++ ){
                al.add( c[i].getName() );
            }
            controls.put( getName(), al );
            controls.flush();
        }
    }
    
    public ToolBar() { super("unnamed bar"); init(); }
    public ToolBar( String name ) { super(name); init(); }
    
    protected void restoreState() {
        ArrayList l = (ArrayList)controls.get( getName() );
        if ( l == null ) {
            return;
        }
        
        marked.clear();
        int cur = 0;
        
        Component[] c = getComponents();
        removeAll();
        
        Iterator i = l.iterator();
        
        while ( i.hasNext() ) {
            String s = (String)i.next();
            int idx = find( s, c);
            if ( idx != -1 ) {
                add( c[idx], cur );
            }
            cur++;
        }
        
        for ( int j = 0; j < c.length; j++ ) {
            if ( !marked.contains( new Integer( j ) ) ) {
                add( c[j], -1 );
            }
        }
    }
    
    private HashSet marked = new HashSet();
    private int find( String s, Component[] c ){
        for ( int i = 0; i < c.length; i++ ) {
            String n = c[i].getName();
            if ( n.equals( s ) && !marked.contains( new Integer(i) ) ) {
                marked.add( new Integer(i) );
                return i;
            }
        }
        return -1;
    }
    
    protected void paintBorder( Graphics g ) {
        g.setColor( Color.black );
        g.drawRect( 0, 0, getWidth() - 2, getHeight() - 2 );
        super.paintBorder( g );
    }
    
    private void init() {
        ShutdownHook.addShutdownListener( new ComponentSave() );
        setBackground( EditButton.defaultColor.get() );
        addMouseListener( this );
        addComponentListener( this );
    }
    
    protected void sizeComponents() {
        java.awt.Component[] c = getComponents();
        boolean isSmall = getSmall();
        for ( int i = 0; i < c.length; i++ ) {
            if ( c[i] instanceof EditButton ) {
                ((EditButton)c[i]).setSmall(isSmall);
            }
        }
    }
    
    protected boolean getSmall() { return getWidth() > getHeight(); }
    
    public void mouseClicked(java.awt.event.MouseEvent mouseEvent) {}
    public void mouseEntered(java.awt.event.MouseEvent mouseEvent) {}
    public void mouseExited(java.awt.event.MouseEvent mouseEvent) {}
    public void mousePressed(java.awt.event.MouseEvent mouseEvent) {}
    public void mouseReleased(java.awt.event.MouseEvent mouseEvent) {
        sizeComponents();
    }
    
    public void componentHidden(java.awt.event.ComponentEvent componentEvent) {sizeComponents();}
    public void componentMoved(java.awt.event.ComponentEvent componentEvent) {sizeComponents();}
    public void componentResized(java.awt.event.ComponentEvent componentEvent) {sizeComponents();}
    public void componentShown(java.awt.event.ComponentEvent componentEvent) {sizeComponents();}
    
}
