package molmaster.gui.tools;

import java.util.*;

import molmaster.model.*;
import molmaster.model.loaders.*;
import molmaster.gui.tools.reflect.*;

/** Implementation of basic animation
 *  functionality, with linear interpolation
 *  between states, and the option to save
 *  state information to disk.
 *
 * @author  rjpower
 */
public class AnimationPanel extends javax.swing.JPanel implements Listener {
    
    
    /** Creates new form AnimationPanel */
    public AnimationPanel() {
        initComponents();
        ((ReflectTable)propTable).addListener( this );
    }
    

    private void initComponents() {//GEN-BEGIN:initComponents
        buttonGroup1 = new javax.swing.ButtonGroup();
        splitter = new javax.swing.JSplitPane();
        propPanel = new javax.swing.JPanel();
        tableScroll = new javax.swing.JScrollPane();
        propTable = new ReflectTable(null);
        commandPanel = new javax.swing.JPanel();
        btnPanel = new javax.swing.JPanel();
        cmdStart = new javax.swing.JButton();
        cmdPause = new javax.swing.JButton();
        cmdEnd = new javax.swing.JButton();
        moviePosScroll = new javax.swing.JScrollBar();
        moviePosition = new javax.swing.JLabel();

        setLayout(new java.awt.BorderLayout());

        addComponentListener(new java.awt.event.ComponentAdapter() {
            public void componentShown(java.awt.event.ComponentEvent evt) {
                formComponentShown(evt);
            }
        });

        splitter.setDividerLocation(200);
        splitter.setOrientation(javax.swing.JSplitPane.VERTICAL_SPLIT);
        propPanel.setLayout(new java.awt.BorderLayout());

        propPanel.setBorder(new javax.swing.border.TitledBorder("Animation Properties"));
        tableScroll.setPreferredSize(new java.awt.Dimension(250, 250));
        tableScroll.setViewportView(propTable);

        propPanel.add(tableScroll, java.awt.BorderLayout.CENTER);

        splitter.setLeftComponent(propPanel);

        commandPanel.setLayout(new java.awt.BorderLayout());

        commandPanel.setMinimumSize(new java.awt.Dimension(208, 10));
        commandPanel.setPreferredSize(new java.awt.Dimension(208, 10));
        cmdStart.setFont(new java.awt.Font("Dialog", 0, 12));
        cmdStart.setText("Start");
        cmdStart.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                cmdStartActionPerformed(evt);
            }
        });

        btnPanel.add(cmdStart);

        cmdPause.setFont(new java.awt.Font("Dialog", 0, 12));
        cmdPause.setText("Pause");
        cmdPause.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                cmdPauseActionPerformed(evt);
            }
        });

        btnPanel.add(cmdPause);

        cmdEnd.setFont(new java.awt.Font("Dialog", 0, 12));
        cmdEnd.setText("Stop");
        cmdEnd.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                cmdEndActionPerformed(evt);
            }
        });

        btnPanel.add(cmdEnd);

        commandPanel.add(btnPanel, java.awt.BorderLayout.SOUTH);

        moviePosScroll.setOrientation(javax.swing.JScrollBar.HORIZONTAL);
        moviePosScroll.addAdjustmentListener(new java.awt.event.AdjustmentListener() {
            public void adjustmentValueChanged(java.awt.event.AdjustmentEvent evt) {
                moviePosScrollAdjustmentValueChanged(evt);
            }
        });

        commandPanel.add(moviePosScroll, java.awt.BorderLayout.NORTH);

        moviePosition.setText("Movie Position: 0/0");
        commandPanel.add(moviePosition, java.awt.BorderLayout.CENTER);

        splitter.setRightComponent(commandPanel);

        add(splitter, java.awt.BorderLayout.CENTER);

    }//GEN-END:initComponents

    private void moviePosScrollAdjustmentValueChanged(java.awt.event.AdjustmentEvent evt) {//GEN-FIRST:event_moviePosScrollAdjustmentValueChanged
        if ( movie != null ){
            int pos = moviePosScroll.getValue();
            movie.setMoviePosition( pos );
        }
    }//GEN-LAST:event_moviePosScrollAdjustmentValueChanged

    private void cmdEndActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_cmdEndActionPerformed
        if ( movie != null ){
            movie.stop();
        }
    }//GEN-LAST:event_cmdEndActionPerformed

    private void cmdPauseActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_cmdPauseActionPerformed
        if ( movie != null ){
            movie.pause();
        }
    }//GEN-LAST:event_cmdPauseActionPerformed
    
    private void formComponentShown(java.awt.event.ComponentEvent evt) {//GEN-FIRST:event_formComponentShown
        //  todo - allow selection of file...
        List l = MolModel.getModel().getFiles();
        Iterator i = l.iterator();
        while ( i.hasNext() ) {
            Object o = i.next();
            if ( o instanceof MovieFile ) {
                try {
                    if ( movie == null ) { 
                        movie = new Movie((MovieFile)o);
                    }
                    else if ( movie != null ) {
                        if ( movie.getFile() == o ) { return; }
                        else { movie = new Movie((MovieFile)o); }
                    }
                    propTable.setModel( new ReflectModel(movie) );
                    moviePosScroll.setMinimum( 0 );
                    moviePosScroll.setMaximum( movie.getFrameCount() + moviePosScroll.getModel().getExtent() );
                    ((ReflectTable)propTable).addListener( this );
                    movie.addListener( this );
                } catch (Exception e){
                    //todo...
                }
                return;
            }
        }
        propTable.setModel( new ReflectModel() );
    }//GEN-LAST:event_formComponentShown
    
    private void cmdStartActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_cmdStartActionPerformed
        if ( movie != null ){
            movie.start();
        }
    }//GEN-LAST:event_cmdStartActionPerformed

    public void wakeUp(Object signaler, int signalType, Object value) {
        if ( signaler instanceof Movie ) {
            Movie m = (Movie)signaler;
            moviePosition.setText("Movie Position: " + m.getMoviePosition() + "/" + m.getFrameCount() );
            moviePosScroll.setValue( m.getMoviePosition() );
        }
        else {
            moviePosScroll.setMaximum( movie.getFrameCount() + moviePosScroll.getModel().getExtent() );
        }
       
    }    
    
    protected Movie movie;
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JScrollPane tableScroll;
    private javax.swing.JLabel moviePosition;
    private javax.swing.JPanel commandPanel;
    private javax.swing.JScrollBar moviePosScroll;
    private javax.swing.JTable propTable;
    private javax.swing.ButtonGroup buttonGroup1;
    private javax.swing.JPanel propPanel;
    private javax.swing.JButton cmdPause;
    private javax.swing.JPanel btnPanel;
    private javax.swing.JButton cmdStart;
    private javax.swing.JSplitPane splitter;
    private javax.swing.JButton cmdEnd;
    // End of variables declaration//GEN-END:variables
    
}
