package molmaster.gui.tools;

import java.io.*;

import java.awt.*;
import java.awt.event.*;

import java.util.*;
import javax.swing.*;
import javax.swing.table.*;

import molmaster.*;
import molmaster.model.*;

/** Represents the current AtomTable in
 *  tabular form.
 *
 * @author  Russell Power
 */
public class AtomEditor extends javax.swing.JDialog {
    
    protected void setUpColorRenderer(JTable table) {
        table.setDefaultRenderer(Color.class, new ColorRenderer());
    }
    
    //Set up the editor for the Color cells.
    private void setUpColorEditor(JTable table) {
        //First, set up the button that brings up the dialog.
        final JButton button = new JButton(""){
            public void setText(String t){}
        };
        button.setBackground(Color.white);
        button.setBorderPainted(false);
        
        //Now create an editor to encapsulate the button, and
        //set it up as the editor for all Color cells.
        final ColorEdit colorEditor = new ColorEdit(button);
        table.setDefaultEditor(Color.class, colorEditor);
        
        //Set up the dialog that the button brings up.
        final JColorChooser colorChooser = new JColorChooser();
        ActionListener okListener = new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                colorEditor.currentColor = colorChooser.getColor();
            }
        };
        final JDialog dialog = JColorChooser.createDialog(button,
        "Pick a Color",
        true,
        colorChooser,
        okListener,
        null);
        
        //Here's the code that brings up the dialog.
        button.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                button.setBackground(colorEditor.currentColor);
                colorChooser.setColor(colorEditor.currentColor);
                dialog.show();
            }
        });
    }
    
    /** Overrides the default java.awt.Color class
     *  and implements a toString() which meshes better
     *  with our serialization.
     */
    class MolColor extends Color {
        MolColor( float r, float g, float b ) { super( r,g,b); }
        MolColor( Color c ) { super( c.getRed(), c.getGreen(), c.getBlue() ); }
        public String toString(){
            return "" + getRed() + "\t" + "\t" + getGreen() + "\t" + getBlue();
        }
    }
    
    /** Creates new form AtomEditor */
    public AtomEditor(java.awt.Frame parent, boolean modal) {
        super(parent, modal);
        initComponents();
        Iterator i = AtomTable.getActive().getAtomAttributes().iterator();
        tblAtoms.setModel(
        new AtomEditorModel(
        new Object[][] {{}},
        new String [] { "Name", "Symbol", "Atomic Number", "Mass", "Van der Waal", "Covalent Radius", "Color"}
        ));
        
        ((AtomEditorModel)tblAtoms.getModel()).clear();
        
        DefaultTableModel model = (DefaultTableModel)tblAtoms.getModel();
        
        while ( i.hasNext() ) {
            AtomAttributes a = (AtomAttributes)i.next();
            model.addRow( new Object[] {
                a.atomName,
                a.atomicSymbol,
                new Integer(a.atomicNumber),
                new Double(a.atomicMass),
                new Double(a.vanDerWaal),
                new Double(a.bondingRadius),
                new MolColor( a.color.get() )} );
        }
        
        setUpColorRenderer( tblAtoms );
        setUpColorEditor( tblAtoms );
        
        tblAtoms.setColumnSelectionAllowed( false );
        MouseAdapter sorter = new MouseAdapter() {
            public void mouseClicked(MouseEvent e) {
                TableColumnModel columnModel = tblAtoms.getColumnModel();
                int viewColumn = columnModel.getColumnIndexAtX(e.getX()); 
                int column = tblAtoms.convertColumnIndexToModel(viewColumn); 
                if (e.getClickCount() == 1 && column != -1) {
                    ((AtomEditorModel)tblAtoms.getModel()).sortCol = column;
                    ((AtomEditorModel)tblAtoms.getModel()).sort();
                    tblAtoms.repaint();
                }
            }
        };
        tblAtoms.getTableHeader().addMouseListener( sorter );
        tblAtoms.getTableHeader().setReorderingAllowed( false );
    }
    
    class ColorRenderer extends JLabel
    implements TableCellRenderer {
        
        public ColorRenderer() {
            super();
            setOpaque(true);
        }
        
        public Component getTableCellRendererComponent(
        JTable table, Object color,
        boolean isSelected, boolean hasFocus,
        int row, int column) {
            setBackground((Color)color);
            return this;
        }
    }
    
    class ColorEdit extends DefaultCellEditor {
        Color currentColor = null;
        
        public ColorEdit(JButton b) {
            super(new JCheckBox());
            editorComponent = b;
            setClickCountToStart(1); //This is usually 1 or 2.
            
            //Must do this so that editing stops when appropriate.
            b.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    fireEditingStopped();
                }
            });
        }
        
        protected void fireEditingStopped() {
            super.fireEditingStopped();
        }
        
        public Object getCellEditorValue() {
            return currentColor;
        }
        
        public Component getTableCellEditorComponent(JTable table,
        Object value,
        boolean isSelected,
        int row,
        int column) {
            ((JButton)editorComponent).setText(value.toString());
            currentColor = (Color)value;
            return editorComponent;
        }
    }
    
    private void initComponents() {//GEN-BEGIN:initComponents
        tableScroll = new javax.swing.JScrollPane();
        tblAtoms = new javax.swing.JTable();
        pnlButtons = new javax.swing.JPanel();
        btnOkay = new javax.swing.JButton();
        btnCancel = new javax.swing.JButton();

        addWindowListener(new java.awt.event.WindowAdapter() {
            public void windowClosing(java.awt.event.WindowEvent evt) {
                closeDialog(evt);
            }
        });

        tableScroll.setViewportView(tblAtoms);

        getContentPane().add(tableScroll, java.awt.BorderLayout.CENTER);

        btnOkay.setText("Okay");
        btnOkay.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnOkayActionPerformed(evt);
            }
        });

        pnlButtons.add(btnOkay);

        btnCancel.setText("Cancel");
        btnCancel.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnCancelActionPerformed(evt);
            }
        });

        pnlButtons.add(btnCancel);

        getContentPane().add(pnlButtons, java.awt.BorderLayout.SOUTH);

        pack();
    }//GEN-END:initComponents
    
    private void btnOkayActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnOkayActionPerformed
        AtomEditorModel m = (AtomEditorModel)tblAtoms.getModel();
        try {
            FileWriter fw = new FileWriter("elements.dat");
            PrintWriter p = new PrintWriter( fw );
            p.println("#");
            for ( int i = 0; i < m.getRowCount(); i++ ){
                Object[] o = m.getRow( i ).toArray();
                for ( int j = 0; j < o.length; j++ ) {
                    if ( o[j] == null ) { break; }
                    p.print( o[j].toString() + "\t" );
                }
                p.print("\n");
            }
            p.close();
            fw.close();
        }catch(Exception e){
            molmaster.Log.writeLog("Error saving to file...");
            e.printStackTrace();
        }
        close();
    }//GEN-LAST:event_btnOkayActionPerformed
    
    private void btnCancelActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnCancelActionPerformed
        close();
    }//GEN-LAST:event_btnCancelActionPerformed
    
    private void close() {
        setVisible( false );
        dispose();
    }
    /** Closes the dialog */
    private void closeDialog(java.awt.event.WindowEvent evt) {//GEN-FIRST:event_closeDialog
        close();
    }//GEN-LAST:event_closeDialog
    
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JScrollPane tableScroll;
    private javax.swing.JButton btnCancel;
    private javax.swing.JPanel pnlButtons;
    private javax.swing.JTable tblAtoms;
    private javax.swing.JButton btnOkay;
    // End of variables declaration//GEN-END:variables
    
}

class AtomEditorModel extends DefaultTableModel {
    
    public AtomEditorModel( Object[][] o, String[] s ) { super(o,s); }
    
    protected int sortCol = 0;
    
    Comparator rowComp = new Comparator() {
        public int compare( Object o1, Object o2 ){
            try {
                Object[] a1 = ((Vector)o1).toArray();
                Object[] a2 = ((Vector)o2).toArray();
                if ( a1[sortCol] instanceof Comparator ) {
                    return ((Comparator)a1[sortCol]).compare( a1[sortCol],  a2[sortCol] );
                }
                return a1[sortCol].toString().compareTo( a2[sortCol].toString() );
            }
            catch(NullPointerException e){
                return 0;
            }
        }
    };
    
    public void addRow( Object[] o ) {
        super.addRow( o );
        sort();
    }
    
    public void sort() { 
        Collections.sort( dataVector, rowComp );
    }
    
    Class[] types = new Class [] {
        String.class,
        String.class,
        Integer.class,
        Double.class,
        Double.class,
        Double.class,
        Color.class
    };
    
    public Class getColumnClass(int columnIndex) {
        return types [columnIndex];
    }
    
    public Vector getRow(int r) { return (Vector)dataVector.get(r); }
    public void clear() { dataVector.clear(); }
}
