package molmaster.gui.tools;

import javax.vecmath.*;

import java.util.*;
import javax.swing.*;

import molmaster.*;
import molmaster.model.*;
import molmaster.gui.tools.reflect.*;
import molmaster.util.*;

/**
 *
 * @author  rjpower
 */
public class BondPanel extends javax.swing.JPanel
implements javax.swing.event.ListSelectionListener, ModelListener {
    
    protected String startAtom = "";
    protected String endAtom = "";
    protected float curLength;
    
    
    /** Creates new form BondPanel */
    public BondPanel() {
        initComponents();
        startAtomList.addListSelectionListener( this );
        endAtomList.addListSelectionListener( this );
        MolModel.getModel().addListener( this );
    }
    
    
    public void valueChanged(javax.swing.event.ListSelectionEvent evt) {
        if ( evt.getSource() == startAtomList ) {
            startAtom = (String)startAtomList.getSelectedValue();
        }
        if ( evt.getSource() == endAtomList ) {
            endAtom = (String)endAtomList.getSelectedValue();
        }
        if ( startAtom != "" && endAtom != "" ) {
            curLength = calcBonds();
        }
        
    }
    
    protected float calcBonds(){
        double sRadius = ((AtomAttributes)AtomTable.getActive().getAtom( startAtom ).getAttributes()).bondingRadius;
        double eRadius = ((AtomAttributes)AtomTable.getActive().getAtom( endAtom ).getAttributes()).bondingRadius;
        
        float total = (float)(sRadius + eRadius);//* Bond.fudge;
        
        BondHolder.setMaxBondSize( total );
        //BondHolder.setMinBondSize( total );
        
        optionTable.repaint();
        return total;
    }
    
    protected void createBonds() {
        BondThread bt = new BondThread(startAtom, endAtom, 
        BondHolder.getMaxBondSize(), BondHolder.getMinBondSize(), BondHolder.getBondColor() );
        new Thread(bt).start();
    }
    
    private void initComponents() {//GEN-BEGIN:initComponents
        jLabel2 = new javax.swing.JLabel();
        jPanel1 = new javax.swing.JPanel();
        atomPanel = new javax.swing.JPanel();
        atomScrollStart = new javax.swing.JScrollPane();
        startAtomList = new javax.swing.JList();
        atomScrollEnd = new javax.swing.JScrollPane();
        endAtomList = new javax.swing.JList();
        jScrollPane1 = new javax.swing.JScrollPane();
        optionTable = new ReflectTable( new BondHolder(), ReflectModel.SHOW_ALL );
        jPanel2 = new javax.swing.JPanel();
        btnCreate = new javax.swing.JButton();
        jButton1 = new javax.swing.JButton();

        setLayout(new java.awt.BorderLayout());

        jLabel2.setText("Generate bonds between:");
        add(jLabel2, java.awt.BorderLayout.NORTH);

        jPanel1.setLayout(new java.awt.BorderLayout());

        atomPanel.setLayout(new java.awt.GridLayout(1, 0));

        atomScrollStart.setPreferredSize(new java.awt.Dimension(159, 131));
        startAtomList.setToolTipText("Select an atom to originate the bond from here");
        startAtomList.setSelectionMode(javax.swing.ListSelectionModel.SINGLE_SELECTION);
        atomScrollStart.setViewportView(startAtomList);

        atomPanel.add(atomScrollStart);

        atomScrollEnd.setPreferredSize(new java.awt.Dimension(159, 131));
        endAtomList.setToolTipText("Select an atom to originate the bond from here");
        endAtomList.setSelectionMode(javax.swing.ListSelectionModel.SINGLE_SELECTION);
        atomScrollEnd.setViewportView(endAtomList);

        atomPanel.add(atomScrollEnd);

        jPanel1.add(atomPanel, java.awt.BorderLayout.CENTER);

        jScrollPane1.setPreferredSize(new java.awt.Dimension(200, 100));
        optionTable.setFont(new java.awt.Font("Dialog", 0, 10));
        optionTable.setMinimumSize(new java.awt.Dimension(200, 40));
        optionTable.setPreferredScrollableViewportSize(new java.awt.Dimension(450, 200));
        optionTable.setPreferredSize(new java.awt.Dimension(300, 100));
        jScrollPane1.setViewportView(optionTable);

        jPanel1.add(jScrollPane1, java.awt.BorderLayout.SOUTH);

        add(jPanel1, java.awt.BorderLayout.CENTER);

        jPanel2.setLayout(new java.awt.FlowLayout(java.awt.FlowLayout.LEFT));

        btnCreate.setText("Create Bonds");
        btnCreate.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnCreateActionPerformed(evt);
            }
        });

        jPanel2.add(btnCreate);

        jButton1.setText("Auto Bond!");
        jButton1.setToolTipText("Automatically generates bonds available for each possible atom type");
        jButton1.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButton1ActionPerformed(evt);
            }
        });

        jPanel2.add(jButton1);

        add(jPanel2, java.awt.BorderLayout.SOUTH);

    }//GEN-END:initComponents
    
    private void jButton1ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButton1ActionPerformed
        BondThread bt = new BondThread(true);
        new Thread(bt).start();
    }//GEN-LAST:event_jButton1ActionPerformed
    
    private void btnCreateActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnCreateActionPerformed
        createBonds();
    }//GEN-LAST:event_btnCreateActionPerformed
    
    public void modelUpdated(MolModel scene, int opType, Object opValue) {
        if (
        !( (opType == ModelListener.ATOM_ADDED) ||
        (opType == ModelListener.FILE_ADDED) ||
        (opType == ModelListener.FILE_REMOVED) ||
        (opType == ModelListener.ATOM_REMOVED)
        )) { return; }
        
        Collection c = molmaster.model.MolModel.getModel().getAtomTypes();
        DefaultListModel m = new DefaultListModel();
        
        ArrayList a = new ArrayList();
        Iterator i = c.iterator();
        while ( i.hasNext() ) {
            Object o = i.next();
            if ( o != null ) {
                a.add( o );
            }
        }
        
        Collections.sort( a );
        
        for ( int j = 0; j < a.size(); j++ ) {
            m.addElement( a.get(j) );
        }
        
        startAtomList.setModel( m );
        endAtomList.setModel( m );
    }
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JScrollPane atomScrollEnd;
    private javax.swing.JButton btnCreate;
    private javax.swing.JPanel jPanel2;
    private javax.swing.JPanel jPanel1;
    private javax.swing.JButton jButton1;
    private javax.swing.JPanel atomPanel;
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JTable optionTable;
    private javax.swing.JList startAtomList;
    private javax.swing.JList endAtomList;
    private javax.swing.JLabel jLabel2;
    private javax.swing.JScrollPane atomScrollStart;
    // End of variables declaration//GEN-END:variables
    
}

class BondThread implements Runnable {
    
    boolean autoBond = false;
    BondGenerator generate;

    BondThread(String startAtom, String endAtom, float maxLength, float minLength, Color3f c) {
        generate = new BondGenerator( minLength, maxLength, startAtom, endAtom, c );
    }
    
    BondThread( boolean dummy ) {
        generate = new BondGenerator();
        autoBond = true;
    }
    
    public void run() {
        if ( !autoBond ) {
            generate.genBondSet();
        }else {
            generate.autoBond();
        }
    }
   
}