package molmaster.gui.tools;

import java.io.*;

import javax.media.j3d.VirtualUniverse;
import javax.swing.*;
import java.util.*;

/** Displays an error dialog to prompt the user that a problem
 *  has occurred in molmaster.
 *
 * @author  Russell Power
 */
public class ErrorDialog extends javax.swing.JDialog {

	String errorString = "Molmaster has encountered an unspecified error:";
	String shortString = "\nA brief description of the error is printed below.";
	String contactString =
		"If you believe this error was due to incorrect operation, please click here to send a report to the author.";
	String criticalError =
		"\n\nAn unexpected application error has occurred.  We recommend you report this error by clicking below.  We will try to repair it as soon as possible.";

	Throwable errorRoot;

	static protected boolean active = false;

	static public boolean isInUse() {
		return active;
	}
	/** Creates new form ErrorWindow */
	private ErrorDialog(java.awt.Frame parent, boolean modal) {
		super(parent, true);
		initComponents();
	}

	/** Creates a new ErrorDialog using the given Throwable as a base.
	 * @param shortError Whether to display a small or detailed dialog.
	 */
	public ErrorDialog(Throwable e, boolean shortError) {
		this(e, e.toString(), shortError);
	}

	/** Constructs a new ErrorDialog using the given Throwable for information.
	 */
	public ErrorDialog(Throwable e) {
		this(e, e.toString(), true);
	}

	public ErrorDialog(Throwable e, String title, boolean shortError) {
		this(molmaster.gui.MainFrame.getActiveWindow(), true);

		setTitle("molmaster - application warning");
		String tOut = errorString;
		if (!shortError) {
			tOut += criticalError;
		} else {
			tOut += shortString;
		}
		txtTitle.setText(tOut);

		exceptionText.setText(e.toString());
		errorRoot = e;
		active = true;
	}

	private void initComponents() { //GEN-BEGIN:initComponents
		reportPanel = new javax.swing.JPanel();
		splitter = new javax.swing.JSplitPane();
		commentPanel = new javax.swing.JPanel();
		commentScroll = new javax.swing.JScrollPane();
		userComment = new javax.swing.JTextArea();
		errorPanel = new javax.swing.JPanel();
		txtTitle = new javax.swing.JTextPane();
		txtScroll = new javax.swing.JScrollPane();
		exceptionText = new javax.swing.JTextArea();
		iconPanel = new javax.swing.JPanel();
		lblErrorIcon = new javax.swing.JLabel();
		btnPanel = new javax.swing.JPanel();
		btnOkay = new javax.swing.JButton();
		btnReport = new javax.swing.JButton();
		btnExit = new javax.swing.JButton();
		btnError = new javax.swing.JButton();

		addWindowListener(new java.awt.event.WindowAdapter() {
			public void windowClosing(java.awt.event.WindowEvent evt) {
				closeDialog(evt);
			}
		});

		reportPanel.setLayout(
			new javax.swing.BoxLayout(
				reportPanel,
				javax.swing.BoxLayout.Y_AXIS));

		splitter.setBorder(
			new javax.swing.border.EmptyBorder(
				new java.awt.Insets(1, 1, 1, 1)));
		splitter.setOrientation(javax.swing.JSplitPane.VERTICAL_SPLIT);
		splitter.setResizeWeight(0.5);
		commentPanel.setLayout(new java.awt.BorderLayout());

		userComment.setLineWrap(true);
		userComment.setText(
			"If you wish, enter a comment about the error here - how it happened, what you were doing...");
		userComment.setWrapStyleWord(true);
		userComment.setBorder(
			new javax.swing.border.EmptyBorder(
				new java.awt.Insets(1, 1, 1, 1)));
		commentScroll.setViewportView(userComment);

		commentPanel.add(commentScroll, java.awt.BorderLayout.CENTER);

		splitter.setBottomComponent(commentPanel);

		errorPanel.setLayout(new java.awt.BorderLayout());

		txtTitle.setBorder(null);
		txtTitle.setEditable(false);
		txtTitle.setFont(new java.awt.Font("Dialog", 0, 11));
		errorPanel.add(txtTitle, java.awt.BorderLayout.NORTH);

		exceptionText.setEditable(false);
		exceptionText.setFont(new java.awt.Font("Dialog", 0, 11));
		exceptionText.setBorder(
			new javax.swing.border.EmptyBorder(
				new java.awt.Insets(1, 6, 1, 1)));
		txtScroll.setViewportView(exceptionText);

		errorPanel.add(txtScroll, java.awt.BorderLayout.CENTER);

		splitter.setTopComponent(errorPanel);

		reportPanel.add(splitter);

		getContentPane().add(reportPanel, java.awt.BorderLayout.CENTER);

		iconPanel.setLayout(new java.awt.BorderLayout());

		iconPanel.setBackground(new java.awt.Color(255, 255, 255));
		lblErrorIcon.setBackground(new java.awt.Color(102, 153, 255));
		lblErrorIcon.setText("            ");
		lblErrorIcon.setOpaque(true);
		iconPanel.add(lblErrorIcon, java.awt.BorderLayout.CENTER);

		getContentPane().add(iconPanel, java.awt.BorderLayout.WEST);

		btnPanel.setBackground(new java.awt.Color(255, 255, 255));
		btnOkay.setText("Okay");
		btnOkay.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				btnOkayActionPerformed(evt);
			}
		});

		btnPanel.add(btnOkay);

		btnReport.setText("Report this Error");
		btnReport.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				btnReportActionPerformed(evt);
			}
		});

		btnPanel.add(btnReport);

		btnExit.setText("Exit MolMaster");
		btnExit.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				btnExitActionPerformed(evt);
			}
		});

		btnPanel.add(btnExit);

		btnError.setText("Error Details >>");
		btnError.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				btnErrorActionPerformed(evt);
			}
		});

		btnPanel.add(btnError);

		getContentPane().add(btnPanel, java.awt.BorderLayout.SOUTH);

		pack();
		java.awt.Dimension screenSize =
			java.awt.Toolkit.getDefaultToolkit().getScreenSize();
		setSize(new java.awt.Dimension(500, 197));
		setLocation(
			(screenSize.width - 500) / 2,
			(screenSize.height - 197) / 2);
	} //GEN-END:initComponents

	private void btnExitActionPerformed(java.awt.event.ActionEvent evt) {
		//GEN-FIRST:event_btnExitActionPerformed
		System.exit(0);
	} //GEN-LAST:event_btnExitActionPerformed

	protected String cancelMessage =
		"Sending error details now.  Click cancel to stop this message from being sent.";
	private void btnReportActionPerformed(java.awt.event.ActionEvent evt) {
		//GEN-FIRST:event_btnReportActionPerformed
		JOptionPane jo =
			new JOptionPane(
				cancelMessage,
				JOptionPane.INFORMATION_MESSAGE,
				0,
				new ImageIcon(),
				new Object[] { "Cancel" },
				"Cancel");
		JDialog jd =
			jo.createDialog(
				molmaster.gui.MainFrame.getActiveWindow(),
				"Sending error report...");

		//if ( !stackTrace ) { btnErrorActionPerformed( evt ); }
		String outText =
			userComment.getText()
				+ "\n\n\n--------------------------------------\n\n"
				+ getLongError();
		MailSender ms =
			new MailSender(
				"mailhost.u.washington.edu",
				"rjpower@u.washington.edu",
				"rjpower@u.washington.edu",
				25,
				outText,
				jd);
		ms.start();
		jd.show();

		if (!ms.finished) {
			ms.abortRun();
			ms.good = false;
		}

		molmaster.Log.writeLog("" + ms.good);
		JOptionPane.showMessageDialog(
			molmaster.gui.MainFrame.getActiveWindow(),
			"Error report was sent "
				+ (ms.good ? "successfully" : "unsuccessfully")
				+ ".");
	} //GEN-LAST:event_btnReportActionPerformed

	protected boolean stackTrace = false;
	protected String hideError = "<< Hide Details";
	protected String showError = "Show Details >>";

	private String getLongError() {
		ByteArrayOutputStream bos = new ByteArrayOutputStream();
		PrintStream ps = new PrintStream(bos);
		errorRoot.printStackTrace(ps);

		ps.print("\n\n");
		ps.print("System Information:\n\n");
		Properties prop = System.getProperties();
		Iterator i = prop.entrySet().iterator();
		while (i.hasNext()) {
			Map.Entry e = (Map.Entry) i.next();
			ps.println(e.getKey() + ": " + e.getValue());
		}
		ps.print("\n\n");
		ps.print("Java3D Information:\n\n");

		i = VirtualUniverse.getProperties().entrySet().iterator();
		while (i.hasNext()) {
			Map.Entry e = (Map.Entry) i.next();
			ps.println(e.getKey() + ": " + e.getValue());
		}
		return bos.toString();
	}
	private void btnErrorActionPerformed(java.awt.event.ActionEvent evt) {
		//GEN-FIRST:event_btnErrorActionPerformed
		String sOut = "";
		if (!stackTrace) {
			btnError.setText(hideError);
			sOut = getLongError();
			setSize(500, 400);
			invalidate();
		} else {
			btnError.setText(showError);
			sOut = errorRoot.toString();
			setSize(500, 150);
			invalidate();
		}

		splitter.setDividerLocation(0.5);
		stackTrace = !stackTrace;
		getLayout().layoutContainer(this);
		exceptionText.setText(sOut);
		//txtScroll.setSize( 400, 200 );

	} //GEN-LAST:event_btnErrorActionPerformed

	private void btnOkayActionPerformed(java.awt.event.ActionEvent evt) {
		//GEN-FIRST:event_btnOkayActionPerformed
		setVisible(false);
		dispose();
		active = false;
	} //GEN-LAST:event_btnOkayActionPerformed

	/** Closes the dialog */
	private void closeDialog(
		java.awt.event.WindowEvent evt) { //GEN-FIRST:event_closeDialog
		setVisible(false);
		dispose();
		active = false;
	} //GEN-LAST:event_closeDialog

	// Variables declaration - do not modify//GEN-BEGIN:variables
	private javax.swing.JButton btnReport;
	private javax.swing.JPanel reportPanel;
	private javax.swing.JPanel commentPanel;
	private javax.swing.JScrollPane txtScroll;
	private javax.swing.JTextArea exceptionText;
	private javax.swing.JPanel errorPanel;
	private javax.swing.JButton btnExit;
	private javax.swing.JTextArea userComment;
	private javax.swing.JButton btnOkay;
	private javax.swing.JButton btnError;
	private javax.swing.JScrollPane commentScroll;
	private javax.swing.JLabel lblErrorIcon;
	private javax.swing.JPanel iconPanel;
	private javax.swing.JPanel btnPanel;
	private javax.swing.JTextPane txtTitle;
	private javax.swing.JSplitPane splitter;
	// End of variables declaration//GEN-END:variables

}
