package molmaster.gui.tools;

import molmaster.*;
import molmaster.gui.*;
import molmaster.model.*;
import molmaster.gui.tools.reflect.*;

import java.awt.Color;
import java.awt.event.*;

import java.util.*;
import javax.swing.*;
import javax.swing.tree.*;
import javax.swing.event.*;
import javax.swing.table.*;

import javax.media.j3d.*;

/** Provides a file based view of all atoms currently
 *  loaded into the scene.
 *
 *  TODO - listen to AtomViews...
 * @author  rjpower
 */
public class FilePanel extends javax.swing.JPanel
implements TreeSelectionListener, Listener{
    
    protected List lastSelected = molmaster.gui.editbar.DefaultAction.selected;
    protected FileTreeModel treeModel = new FileTreeModel(this);
    
    /** Creates new form FilePanel */
    public FilePanel() {
        initComponents();
        fileTree.setModel( treeModel );
        fileTree.addTreeSelectionListener( this );
        fileTree.setExpandsSelectedPaths( true );
        DefaultTreeSelectionModel s = new DefaultTreeSelectionModel();
        s.setSelectionMode( TreeSelectionModel.DISCONTIGUOUS_TREE_SELECTION );
        fileTree.setSelectionModel( s );
        
        final JButton b = new JButton(){ public void setText(String s){} };
        
        final JColorChooser colorChooser = new JColorChooser();
        
        final ColorEditor t = new ColorEditor(b);
        ActionListener okListener = new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                t.currentColor = colorChooser.getColor();
            }
        };
        
        final JDialog dialog = JColorChooser.createDialog(new JButton(),
        "Pick a Color",
        true,
        colorChooser,
        okListener,
        null);
        
        
        b.addActionListener( new ActionListener() {
            public void actionPerformed(ActionEvent e){
                b.setBackground(t.currentColor);
                colorChooser.setColor(t.currentColor);
                dialog.show();
            }
        });
        
        propTable.setDefaultEditor( Object.class, t );
        propTable.setDefaultRenderer( Object.class, new TableRender() );
        
        setDoubleBuffered( true );
        propTable.setDoubleBuffered( true );
        fileTree.setDoubleBuffered( true );
        
        fileTree.addKeyListener(
        new KeyAdapter() {
            public void keyPressed( KeyEvent e ) {
                if ( e.getKeyCode() == KeyEvent.VK_DELETE ) {
                    List l = new ArrayList();
                    TreePath[] tp = fileTree.getSelectionPaths();
                    for ( int i = 0; i < tp.length; i++ ) {
                        l.add( tp[i].getLastPathComponent() );
                    }
                    currentRow = l;
                    removeValues( currentRow );
                }
            }
        } );
    }
    
    private void initComponents() {//GEN-BEGIN:initComponents
        fileTree = new javax.swing.JTree();
        treePopup = new javax.swing.JPopupMenu();
        mnuRemove = new javax.swing.JMenuItem();
        splitter = new javax.swing.JSplitPane();
        fileScroll = new JScrollPane(fileTree);
        propPanel = new javax.swing.JPanel();
        propScroll = new javax.swing.JScrollPane();
        propTable = new javax.swing.JTable();
        applyButton = new javax.swing.JButton();

        fileTree.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                fileTreeMouseClicked(evt);
            }
        });

        treePopup.setLightWeightPopupEnabled(false);
        mnuRemove.setText("Remove");
        mnuRemove.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                mnuRemoveActionPerformed(evt);
            }
        });

        treePopup.add(mnuRemove);

        setLayout(new java.awt.BorderLayout());

        splitter.setDividerLocation(200);
        fileScroll.setToolTipText("Currently open files");
        splitter.setRightComponent(fileScroll);

        propPanel.setLayout(new java.awt.BorderLayout());

        propScroll.setToolTipText("Properties of the currently selected group");
        propScroll.setViewportView(propTable);

        propPanel.add(propScroll, java.awt.BorderLayout.CENTER);

        applyButton.setText("Apply");
        propPanel.add(applyButton, java.awt.BorderLayout.SOUTH);

        splitter.setLeftComponent(propPanel);

        add(splitter, java.awt.BorderLayout.CENTER);

    }//GEN-END:initComponents
    
    private void mnuRemoveActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_mnuRemoveActionPerformed
        removeValues( currentRow );
        propTable.setModel( new DefaultTableModel() );
        fileTree.setSelectionInterval( 0, 0 );
    }//GEN-LAST:event_mnuRemoveActionPerformed
    
    private void fileTreeMouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_fileTreeMouseClicked
        //right click
        if (evt.getButton() == MouseEvent.BUTTON3 ) {
            List l = new ArrayList();
            TreePath[] tp = fileTree.getSelectionPaths();
            if ( tp == null ) { return; }
            for ( int i = 0; i < tp.length; i++ ) {
                l.add( tp[i].getLastPathComponent() );
            }
            currentRow = l;
            treePopup.show( fileTree, evt.getX(), evt.getY() );
        }
    }//GEN-LAST:event_fileTreeMouseClicked
    
    protected void removeValues( Object row ) {
        Object o = row;
        if ( row instanceof DefaultMutableTreeNode ) {
            o = ((DefaultMutableTreeNode)row).getUserObject();
        }
        
        MolModel theModel = MolModel.getModel();
        
        Object title = null;
        if( row instanceof TinyNode ) { title = ((TinyNode)row).getStringObject(); }
        if ( o instanceof Map ) {
            Map m = (Map)o;
            removeValues( m.values() );
        }
        if ( title  instanceof MoleculeFile ) {
            MoleculeFile mf = (MoleculeFile)title;
            mf.removeFromScene( theModel );
            //fileTree.repaint();
        }
        if ( o instanceof Collection ) {
            Iterator i = ((Collection)o).iterator();
            while ( i.hasNext() ) {
                Object n = i.next();
                removeValues( n );
            }
        }
        if ( o instanceof AtomView ) {
            theModel.removeAtom( ((AtomView)o).getAtom() );
        }
        if ( o instanceof BondView ) {
            Bond b = ((BondView)o).getBond();
            theModel.removeBond( b );
        }
    }
    
    private boolean selecting = false;
    private Class selectedType = null;
    
    public void valueChanged(javax.swing.event.TreeSelectionEvent evt) {
        if ( selecting ) { return; }
        
        TreePath[] tp = fileTree.getSelectionPaths();
        
        clearSelected();
        
        if ( tp == null )  { return; }
        for ( int i = 0; i < tp.length; i++ ) {
            addSelection( tp[i] );
        }
        
        propTable.invalidate();
    }
    
    protected void addSelection( TreePath tp ) {
        DefaultMutableTreeNode t = (DefaultMutableTreeNode)tp.getLastPathComponent();
        Object userObj = t.getUserObject();
        Object stringObj = null;
        
        if ( t instanceof TinyNode ) {
            stringObj = ((TinyNode)t).getStringObject();
        }
        if ( userObj instanceof AtomView  ){
            AtomView av = (AtomView)userObj;
            av.select();
            lastSelected.add( av );
            propTable.setModel( new AtomModel( lastSelected ) );
        }
        //  this is a group node, and we select
        //  all atoms underneath it.
        else if( userObj instanceof List ) {
            List v = (List)t.getUserObject();
            if ( v.size() < 1 ) { return; }
            
            boolean isAtom = false;
            selecting = true;
            Iterator i = v.iterator();
            while ( i.hasNext() ) {
                MolView av = (MolView)i.next();
                isAtom = (av instanceof AtomView);
                av.select();
                if ( isAtom ) {
                    lastSelected.add( av );
                }
            }
            selecting = false;
            if ( isAtom ) {
                propTable.setModel( new AtomModel( lastSelected ) );
            }else {
                propTable.setModel( new BondModel( v ) );
            }
        }
        else if( stringObj instanceof MoleculeFile ){
            propTable.setModel( new FileModel( (MoleculeFile)stringObj ));
        }
        else if( userObj instanceof BondView ) {
            BondView av = (BondView)userObj;
            av.select();
            //lastSelected.add( av );
            propTable.setModel( new BondModel( (BondView)userObj ) );
        }
        else {
            propTable.setModel( new DefaultTableModel() );
        }
    }
    
    /** Unselects all atoms and bonds in the scene. */    
    protected void clearSelected() {
        selecting = true;
        molmaster.gui.editbar.DefaultAction.clearSelected();
        selecting = false;
    }
    
    public void wakeUp(Object signaler, int signalType, Object value) {
        if ( selecting ) { return; }
        
        Object o = ((MolView)signaler).getModelObject();
        DefaultMutableTreeNode n = (DefaultMutableTreeNode)treeModel.nodeMap.get( o );
        if ( n == null ) { return; }
        if ((signalType & WatchedObject.SELECTED) != 0 ) {
            selecting = true;
            fileTree.addSelectionPath( new TreePath(treeModel.getPathToRoot( n )));
            selecting = false;
        }
        else if ((signalType & WatchedObject.UNSELECTED) != 0 ) {
            selecting = true;
            fileTree.removeSelectionPath( new TreePath( treeModel.getPathToRoot(n)));
            selecting = false;
        }
        if( signaler instanceof BondView ) {
            propTable.setModel( new BondModel( (BondView)signaler ) );
        }else {
            propTable.setModel( new AtomModel( (AtomView)signaler ) );
        }
        propTable.invalidate();
    }
    
    private Object currentRow;
    class TableRender extends javax.swing.table.DefaultTableCellRenderer{
        Color lastColor;
        int cr, cc;
        TableModel tm;
        
        public java.awt.Component getTableCellRendererComponent(
        JTable t, Object val, boolean sel, boolean foc, int row, int col ){
            cr = row; cc = col;
            tm = t.getModel();
            
            if ( val instanceof Boolean ) {
                return new JCheckBox("", ((Boolean)val).booleanValue() );
            }
            if ( val instanceof Color ) {
                lastColor = (Color)val;
                JLabel l = new JLabel("");
                l.setBackground( (Color)val);
                l.setOpaque( true );
                return l;
            }
            return super.getTableCellRendererComponent(t,val,sel,foc,row,col);
        }
        
    }
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JTree fileTree;
    private javax.swing.JPopupMenu treePopup;
    private javax.swing.JTable propTable;
    private javax.swing.JScrollPane fileScroll;
    private javax.swing.JPanel propPanel;
    private javax.swing.JMenuItem mnuRemove;
    private javax.swing.JScrollPane propScroll;
    private javax.swing.JSplitPane splitter;
    private javax.swing.JButton applyButton;
    // End of variables declaration//GEN-END:variables
    
}

class AtomModel extends DefaultTableModel {
    
    List atoms;
    Transform3D trans = new Transform3D();
    AtomModel( List av ) {
        super(new Object [][] {
            {"Size(in Angstroms)", null},
            {"Color", null},
            {"Material", null},
            {"Visible", null}
        },
        new String [] {
            "Name", "Value"
        });
        
        atoms = av;
        
    }
    
    AtomModel( MolView av ) {
        this( Arrays.asList( new Object[] {av} ) );
    }
    
    public boolean isCellEditable(int row, int col){
        if ( col == 0 ) { return false; }
        return true;
    }
    
    public Object getValueAt( int row, int col) {
        if ( col == 0 ) { return super.getValueAt(row,col); }
        MolView rep = (MolView)atoms.get(0);
        MolObject a = rep.getModelObject();
        
        TransformGroup tg = (TransformGroup)rep.getOwner();
        tg.getTransform( trans );
        
        MolAttributes aa = a.getAttributes();
        javax.vecmath.Color3f c = aa.color;
        
        switch( row ) {
            case 0:
                return new Float( rep.getSize() );
            case 1:
                return new java.awt.Color(c.x, c.y, c.z);
            case 2:
                return "Not Implemented.";
            case 3:
                return new Boolean( rep.isVisible() );
        }
        return "";
    }
    
    public void setValueAt( Object val, int row, int col ) {
        if ( row == 0 ) {
            try {
                Float f = new Float( val.toString() );
                float rad = f.floatValue();
                /*
                MolView rep = (MolView)atoms.get(0);
                MolObject a = rep.getModelObject();//rep.getAtom();
                rad /= a.getAttributes().radius;
                 */
                Iterator i = atoms.iterator();
                
                while ( i.hasNext() ) {
                    MolView av = (MolView)i.next();
                    /*
                    TransformGroup tg = (TransformGroup)av.getOwner();
                    tg.getTransform( trans );
                    if ( av instanceof AtomView ){
                        trans.setScale( rad );
                    }else {
                        Bond b = ((BondView)av).getBond();
                        trans.setScale( new javax.vecmath.Vector3d(rad, b.getLength(), rad) );
                    }
                    tg.setTransform( trans );
                     */
                    av.setSize( rad );
                }
            }catch(Exception e){
                molmaster.Log.writeLog(e);
            }
        }
        if ( row == 1 ) {
            float[] f = ((Color)val).getColorComponents( null );
            
            javax.vecmath.Color3f c = new javax.vecmath.Color3f( f );
            Appearance a = AtomTable.getActive().getAppearance( c );
            
            Iterator i = atoms.iterator();
            while ( i.hasNext() ) {
                MolView av = (MolView)i.next();
                MolObject at = av.getModelObject();
                at.getAttributes().color = c;
                av.setDefaultAppearance( a );
            }
        }
        if ( row == 3 ) {
            Iterator i = atoms.iterator();
            boolean vis = ((Boolean)val).booleanValue();
            
            while ( i.hasNext() ) {
                MolView av = (MolView)i.next();
                av.setVisible( vis );
            }
        }
    }
}

class BondModel extends AtomModel {
    
    BondModel( BondView b ) {
        this( Arrays.asList( new Object[] {b} ) );
    }
    
    BondModel( List l ) {
        super( l );
        this.addRow( new Object[]{ "Hydrogen bond", null } );
    }
    
    
    public boolean isCellEditable(int row, int col){
        if ( col == 0 ) { return false; }
        return true;
    }
    
    public Object getValueAt( int row, int col ){
        if ( col == 0 ) { return super.getValueAt( row, col ); }
        Iterator i = atoms.iterator();
        BondView mv = (BondView)i.next();
        Bond b = mv.getBond();
        if ( row == getRowCount()-1 ) { return new Boolean( b.getType() == Bond.BOND_TYPE_HYDROGEN ); }
        return super.getValueAt( row, col );
    }
    
    public void setValueAt( Object val, int row, int col ) {
        if ( row != getRowCount()-1 ) { super.setValueAt( val, row, col ); return; }
        Iterator i = atoms.iterator();
        boolean res = ((Boolean)val).booleanValue();
        
        while ( i.hasNext() ){
            BondView mv = (BondView)i.next();
            Bond b = mv.getBond();
            if ( res ) { b.setType( Bond.BOND_TYPE_HYDROGEN ); }
            else { b.setType( Bond.BOND_TYPE_NORMAL ); }
            mv.refresh();
        }
    }
}

class FileModel extends ReflectModel {
    MoleculeFile file;
    final int BOUNDING_VISIBLE = 0x00;
    final int FILE_VISIBLE = 0x01;
    FileModel( MoleculeFile m ) {
        super( m ) ;
        insertRow( 0, new Object[] {"Show additional file data", null } );
        insertRow( 1, new Object[] {"Show file", null } );
        file = m;
    }
    public boolean isCellEditable( int row, int col ) {
        if ( col == 0 ) { return false; }
        return true;
    }
    public Object getValueAt( int row, int col ) {
        BranchGroup g =null;
        if ( col != 0 ) {
            switch( row ) {
                case BOUNDING_VISIBLE:
                    g = file.getAdditionalData();
                    return new Boolean( g.isLive() );
                case FILE_VISIBLE:
                    g = (BranchGroup)SceneControl.activeScene.fileMap.get( file );
                    return new Boolean(g.isLive());
            }
        }
        return super.getValueAt( row, col );
    }
    
    public void setValueAt( Object val, int row, int col ) {
        BranchGroup g = file.getAdditionalData();
        BranchGroup fileGroup = (BranchGroup)SceneControl.activeScene.fileMap.get( file );
        
        if ( row == BOUNDING_VISIBLE ) {
            boolean vis = ((Boolean)val).booleanValue();
            
            if ( vis && !g.isLive()) {
                if ( !fileGroup.isLive() ) { return; }
                fileGroup.addChild( g );
            }
            else {
                g.detach();
            }
        }
        else if( row == FILE_VISIBLE ) {
            boolean b = ((Boolean)val).booleanValue();
            if ( b && !fileGroup.isLive() ) {
                SceneControl.activeScene.rootGroup.addChild( fileGroup );
            }
            else if ( !b && fileGroup.isLive() ){
                fileGroup.detach();
            }
        }
        super.setValueAt( val, row, col );
    }
}