/*
 * FileTreeModel.java
 *
 * Created on May 20, 2002, 2:30 PM
 */

package molmaster.gui.tools;

import molmaster.*;
import molmaster.gui.*;
import molmaster.model.*;

import java.util.*;

import javax.swing.tree.*;
/**
 *
 * @author  rjpower
 */

class FileTreeModel extends javax.swing.tree.DefaultTreeModel
implements ModelListener {
    
    /** Used when removing nodes.  Provides a way to easily
     *  find a node given the file it is storing.   */
    protected Map nodeMap = new WeakHashMap();
    protected Map bondMap = new WeakHashMap();
    
    protected FilePanel owner;
    
    public FileTreeModel(FilePanel owner) {
        super(new DefaultMutableTreeNode("Open Files"));
        MolModel.getModel().addListener( this, true);
        this.owner = owner;
    }
    public void modelUpdated(MolModel scene, int opType, Object opValue) {
        try {
            if ( (opType & FILE_ADDED) != 0 ) {
                MoleculeFile file = (MoleculeFile)opValue;
                DefaultMutableTreeNode fileNode;
                if ( nodeMap.containsKey( file ) ) {
                    fileNode = (DefaultMutableTreeNode)nodeMap.get(file);
                }
                else {
                    fileNode = new TinyNode( file );
                }
                
                insertNodeInto( fileNode, (MutableTreeNode)root, 0 );
                System.out.println("Attached file node " + file.toString() + " to tree.");
                
                HashMap typeMap = new HashMap();
                
                Iterator i = file.getAtoms().iterator();
                while ( i.hasNext() ) {
                    
                    Atom a = (Atom)i.next();
                    
                    //  Make sure we only add a value once...
                    if ( nodeMap.containsKey( a ) ) {
                        continue;
                    }
                    
                    String sym = ((AtomAttributes)a.getAttributes()).atomicSymbol;
                    if ( typeMap.containsKey(sym) ){
                        Vector t = (Vector)typeMap.get(sym);
                        t.add( a );
                    }
                    else {
                        Vector t = new Vector();
                        t.add( a );
                        typeMap.put(sym, t );
                    }
                }
                
                i = typeMap.entrySet().iterator();
                while ( i.hasNext() ) {
                    Map.Entry me = (Map.Entry)i.next();
                    Vector obj = (Vector)me.getValue();
                    Vector view = new Vector();
                    String typeName = (String)me.getKey();
                    
                    MutableTreeNode typeNode = new TinyNode( typeName );
                    insertNodeInto( typeNode, fileNode, 0 );
                    
                    Collections.sort( obj, new Atom() );
                    Iterator j = obj.iterator();
                    while ( j.hasNext() ){
                        Atom a = (Atom)j.next();
                        /** Lookup the AtomView from the given Atom.    */
                        AtomView av = AtomView.atomToView( a );
                        
                        //  Let the tree know about this atomview
                        av.addListener( owner );
                        
                        MutableTreeNode avNode = new DefaultMutableTreeNode(av);
                        
                        insertNodeInto( avNode, typeNode, 0 );
                        nodeMap.put( a, avNode );
                        view.add( av );
                    }
                    typeNode.setUserObject( view );
                }
                //fileNode.setUserObject( typeMap );
                nodeMap.put( file, fileNode );
                
            }
            else if ( (opType & FILE_REMOVED) != 0 ) {
                MoleculeFile f = (MoleculeFile)opValue;
                MutableTreeNode n = (MutableTreeNode)nodeMap.get(f);
                if( n != null ) {
                    nodeMap.remove( f );
                    removeNodeFromParent( n );
                }
                
            }
            else if ( (opType & BOND_GROUP_ADDED) != 0 ) {
                Collection l = (Collection)opValue;
                if ( l.size() == 0 ) { return; }
                Iterator i = l.iterator();
                Bond b = (Bond)i.next();
                MoleculeFile owner = b.getOwner();
                DefaultMutableTreeNode bondNode = null;
                DefaultMutableTreeNode fileNode = (DefaultMutableTreeNode)nodeMap.get( owner );
                
                
                Enumeration enum = fileNode.children();
                while ( enum.hasMoreElements() ) {
                    Object o = enum.nextElement();
                    if ( o.toString().equals( "Bonds" ) ) {
                        bondNode = (DefaultMutableTreeNode)o;
                    }
                }
                
                if ( bondNode == null ) {
                    bondNode = new TinyNode("Bonds");
                    bondMap.put( owner, bondNode );
                    insertNodeInto( bondNode, fileNode, fileNode.getChildCount());
                }
                
                Map tempMap = (Map)bondNode.getUserObject();
                if ( tempMap == null ) {
                    tempMap = new HashMap();
                    bondNode.setUserObject( tempMap );
                }
                
                i = l.iterator();
                while ( i.hasNext() ) {
                    b = (Bond)i.next();
                    List current = (List)tempMap.get( b.getBondTypeID() );
                    if ( current == null ) {
                        current = new ArrayList();
                        tempMap.put( b.getBondTypeID(), current );
                    }
                    current.add( BondView.bondToView(b) );
                }
                
                bondNode.removeAllChildren();
                i = tempMap.entrySet().iterator();
                while ( i.hasNext() ) {
                    Map.Entry me = (Map.Entry)i.next();
                    
                    TinyNode t = new TinyNode( me.getKey() );
                    List bonds = (List)me.getValue();
                    List view = new ArrayList();
                    Collections.sort( bonds, new BondView() );
                    Iterator j = bonds.iterator();
                    while ( j.hasNext() ) {
                        BondView bv = (BondView)j.next();
                        DefaultMutableTreeNode bond = new DefaultMutableTreeNode(bv);
                        t.add( bond );
                        nodeMap.put( bv.getBond(), bond );
                    }
                    t.setUserObject( bonds );
                    bondNode.add( t );
                }
                
                bondNode.setUserObject( tempMap );
                
            }
            else if ( (opType & BOND_REMOVED) != 0 || (opType & ATOM_REMOVED) != 0) {
                DefaultMutableTreeNode node = (DefaultMutableTreeNode)nodeMap.get(opValue);
                if ( node == null ) { return; }
                MutableTreeNode parent = (MutableTreeNode)node.getParent();
                removeNodeFromParent( node );
                if ( parent != null && parent.getChildCount() == 0 ) {
                    removeNodeFromParent( parent );
                }
            }
            
            
        }
        catch(Exception e){
            //e.printStackTrace( molmaster.Log.getLogStream() );
            e.printStackTrace();
        }
        
    }
    
    public void removeNodeFromParent( MutableTreeNode child ) {
        try {
            MutableTreeNode parent = (MutableTreeNode)child.getParent();
            super.removeNodeFromParent( child );
            if (parent instanceof TinyNode ) {
                if ( ((TinyNode)parent).getStringObject() instanceof MoleculeFile ) { return; }
            }
            if ( parent.getChildCount() == 0 && parent != root ) {
                removeNodeFromParent( parent );
            }
        } catch( Exception e ){}
        
    }
}


/** Trivial extension to the normal node that allows to have
 *  a different text value then our user object.  */
class TinyNode extends DefaultMutableTreeNode {
    private Object startText;
    TinyNode(Object s) { startText = s; setUserObject(null); }
    public String toString() { return startText.toString(); }
    public Object getStringObject(){ return startText; }
}

