package molmaster.gui.tools.reflect;

import java.lang.reflect.*;
import java.util.*;

import javax.swing.*;

/** Implements a simple range selector for
 *  integer values.
 *
 * @author  Russell Power
 */
public class RangePopup extends javax.swing.JDialog implements PopupEditor {
    
    protected MethodValue mv;
    protected Method rangeMethod;
    private JLabel noSupport = new JLabel("Range editing not supported for this value.");
    
    public RangePopup() {
        super( molmaster.gui.MainFrame.getActiveWindow(), true );
        initComponents();
    }
    
    /** Creates new form RangePopup */
    public RangePopup(MethodValue m) {
        this();
        setValue( m );
    }
    
    private void setValue( MethodValue mv ) {
        this.mv = mv;
        Object o = mv.getOwner();
        propertyName.setText(mv.suffixName);
        RangeValue d;
        
        getContentPane().removeAll();
        
        noSupport.setFont( new java.awt.Font( "Dialog", java.awt.Font.PLAIN, 11 ) );
        setSize( 400, 100 );
        
        try {
            rangeMethod = o.getClass().getMethod(mv.suffixName + "Range", new Class[]{} );
            d = (RangeValue)rangeMethod.invoke( o, new Object[]{} );
            getContentPane().add( propertyName, java.awt.BorderLayout.NORTH );
            getContentPane().add( rangeSlider, java.awt.BorderLayout.SOUTH );
        }catch(Exception e) {
            //molmaster.Log.writeLog( e.toString() );
            getContentPane().add( noSupport );
            return;
        }
        
        rangeSlider.setMinimum( d.min );
        rangeSlider.setMaximum( d.max);
        rangeSlider.setMajorTickSpacing( 10 );
        rangeSlider.setMinorTickSpacing( d.max - d.min / 50 );
        Hashtable h = rangeSlider.createStandardLabels( 10 );
        h.putAll( d.labelTable );
        rangeSlider.setLabelTable( h );
        rangeSlider.setPaintTrack( true );
        rangeSlider.setValue( ((Integer)mv.get()).intValue() );
    }
    private void initComponents() {//GEN-BEGIN:initComponents
        rangeSlider = new javax.swing.JSlider();
        propertyName = new javax.swing.JLabel();

        addWindowListener(new java.awt.event.WindowAdapter() {
            public void windowClosing(java.awt.event.WindowEvent evt) {
                closeDialog(evt);
            }
        });

        rangeSlider.setPaintLabels(true);
        rangeSlider.setPaintTicks(true);
        rangeSlider.setToolTipText("null");
        rangeSlider.addChangeListener(new javax.swing.event.ChangeListener() {
            public void stateChanged(javax.swing.event.ChangeEvent evt) {
                rangeSliderStateChanged(evt);
            }
        });

        getContentPane().add(rangeSlider, java.awt.BorderLayout.SOUTH);

        propertyName.setText("property name");
        getContentPane().add(propertyName, java.awt.BorderLayout.NORTH);

        pack();
    }//GEN-END:initComponents
    
    private void rangeSliderStateChanged(javax.swing.event.ChangeEvent evt) {//GEN-FIRST:event_rangeSliderStateChanged
        mv.set( new Integer( rangeSlider.getValue() ) );
        rangeSlider.setValue( ((Integer)mv.get()).intValue() );
    }//GEN-LAST:event_rangeSliderStateChanged
    
    /** Closes the dialog */
    private void closeDialog(java.awt.event.WindowEvent evt) {//GEN-FIRST:event_closeDialog
        setVisible(false);
        dispose();
    }//GEN-LAST:event_closeDialog
    
    public javax.swing.JDialog getPopup(MethodValue m) {
        //RangePopup d =  new RangePopup( mv );
        setValue( m );
        mv = m;
        return this;
    }
    
    public Class[] getSupportedTypes() {
        return new Class[] { Integer.TYPE, Integer.class };
    }
    
    public Object getValue() {
        return mv.get();
    }
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JSlider rangeSlider;
    private javax.swing.JLabel propertyName;
    // End of variables declaration//GEN-END:variables
    
}
