package molmaster.gui.tools.reflect;

import java.awt.*;
import java.awt.event.*;

import java.util.*;
import javax.swing.*;
import javax.swing.table.*;

/** This table renders the public setter and getter attributes
 *  available from an input object.  It supports the parsing of
 *  common data formats, and allows for custom rendering and editing
 *  of specific properties.  
 *
 *  Static and object methods are differentiated - this is to allow for
 *  easy setting of class level 'constants' without having to view methods
 *  available for a particular object.
 *
 * @author  Russell Power
 */
public class ReflectTable extends javax.swing.JTable {
    
    static Map render = new HashMap();
    
    static {
        render.put( Boolean.TYPE, new BooleanRenderer() );
        render.put( Boolean.class, new BooleanRenderer() );
        render.put( java.io.File.class, new FileRenderer() );
        render.put( Integer.TYPE, new RangePopup() );
        render.put( Integer.class, new RangePopup() );
        render.put( java.awt.Color.class, new ColorEditor() );
        render.put( javax.vecmath.Color3f.class, new ColorEditor() );
    }
    
    static public Object getEditor( Class c ) {
        return render.get(c);
    }
    
    static public Object getRenderer( Class c ) {
        return render.get(c);
    }
    /** Creates a new instance of ReflectTable */
    public ReflectTable(Object o, int methodType ) {
        setModel( new ReflectModel( o, methodType ) );
        setDefaultEditor( Object.class, new Editor() );
        setDefaultRenderer( Object.class, new ReflectRenderer() );
    }
    
    public ReflectTable( Object o ) {
        this( o, ReflectModel.SHOW_INSTANCE );
    }
    public ReflectTable() {this(null);}
    
    public int getRowCount() {
        TableModel m = getModel();
        if ( m == null ) { return 0; }
        return m.getRowCount();
    }
    
    public void addListener( molmaster.model.Listener l ) {
        if ( getModel() == null ) { return; }
        ((ReflectModel)getModel()).addListener( l );
    }
    public void removeListener( molmaster.model.Listener l ) {
        if ( getModel() == null ) { return; }
        ((ReflectModel)getModel()).removeListener( l );
    }
    
    /*
    public void setModel( TableModel model ) {
        if ( !(model instanceof ReflectModel) ) { return; }
        super.setModel( model );
    }*/
}

class ReflectRenderer extends DefaultTableCellRenderer {
    public Component getTableCellRendererComponent(
    JTable tbl,
    Object val,
    boolean sel,
    boolean foc,
    int row,
    int col ) {
        if ( val != null ) {
            Object o = ReflectTable.getRenderer( val.getClass() );
            if ( o != null ) {
                if ( o instanceof ReflectComponent ) {
                    ReflectComponent c = (ReflectComponent)o;
                    //  todo catch exception
                    c.setValue( val, null );
                    return c.getComponent();
                }
                //else if ( o instanceof PopupEditor ) {
                //    return super.getTableCellRendererComponent( tbl, val, sel, foc, row, col );
                //}
            }
        }    
            return super.getTableCellRendererComponent( tbl, val, sel, foc, row, col );
        
    }
}

class Editor extends DefaultCellEditor {
    
    protected ReflectEditor currentComp = null;
    protected Object retVal;
    protected int row, col;
    protected JTable tbl;
    protected JDialog popup;
    protected JTextField textField;
    
    ActionListener defaultListener = new ActionListener(){
        public void actionPerformed(ActionEvent e) {
            fireEditingStopped(textField.getText());
        }
    };
    
    ActionListener customListener = new ActionListener() {
        public void actionPerformed(ActionEvent e) {
            fireEditingStopped( currentComp.getValue() );
        }
    };
    
    ActionListener popupListener = new ActionListener() {
        public void actionPerformed(ActionEvent e) {
            popup.show();
            Object o = currentComp.getValue();
            if ( o == null ) { o = textField.getText(); }
            fireEditingStopped(o);
        }
    };
    
    Editor() {
        super( new JTextField() );
        setClickCountToStart( 1 );
    }
    
    
    public void fireEditingStopped(Object value) {
        retVal = value;
        super.fireEditingStopped();
        tbl.repaint();
    }

    public Object getCellEditorValue() {
        return retVal;
    }
    
    public Component getTableCellEditorComponent( JTable tbl, Object val, boolean sel, int row, int col ) {
        this.tbl = tbl;
        this.row = row;
        this.col = col;
        
        if ( val == null ) { val = "null"; }
        retVal = val;
        
        currentComp = null;
        
        JTextField j = new JTextField( val.toString() );
        j.setFont( new Font( j.getFont().getName(), Font.PLAIN, 11 ) );
        j.setBorder( null );
        j.setBackground( new Color( 0.8f, 0.92f, 1.0f ) );
        j.addActionListener( defaultListener );
        editorComponent = j;
        textField = j;
        
        
        Object o = ReflectTable.getEditor( val.getClass() );
        if ( o instanceof PopupEditor ) {
            JPanel customPanel = new JPanel();
            customPanel.setLayout( new java.awt.BorderLayout() );
            JButton b = new JButton("...");
            b.addActionListener( popupListener );
            customPanel.add( b, java.awt.BorderLayout.EAST );
            customPanel.add( j, java.awt.BorderLayout.CENTER );
            MethodValue m = ((ReflectModel)tbl.getModel()).getMethod( row,  col );
            popup = ((PopupEditor)o).getPopup( m );
            
            editorComponent = customPanel;
            currentComp = (PopupEditor)o;
            
            return customPanel;
            //popup.show();
        }
        else if ( o instanceof ReflectComponent ){
            ReflectComponent c = (ReflectComponent)o;
            
            if ( c != null ) {
                currentComp = c;
                c.setValue( val, customListener );
                editorComponent = c.getComponent();
            }
            //return editorComponent;
        }
        
        return super.getTableCellEditorComponent( tbl, val, sel, row, col );
    }
}
