package molmaster.model;

import java.util.*;
import javax.swing.*;

/** This dialog is shown when the loader type cannot be
 *  automatically determined by molmaster.  In this case,
 *  it needs to ask the user which loader type to use.
 *
 * @author Russell Power
 */
public class MultiLoader extends javax.swing.JDialog {
    
    class ClassView {
        Class dlg;
        
        ClassView( Class c ) { dlg = c; }
        /** String indication of the given loader type.
         */        
        public String toString() {
            String s = dlg.toString();
            int idx = s.lastIndexOf( '.' );
            return s.substring( idx + 1 );
        }
    }
    
    ClassView outClass;
    
    /** Creates new form MultiLoader */
    public MultiLoader(java.awt.Frame parent, boolean modal) {
        super(parent, modal);
        initComponents();
        DefaultListModel m = new DefaultListModel();
        loaderList.setModel(m);
        Iterator i = MoleculeFile.loaderTypes.values().iterator();
        while ( i.hasNext() ) {
            m.addElement( new ClassView((Class)i.next()) );
        }
    }

    private void initComponents() {//GEN-BEGIN:initComponents
        jLabel1 = new javax.swing.JLabel();
        btnOkay = new javax.swing.JButton();
        loaderScroll = new javax.swing.JScrollPane();
        loaderList = new javax.swing.JList();
        txtTitle = new javax.swing.JTextArea();
        btnCancel = new javax.swing.JButton();

        jLabel1.setText("jLabel1");

        getContentPane().setLayout(null);

        setBackground(java.awt.Color.white);
        addWindowListener(new java.awt.event.WindowAdapter() {
            public void windowClosing(java.awt.event.WindowEvent evt) {
                closeDialog(evt);
            }
        });

        btnOkay.setText("Okay");
        btnOkay.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnOkayActionPerformed(evt);
            }
        });

        getContentPane().add(btnOkay);
        btnOkay.setBounds(160, 210, 90, 30);

        loaderList.setSelectionMode(javax.swing.ListSelectionModel.SINGLE_SELECTION);
        loaderList.addListSelectionListener(new javax.swing.event.ListSelectionListener() {
            public void valueChanged(javax.swing.event.ListSelectionEvent evt) {
                loaderListValueChanged(evt);
            }
        });

        loaderList.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                loaderListMouseClicked(evt);
            }
        });

        loaderScroll.setViewportView(loaderList);

        getContentPane().add(loaderScroll);
        loaderScroll.setBounds(130, 10, 230, 190);

        txtTitle.setBackground(new java.awt.Color(153, 204, 255));
        txtTitle.setEditable(false);
        txtTitle.setFont(new java.awt.Font("Dialog", 0, 14));
        txtTitle.setLineWrap(true);
        txtTitle.setText("Molmaster could not automatically locate a loader - please select the loader to use:");
        txtTitle.setWrapStyleWord(true);
        txtTitle.setMinimumSize(new java.awt.Dimension(20, 19));
        getContentPane().add(txtTitle);
        txtTitle.setBounds(0, 0, 120, 260);

        btnCancel.setText("Cancel");
        btnCancel.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnCancelActionPerformed(evt);
            }
        });

        getContentPane().add(btnCancel);
        btnCancel.setBounds(250, 210, 90, 30);

        pack();
        java.awt.Dimension screenSize = java.awt.Toolkit.getDefaultToolkit().getScreenSize();
        setSize(new java.awt.Dimension(390, 298));
        setLocation((screenSize.width-390)/2,(screenSize.height-298)/2);
    }//GEN-END:initComponents

    private void btnCancelActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnCancelActionPerformed
        outClass = null;
        setVisible( false );
        dispose();
    }//GEN-LAST:event_btnCancelActionPerformed

    private void loaderListMouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_loaderListMouseClicked
        Object o = loaderList.getSelectedValue();
        if ( o == null ) { return; }
        if ( evt.getClickCount() == 2 ) 
            outClass = (ClassView)o;
        setVisible( false );
        dispose();
    }//GEN-LAST:event_loaderListMouseClicked

    private void btnOkayActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnOkayActionPerformed
        setVisible(false);
        dispose();
    }//GEN-LAST:event_btnOkayActionPerformed

    private void loaderListValueChanged(javax.swing.event.ListSelectionEvent evt) {//GEN-FIRST:event_loaderListValueChanged
        outClass = (ClassView)loaderList.getSelectedValue();
    }//GEN-LAST:event_loaderListValueChanged
    
    /** Closes the dialog */
    private void closeDialog(java.awt.event.WindowEvent evt) {//GEN-FIRST:event_closeDialog
        outClass = null;
        setVisible(false);
        dispose();
    }//GEN-LAST:event_closeDialog
    
    /** Returns the type of loader the user selected.
     */    
    public Class getLoader() { 
        if ( outClass == null ) { return null; }
        return outClass.dlg; 
    }
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton btnCancel;
    private javax.swing.JList loaderList;
    private javax.swing.JButton btnOkay;
    private javax.swing.JTextArea txtTitle;
    private javax.swing.JLabel jLabel1;
    private javax.swing.JScrollPane loaderScroll;
    // End of variables declaration//GEN-END:variables
    
}
