package molmaster.model.loaders;

import molmaster.*;
import java.io.*;

/** Loads electron charge density information from a VASP type
 *  output file.  Also loads information regarding atom positions
 *  like the default vasp loader.
 *
 * @author  Russell Power
 */
public class ChgCarLoader extends VaspLoader implements FieldFile {

	protected double[][][] field;
	protected double min = Double.POSITIVE_INFINITY;
	protected double max = Double.NEGATIVE_INFINITY;

	/** Creates a new instance of ChgCarLoader */
	public ChgCarLoader() {
	}

	public void loadFromFile(String fileName) throws java.io.IOException {
		diskFile = new File(fileName);
		holdOpen = true;
		super.loadFromFile(fileName);
		//        skipLine( tok );

		tok.nextToken();
		int l = (int) tok.nval;
		tok.nextToken();
		int w = (int) tok.nval;
		tok.nextToken();
		int h = (int) tok.nval;

		//skipLine( tok );
		//while ( r.read() != '\n' ) {}
		//r.read();

		tok.eolIsSignificant(false);
		Log.writeLog(
			"Reading field coordinate array of size: " + l + "x" + w + "x" + h);
		molmaster.gui.MainFrame.setStatusMessage(
			"Reading field coordinates...");
		field = new double[l][w][h];

		int count = 0;
		int total = l * w * h;

		StringBuffer sb = new StringBuffer();
		for (int k = 0; k < h; k++) {
			for (int j = 0; j < w; j++) {
				for (int i = 0; i < l; i++) {
					sb.setLength(0);
					int next = 0;
					while ( (next = r.read()) != -1 ) {
						if ( next != '\r' && next != '\n' ) {
							sb.append((char)next);
							if ( sb.length() == 8 )
								break;
						}
					}
					//double num = getFixed(r);//getNumber( r, tok.nval );
					double num = Double.parseDouble(sb.toString());
					field[i][j][k] = num;

					if (num > max) {
						max = num;
					}
					if (num < min) {
						min = num;
					}
				}
			}
			molmaster.gui.MainFrame.setProgressMeter(k * 100 / h);
		}

		Log.writeLog("" + max);
		Log.writeLog("" + min);

		Log.writeLog("Finished reading array.");
	}

	/** Save the current contents of this resource to
	 * the filename specified.  */
	public void saveToFile(String fileName) throws java.io.IOException {
	}

	/** Returns the field associated with this file.
	 */
	public double[][][] getField() {
		return field;
	}

	/** Returns the basis vector for this field.  This is the set of value by which
	 * which field indices are multiplied to find the position of the field value
	 * in space.
	 */
	public javax.vecmath.Vector3d[] getBasis() {
		return basis;
	}

	/** Returns the maximum value found in this field.
	 */
	public double getFieldMax() {
		return max;
	}

	/** Returns the minimum value found in this field.
	 */
	public double getFieldMin() {
		return min;
	}

}
