package molmaster.model.loaders;

import java.io.*;
import java.util.*;
import javax.swing.*;
import javax.vecmath.*;
import javax.media.j3d.*;

import molmaster.*;
import molmaster.gui.*;
import molmaster.model.*;

/** Reads and parses the molecular configuration
 *  (.con) file format, and renders the information
 *  found therein onto the screen.
 *
 * @author  rjpower
 */
public class ConLoader extends MoleculeFile implements CubeFile {
    
    double seed;
    int numTypes;
    double[] viewBox = new double[3];
    
    Vector atomCount = new Vector();
    Vector masses = new Vector() ;
    
    /** Creates a new instance of ConLoader */
    public ConLoader(String fileName) {
        myName = fileName;
    }
    
    public ConLoader() {}
    
    public void loadFromFile(String fileName) throws java.io.IOException {
        diskFile = new File( fileName );
        rootGroup.setCapability( BranchGroup.ALLOW_DETACH );
        rootGroup.clearCapabilityIsFrequent( BranchGroup.ALLOW_DETACH );
        
        myName = new File( fileName ).getName();
        BufferedReader br = new BufferedReader( new FileReader( new File(fileName)) );
        //StreamTokenizer tok = new StreamTokenizer( br );
        MainFrame.setStatusMessage( "Loading file..." );
        MainFrame.setProgressMeter(0);
        
        //tok.nextToken();
        //br.readLine();
        seed = this.getFixed( br );//getNumber(tok);
        br.readLine();
        br.readLine();
        //skipLine( tok );    //  rest of seed line
        //skipLine( tok );    //  time line
        
        //tok.nextToken(); viewBox[0] = getNumber(tok);
        viewBox[0] = getFixed( br );
        viewBox[1] = getFixed( br );
        viewBox[2] = getFixed( br );
        
        br.readLine();
        br.readLine();
        br.readLine();
        //skipLine(tok);          // line 3  skip comment
        //skipLine(tok);          // line 4  skip alpha, beta, gamma
        //skipLine(tok);          // line 5  skip pressure
        //tok.nextToken();
        
        br.readLine();
        //skipLine(tok);
        
        //tok.nextToken();
        numTypes = (int)getFixed( br ); //getNumber(tok);
        //skipLine(tok);
        br.readLine();
        
        int totalAtoms = 0;
        for(int i = 0; i < numTypes; i++) {
            //tok.nextToken();
            int next = (int)getFixed(br);
            atomCount.add( new Integer(next) );
            totalAtoms += next;
        }
        
        if ( totalAtoms < 1 ) {
            throw new IOException( "File is invalid..." );
        }
        //br.readLine();
        //skipLine(tok);
        
        for(int i = 0; i < numTypes; i++) {
            //tok.nextToken();
            masses.add( new Double( getFixed(br) ) );
        }
        
        //skipLine(tok);
        //String line = br.readLine();
        
        int loadCount = 0;
        
        for ( int i = 0; i < numTypes; i++ ) {
            //tok.nextToken();
            String currentName = getString( br );
            
            //skipLine(tok);  //  comment line
            //skipLine(tok);  //  comment line
            br.readLine();
            br.readLine();
            
            double x, y, z;
            int refNum;
            int id;
            
            int count = ((Integer)atomCount.get(i)).intValue();
            refNum = i;
            for (int j = 0; j < count; j++ ){
                
                x = getFixed(br);
                y = getFixed(br);
                z = getFixed(br);
                
                //  tok.nextToken(); x = tok.nval...
                
                //System.out.println( loadCount );
                //System.out.println(x);
                
                //System.out.println(x);
                
                //tok.nextToken(); refNum = (int)getNumber(tok);
                //tok.nextToken(); id = (int)getNumber(tok);
                
                //skipLine( tok );
                br.readLine();
                
                String atomName = currentName;//(String)refNumToName.get( new Integer(refNum) );
                Atom a = AtomTable.getActive().getAtom( atomName );
                a.setID( loadCount++ );
                a.setLocation( new Vector3d( x,y,z ) );
                a.setOwner( this );
                
                atoms.add( a );
                if ( loadCount % 10 == 0 ){
                    MainFrame.setProgressMeter( (loadCount * 100)/ totalAtoms );
                }
                
            }
        }
        
        Log.writeLog( "Loaded " + loadCount + " atoms from file." );
        
        float sx = (float)viewBox[0]/2; float sy = (float)viewBox[1]/2; float sz = (float)viewBox[2]/2;
        makeBox( sx, sy, sz );
    }
    
    public void saveToFile(String fileName) throws java.io.IOException{
        BufferedWriter w = new BufferedWriter( new FileWriter( new File( fileName ) ) );
        Collection bonds = MolModel.getModel().getBonds();
        if ( bonds.size() != 0 ) {
            JOptionPane.showMessageDialog(
            molmaster.gui.MainFrame.getActiveWindow(),
            "This file format cannot save bonds, but the current scene contains them.\n\nNo bonds will be stored in the specified file.",
            "Save Warning", JOptionPane.WARNING_MESSAGE );
        }
        
        w.write( "0\tRandom Seed\n" );
        w.write( "0\tTime\n" );
        
        BoundingBox box = new BoundingBox( MainFrame.root.getBounds() );
        Point3d low = new Point3d();
        Point3d high = new Point3d();
        box.getLower( low );
        box.getUpper( high );
        high.sub( low );
        high.absolute();
        w.write( high.x + "\t" + high.y + "\t" + high.z + "\t\n" );
        w.write( "90\t90\t0\t alpha,beta,gamma\n" );
        w.write( "0\t0\t Pmass, Volv\n");
        w.write( "0\t0\t0\n" );
        Collection types = MolModel.getModel().getAtomTypes();
        w.write( types.size() + "\n" );
        List allTypes = new ArrayList();
        Iterator i = types.iterator();
        while ( i.hasNext() ) {
            allTypes.add( MolModel.getModel().getAtomsOfType( (String)i.next() ) );
        }
        i = allTypes.iterator();
        while ( i.hasNext() ) {
            Atom[] a = (Atom[])i.next();
            w.write(a.length + " ");
        }
        w.write( "\n" );
        i = allTypes.iterator();
        while ( i.hasNext() ) {
            i.next();
            w.write( "1.0\t" );
        }
        w.write( "\n" );
        
        
        int count = 0;
        i = allTypes.iterator();
        while ( i.hasNext() ) {
            Atom[] atoms = (Atom[])i.next();
            String type = ((AtomAttributes)atoms[0].getAttributes()).atomName;
            w.write( type + "\n" );
            w.write( "Coordinates of type: " + type + "\n" );
            Log.writeLog( "Writing " + atoms.length + " atoms of type " + type + " to file." );
            for ( int j = 0; j < atoms.length; j++ ) {
                Vector3d loc = atoms[j].getLocation();
                w.write( loc.x + "\t\t" + loc.y + "\t\t" + loc.z + "\t\t" + 0 + "\t\t" + count++ );
                w.write( "\n" );
            }
        }
        
        w.close();
        
    }
    
    /*
    public javax.media.j3d.BranchGroup getAdditionalData() {
        return rootGroup;
    }*/
    
    protected boolean boxCentered = false;
    
    /** Returns an array describing the length, width, and height respectively,
     * of the bounding box.
     */    
    public double[] getBoundingBox() {
        return viewBox;
    }
    /** Sets the position of the bounding box loaded in with this file to be centered
     * about the middle of the loaded box coordinates ( rather than the origin ).
     */    
    public void setBoundingBoxCentered( boolean val ) {
        boxCentered = val;
        
        Transform3D trans = new Transform3D();
        boxTrans.getTransform( trans );
        if ( val ) {
            trans.setTranslation( new Vector3d( viewBox[0]/2, viewBox[1]/2, viewBox[2]/2 ) );
        }
        else {
            trans.setTranslation( new Vector3d( 0, 0, 0 )  );
        }
        boxTrans.setTransform( trans );
    }
    
    /** Returns whether the bounding box is centered.
     */    
    public boolean getBoundingBoxCentered() { return boxCentered; }
    
}
