package molmaster.model.loaders;

import javax.vecmath.*;

import java.io.*;
import molmaster.model.*;

/** Loads in data generated by charge density analysis.
 *
 * @author  Russell Power
 */
public class MaxRhoLoader extends MoleculeFile implements FieldFile {

	private double fieldMin = Double.MAX_VALUE;
	private double fieldMax = Double.MIN_VALUE;
	private double[][][] field;
	private Vector3d[] basis =
		new Vector3d[] {
			new Vector3d(1, 0, 0),
			new Vector3d(0, 1, 0),
			new Vector3d(0, 0, 1)};

	/** Creates a new instance of CNLoader */
	public MaxRhoLoader() {
	}

	/** Returns the basis vectors used to obtain field locations.    */
	public javax.vecmath.Vector3d[] getBasis() {
		return basis;
	}

	/** Returns a 3d array representing the field values
	 *  at points in space.  Each index position is multiplied
	 *  by the basis vector to obtain the field location in space.   */
	public double[][][] getField() {
		return field;
	}
	
	public void setField(double[][][] f) {
		field = f;
	}

	/** Maximum value taken by this field.     */
	public double getFieldMax() {
		return fieldMax;
	}

	/** Returns the minimum field value taken on by this field.  */
	public double getFieldMin() {
		return fieldMin;
	}

	public void loadFromFile(String fileName) throws java.io.IOException {
		diskFile = new File(fileName);
		int lx, ly, lz;
		myName = fileName;
		FileReader fr = new FileReader(fileName);
		BufferedReader buf = new BufferedReader(fr);
		StreamTokenizer tok = new StreamTokenizer(buf);
		tok.nextToken();
		lx = (int) tok.nval;
		tok.nextToken();
		ly = (int) tok.nval;
		tok.nextToken();
		lz = (int) tok.nval;
		molmaster.Log.info(
			"Loading array of dimension: " + lz + "," + ly + "," + lx);
		field = new double[lz][ly][lx];

		int count = 0;

		for (int k = 0; k < lx; k++) {
			for (int j = 0; j < ly; j++) {
				for (int i = 0; i < lz; i++) {
					tok.nextToken();
					field[i][j][k] = tok.nval;
					if (field[i][j][k] > fieldMax) {
						fieldMax = field[i][j][k];
					}
					if (field[i][j][k] < fieldMin) {
						fieldMin = field[i][j][k];
					}
				}
			}
			molmaster.gui.MainFrame.setProgressMeter(k * 100 / lz);
		}
		molmaster.gui.MainFrame.setProgressMeter(100);
	}

	/** Save the current contents of this resource to
	 *  the filename specified.  */
	public void saveToFile(String fileName) throws java.io.IOException {
	}

	/**
	 * @see molmaster.model.loaders.FieldFile#setBasis(javax.vecmath.Vector3d[])
	 */
	public void setBasis(Vector3d[] v) {
		basis = v;
	}

}
