package molmaster.model.loaders;

import molmaster.*;
import molmaster.model.*;

import java.io.*;
import javax.vecmath.*;

/** Loads NIST webbook .mol files.
 *
 * @author  rjpower
 */
public class MolLoader extends MoleculeFile {
    
    /** Creates a new instance of MolLoader */
    public MolLoader() {
    }
    
    public void loadFromFile(String fileName) throws java.io.IOException {
        diskFile = new File( fileName );
        myName = fileName;
        BufferedReader r = new BufferedReader( new FileReader( new File( fileName ) ) );
        StreamTokenizer tok = new StreamTokenizer( r );
        skipLine( tok );
        skipLine( tok );
        skipLine( tok );
        
        tok.nextToken(); int numAtoms = (int)tok.nval;
        tok.nextToken(); int numBonds = (int)tok.nval;
        skipLine( tok );
        
        double x,y,z;
        String atomName;
        Atom[] atoms = new Atom[numAtoms];
        for ( int i = 0; i < numAtoms; i++ ){
            tok.nextToken(); x = tok.nval;
            tok.nextToken(); y = tok.nval;
            tok.nextToken(); z = tok.nval;
            tok.nextToken(); atomName = tok.sval;
            skipLine( tok );
            
            Atom a = AtomTable.getActive().getAtom( atomName );
            a.setLocation( new Vector3d( x, y, z ) );
            a.setOwner( this );
            a.setID( i );
            
            atoms[i] = a;
        }
        
        for  (int i = 0; i < numBonds; i++ ){
            tok.nextToken(); int start = (int)tok.nval;
            tok.nextToken(); int end = (int)tok.nval;
            //  bond entries are 1-based
            start--;
            end--;
            
            skipLine( tok );
            Bond b = new Bond( atoms[start], atoms[end], Bond.BOND_TYPE_NORMAL );
            bonds.add( b );
        }
        
        for ( int i = 0; i < numAtoms; i++ ){
            this.atoms.add( atoms[i] );
        }
    }
    
    /** Save the current contents of this resource to
     * the filename specified.  */
    public void saveToFile(String fileName) throws java.io.IOException {
    }
    
}
