package molmaster.model.loaders;

import molmaster.*;
import molmaster.model.*;

import java.io.*;
import java.util.*;

import javax.vecmath.*;
import javax.swing.*;

/** Loads in files for the XYZ format.  This description of this
 *  format is trivial.  Look in the examples/ directory for some
 *  examples.
 *
 * @author  Russell
 */
public class XYZLoader extends MoleculeFile implements MovieFile {

	protected List frames = new ArrayList();

	public XYZLoader() {
	}

	/** The XYZ file format is simple:
	 *
	 *  Frame count ( number of atoms in this frame )
	 *  [Atom Name] x y z
	 *  ...
	 *  ...
	 *  Next Frame
	 */
	public void loadFromFile(String fileName) throws java.io.IOException {
		diskFile = new File(fileName);

		Reader r =
			new BufferedReader(
				new InputStreamReader(new FileInputStream(new File(fileName))));

		StreamTokenizer tok = new StreamTokenizer(r);
		String atomName;
		double x = 0;
		double y = 0;
		double z = 0;

		int tokType = 0;
		while ((tokType = tok.nextToken()) != StreamTokenizer.TT_EOF) {
			List frameAtoms = new ArrayList();

			int numAtoms = (int) tok.nval;
			//  blank line here.... comment?
			skipLine(tok, 2);

			molmaster.gui.MainFrame.setStatusMessage(
				"Loading frame #" + (frames.size() + 1));

			for (int i = 0; i < numAtoms; i++) {
				//Log.writeLog("iteration" + i);
				tok.nextToken();
				atomName = tok.sval;
				x = getFixed(r);
				y = getFixed(r);
				z = getFixed(r);
				//tok.nextToken(); x = tok.nval;
				//tok.nextToken(); y = tok.nval;
				//tok.nextToken(); z = tok.nval;

				Atom a = AtomTable.getActive().getAtom(atomName);
				a.setLocation(new Vector3d(x, y, z));
				a.setOwner(this);
				a.setID(i);

				frameAtoms.add(a);
				//skipLine( tok );
				if ((i % 20) == 0) {
					molmaster.gui.MainFrame.setProgressMeter(
						100 * i / numAtoms);
				}
			}
			frames.add(frameAtoms);
			//skipLine( tok );
		}

		List l = (List) frames.get(0);
		Iterator i = l.iterator();
		while (i.hasNext()) {
			atoms.add(i.next());
		}
		Log.writeLog("" + frames.size());
	}

	protected void skipLine(StreamTokenizer tok, int numLines)
		throws java.io.IOException {
		tok.eolIsSignificant(true);

		for (int i = 0; i < numLines; i++) {
			while (tok.ttype != StreamTokenizer.TT_EOF
				&& tok.ttype != StreamTokenizer.TT_EOL) {
				tok.nextToken();
			}
			if (i != numLines - 1) {
				tok.nextToken();
			}
		}
		tok.eolIsSignificant(false);
	}
	/** Save the current contents of this resource to
	 * the filename specified.  */
	public void saveToFile(String fileName) throws java.io.IOException {
		Collection c = MolModel.getModel().getBonds();
		if (c.size() != 0) {
			JOptionPane.showMessageDialog(
				molmaster.gui.MainFrame.getActiveWindow(),
				"This file format cannot save bonds, but the current scene contains them.\n\nNo bonds will be stored in the specified file.",
				"Save Warning",
				JOptionPane.WARNING_MESSAGE);
		}

		c = MolModel.getModel().getAtoms();
		Iterator i = c.iterator();
		BufferedWriter w =
			new BufferedWriter(new FileWriter(new File(fileName)));
		w.write(c.size() + "\n");
		w.write("//Generated by MolMaster\n");
		while (i.hasNext()) {
			Atom a = (Atom) i.next();
			AtomAttributes attr = (AtomAttributes) a.getAttributes();
			w.write(attr.atomName + "\t");
			Vector3d loc = a.getLocation();
			w.write(loc.x + "\t" + loc.y + "\t" + loc.z + "\t\n");
		}
		w.close();
	}

	public java.util.List getAtomFrame(int frame) {
		return (List) frames.get(frame);
	}

	public java.util.List getBondFrame(int frame) {
		return new ArrayList();
	}

	public int getFrameCount() {
		return frames.size();
	}

}
