package molmaster.util;

import java.io.*;
import java.util.*;
import javax.swing.*;
import java.awt.event.*;


/** Provides an interface to add and remove
 *  files from the given file list.
 *
 * @author  Russell Power
 */
public class FileEditPanel extends javax.swing.JPanel
implements java.beans.PropertyChangeListener  {
    
    protected FileList fileList;
    protected Set files;
    protected File currentFile;
    protected File[] currentFiles;
    protected JDialog owner;
    
    /** Creates a new panel using the information contained
     *  in the given FileList.
     */
    public FileEditPanel(FileList fl) {
        fileList = fl;
        
        initComponents();
        
        files = new HashSet();
        Iterator i = fileList.getFiles().iterator();
        while ( i.hasNext() ) {
            files.add( i.next() );
        }
        loadModel();
        fileSelect.addPropertyChangeListener( this );
        fileSelect.setCurrentDirectory( new File( "." ) );
        addComponentListener( new ComponentAdapter() {
            public void componentShown(java.awt.event.ComponentEvent componentEvent) {
                files = new HashSet();
                Iterator i = fileList.getFiles().iterator();
                while ( i.hasNext() ) {
                    files.add( i.next() );
                }
                loadModel();
            }
        } );
    }
    
    /** Creates a new instance of the editor panel using the given list with the
     * given dialog as a parent.
     */
    public FileEditPanel(FileList fl, JDialog jc) {
        this( fl );
        owner = jc;
    }
    
    
    private void loadModel() {
        DefaultListModel m = new DefaultListModel();
        Iterator i = files.iterator();
        while ( i.hasNext() ) {
            m.addElement( i.next() );
        }
        
        lstFile.setModel( m );
    }
    
    private void initComponents() {//GEN-BEGIN:initComponents
        fileSplitter = new javax.swing.JSplitPane();
        fileSelect = new javax.swing.JFileChooser();
        cmdSplitter = new javax.swing.JSplitPane();
        lstScroll = new javax.swing.JScrollPane();
        lstFile = new javax.swing.JList();
        cmdPanel = new javax.swing.JPanel();
        btnRemove = new javax.swing.JButton();
        btnAdd = new javax.swing.JButton();
        btnOkay = new javax.swing.JButton();
        btnCancel = new javax.swing.JButton();

        setLayout(new java.awt.BorderLayout());

        setBackground(new java.awt.Color(0, 102, 153));
        setOpaque(false);
        fileSelect.setControlButtonsAreShown(false);
        fileSelect.setMultiSelectionEnabled(true);
        fileSplitter.setLeftComponent(fileSelect);

        cmdSplitter.setOrientation(javax.swing.JSplitPane.VERTICAL_SPLIT);
        lstScroll.setPreferredSize(new java.awt.Dimension(260, 200));
        lstScroll.setViewportView(lstFile);

        cmdSplitter.setLeftComponent(lstScroll);

        cmdPanel.setLayout(new javax.swing.BoxLayout(cmdPanel, javax.swing.BoxLayout.Y_AXIS));

        btnRemove.setMnemonic('r');
        btnRemove.setText("<< Remove");
        btnRemove.setToolTipText("Removes the currently selected file(s) from the list");
        btnRemove.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnRemoveActionPerformed(evt);
            }
        });

        cmdPanel.add(btnRemove);

        btnAdd.setMnemonic('a');
        btnAdd.setText("Add >>");
        btnAdd.setToolTipText("Adds the currently selected file(s) to the list");
        btnAdd.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnAddActionPerformed(evt);
            }
        });

        cmdPanel.add(btnAdd);

        btnOkay.setText("Okay");
        btnOkay.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnOkayActionPerformed(evt);
            }
        });

        cmdPanel.add(btnOkay);

        btnCancel.setText("Cancel");
        btnCancel.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnCancelActionPerformed(evt);
            }
        });

        cmdPanel.add(btnCancel);

        cmdSplitter.setRightComponent(cmdPanel);

        fileSplitter.setRightComponent(cmdSplitter);

        add(fileSplitter, java.awt.BorderLayout.NORTH);

    }//GEN-END:initComponents
    
    private void btnOkayActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnOkayActionPerformed
        fileList.clear();
        Iterator i = files.iterator();
        System.out.println("Files size:" + files.size() );
        while ( i.hasNext() ) {
            fileList.addFile( (File)i.next() );
        }
        quitDialog();
    }//GEN-LAST:event_btnOkayActionPerformed
    
    private void btnCancelActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnCancelActionPerformed
        quitDialog();
    }//GEN-LAST:event_btnCancelActionPerformed
    
    private void btnRemoveActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnRemoveActionPerformed
        Object[] o = lstFile.getSelectedValues();
        for ( int i = 0; i < o.length; i++ ) {
            files.remove( o[i] );
        }
        loadModel();
    }//GEN-LAST:event_btnRemoveActionPerformed
    
    private void btnAddActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnAddActionPerformed
        if ( currentFiles == null ) { return; }
        System.out.println(currentFiles.length);
        for ( int i = 0; i < currentFiles.length; i++ ) {
            files.add( currentFiles[i] );
        }
        loadModel();
    }//GEN-LAST:event_btnAddActionPerformed
    
    private void quitDialog() {
        owner.setVisible( false );
        owner.dispose();
    }
    
    /** Fired when a change has occurred to the JFileChooser.  Allows us to find
     * what file(s) have been selected.
     */
    public void propertyChange(java.beans.PropertyChangeEvent e) {
        String prop = e.getPropertyName();
        if (prop.equals(JFileChooser.SELECTED_FILE_CHANGED_PROPERTY)) {
            currentFile = (File) e.getNewValue();
        }
        currentFiles = fileSelect.getSelectedFiles();
    }
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton btnAdd;
    private javax.swing.JFileChooser fileSelect;
    private javax.swing.JPanel cmdPanel;
    private javax.swing.JList lstFile;
    private javax.swing.JSplitPane fileSplitter;
    private javax.swing.JSplitPane cmdSplitter;
    private javax.swing.JButton btnRemove;
    private javax.swing.JScrollPane lstScroll;
    private javax.swing.JButton btnOkay;
    private javax.swing.JButton btnCancel;
    // End of variables declaration//GEN-END:variables
    
}
