package molmaster.util;

import java.awt.event.*;

import java.io.*;
import java.util.*;
import javax.swing.*;

/** Displays a dialog with a listing of the given
 *  FileList's files.  Also contains command buttons
 *  to edit files.
 *
 * @author  Russell Power
 * @see molmaster.util.FileList
 */
public class FileListPanel extends javax.swing.JDialog implements ActionListener {
    protected FileList files;
    
    /** Creates new form FileListPanel */
    public FileListPanel(FileList files, java.awt.Dialog parent ) {
        super(parent, false);
        init(files);
    }
    
    /** Creates a new listing panel using the given list and Frame as parents.
     */    
    public FileListPanel(FileList files, java.awt.Frame parent ) {
        super(parent, false);
        init(files);
    }
    
    private void init(FileList files) {
        this.files = files;
        initComponents();
        files.addListener( this );
        loadList();
        //addComponentListener( this );
        setTitle( "Analysis file listing" );
        
		java.awt.Dimension d = getToolkit().getScreenSize();
		setLocation( d.width - getWidth() - 10, 0);
    }
    
    protected void loadList() {
        Iterator i = files.getFiles().iterator();
        DefaultListModel lm = new DefaultListModel();
        while ( i.hasNext() ) {
            File f = (File)i.next();
            lm.addElement( f  );
        }
        fileList.setModel( lm );
    }
    private void initComponents() {//GEN-BEGIN:initComponents
        splitter = new javax.swing.JSplitPane();
        cmdPanel = new javax.swing.JPanel();
        btnEditFiles = new javax.swing.JButton();
        btnHelp = new javax.swing.JButton();
        btnRemove = new javax.swing.JButton();
        fileScroll = new javax.swing.JScrollPane();
        fileList = new javax.swing.JList();

        addWindowListener(new java.awt.event.WindowAdapter() {
            public void windowClosing(java.awt.event.WindowEvent evt) {
                closeDialog(evt);
            }
        });

        splitter.setOrientation(javax.swing.JSplitPane.VERTICAL_SPLIT);
        cmdPanel.setLayout(new javax.swing.BoxLayout(cmdPanel, javax.swing.BoxLayout.Y_AXIS));

        btnEditFiles.setText("Edit file list");
        btnEditFiles.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnEditFilesActionPerformed(evt);
            }
        });

        cmdPanel.add(btnEditFiles);

        btnHelp.setText("Help");
        btnHelp.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnHelpActionPerformed(evt);
            }
        });

        cmdPanel.add(btnHelp);

        btnRemove.setMnemonic('r');
        btnRemove.setText("Remove");
        btnRemove.setToolTipText("Remove the selected file from the list");
        btnRemove.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnRemoveActionPerformed(evt);
            }
        });

        cmdPanel.add(btnRemove);

        splitter.setLeftComponent(cmdPanel);

        fileScroll.setViewportView(fileList);

        splitter.setRightComponent(fileScroll);

        getContentPane().add(splitter, java.awt.BorderLayout.CENTER);

        pack();
        java.awt.Dimension screenSize = java.awt.Toolkit.getDefaultToolkit().getScreenSize();
        setSize(new java.awt.Dimension(173, 300));
        setLocation((screenSize.width-173)/2,(screenSize.height-300)/2);
    }//GEN-END:initComponents
    
    private void btnRemoveActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnRemoveActionPerformed
        Object[] f = fileList.getSelectedValues();
        for ( int i = 0; i < f.length; i++ ) {
            files.removeFile( (File)f[i] );
        }
        loadList();
        fireActionEvent();
    }//GEN-LAST:event_btnRemoveActionPerformed
    
    private void btnHelpActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnHelpActionPerformed
        GetText.showHelpDialog( this, "FileListPanel.help" );
    }//GEN-LAST:event_btnHelpActionPerformed
    
    private void btnEditFilesActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnEditFilesActionPerformed
        files.showAddDialog( this );
        loadList();
        fireActionEvent();
    }//GEN-LAST:event_btnEditFilesActionPerformed
    
    private void fireActionEvent() {
        files.fireActionEvent();
    }
    
    /** Closes the dialog */
    private void closeDialog(java.awt.event.WindowEvent evt) {//GEN-FIRST:event_closeDialog
        setVisible(false);
        dispose();
    }//GEN-LAST:event_closeDialog

    /** Called by the FileList when a change has occured.
     */    
    public void actionPerformed(java.awt.event.ActionEvent actionEvent) {
        loadList();
    }
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton btnHelp;
    private javax.swing.JList fileList;
    private javax.swing.JPanel cmdPanel;
    private javax.swing.JScrollPane fileScroll;
    private javax.swing.JButton btnEditFiles;
    private javax.swing.JSplitPane splitter;
    private javax.swing.JButton btnRemove;
    // End of variables declaration//GEN-END:variables
    
}
