package molmaster.util;

/** Displays a dialog informing the user an update
 *  is available for molmaster.
 *
 * @author  Russell Power
 */
public class UpdateDialog extends javax.swing.JDialog {
    
    /** A return status code - returned if Cancel button has been pressed */
    public static final int RET_CANCEL = 0;
    /** A return status code - returned if OK button has been pressed */
    public static final int RET_OK = 1;
    
    private static String text;
    
    /** Creates new form UpdateDialog */
    public UpdateDialog(String current, String newbuild, String changeLog ) {
        super(molmaster.gui.MainFrame.getActiveWindow(), true );
        initComponents();
        text = "A new version of molmaster has been detected.\n"+
        "The current build number is " + current + ", and the new build is " + newbuild + 
        ".\n\nWould you like to update molmaster?\n\n" + 
        "If you choose no, you will be reminded in a week.";

        updateText.setText( text );
        updateText.setEditable( false );
        txtChangelog.setText( "Changelog:\n\n" + changeLog );
        txtChangelog.setEditable( false );
    }
    
    /** @return the return status of this dialog - one of RET_OK or RET_CANCEL */
    public int getReturnStatus() {
        return returnStatus;
    }
    
    private void initComponents() {//GEN-BEGIN:initComponents
        buttonPanel = new javax.swing.JPanel();
        okButton = new javax.swing.JButton();
        cancelButton = new javax.swing.JButton();
        txtPanel = new javax.swing.JPanel();
        updateText = new javax.swing.JTextArea();
        scrollChange = new javax.swing.JScrollPane();
        txtChangelog = new javax.swing.JTextArea();

        setTitle("Molmaster Update");
        setResizable(false);
        addWindowListener(new java.awt.event.WindowAdapter() {
            public void windowClosing(java.awt.event.WindowEvent evt) {
                closeDialog(evt);
            }
        });

        buttonPanel.setLayout(new java.awt.FlowLayout(java.awt.FlowLayout.RIGHT));

        buttonPanel.setBackground(new java.awt.Color(153, 153, 255));
        okButton.setText("Yes");
        okButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                okButtonActionPerformed(evt);
            }
        });

        buttonPanel.add(okButton);

        cancelButton.setText("No");
        cancelButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                cancelButtonActionPerformed(evt);
            }
        });

        buttonPanel.add(cancelButton);

        getContentPane().add(buttonPanel, java.awt.BorderLayout.SOUTH);

        txtPanel.setLayout(null);

        txtPanel.setBackground(new java.awt.Color(255, 255, 255));
        updateText.setEditable(false);
        txtPanel.add(updateText);
        updateText.setBounds(20, 0, 380, 70);

        txtChangelog.setLineWrap(true);
        txtChangelog.setWrapStyleWord(true);
        scrollChange.setViewportView(txtChangelog);

        txtPanel.add(scrollChange);
        scrollChange.setBounds(20, 74, 380, 180);

        getContentPane().add(txtPanel, java.awt.BorderLayout.CENTER);

        pack();
        java.awt.Dimension screenSize = java.awt.Toolkit.getDefaultToolkit().getScreenSize();
        setSize(new java.awt.Dimension(421, 327));
        setLocation((screenSize.width-421)/2,(screenSize.height-327)/2);
    }//GEN-END:initComponents
    
    private void okButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_okButtonActionPerformed
        doClose(RET_OK);
    }//GEN-LAST:event_okButtonActionPerformed
    
    private void cancelButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_cancelButtonActionPerformed
        doClose(RET_CANCEL);
    }//GEN-LAST:event_cancelButtonActionPerformed
    
    /** Closes the dialog */
    private void closeDialog(java.awt.event.WindowEvent evt) {//GEN-FIRST:event_closeDialog
        doClose(RET_CANCEL);
    }//GEN-LAST:event_closeDialog
    
    private void doClose(int retStatus) {
        returnStatus = retStatus;
        setVisible(false);
        dispose();
    }
    
    /**
     * @param args the command line arguments
     */
    public static void main(String args[]) {
        new UpdateDialog("1.001", "1.002", "Testing differences..." ).show();
    }
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JPanel txtPanel;
    private javax.swing.JPanel buttonPanel;
    private javax.swing.JButton okButton;
    private javax.swing.JScrollPane scrollChange;
    private javax.swing.JTextArea updateText;
    private javax.swing.JButton cancelButton;
    private javax.swing.JTextArea txtChangelog;
    // End of variables declaration//GEN-END:variables
    
    private int returnStatus = RET_CANCEL;
}
